/*  

    Copyright (c) 2001 - 2004. All Rights Reserved.
    :  ..
*/

#pragma once
#include "DB/Room.h"
#include "DB/Char/CharExtData.h"
#include "DB/Subject/Subject.h"
#include "DB/Subject/ObjVirtual.h"

_KERNEL void SetZoom( ScanID id, int tNum, double tVal );
_KERNEL void GetZoom( ScanID id, int tNum, double *tVal );

class CExit;
class CRoom;

//    
class CCharAddon
{

public: //

    // ID 
    _KERNEL CCharAddon( int zoneID ) :
        m_zoneID( zoneID ), m_save( false )
    {}

    _KERNEL virtual ~CCharAddon() {}

public: // /

    _KERNEL virtual void Serialize( BYTE** data, bool store, int version ) {}
    _KERNEL virtual void Serialize( CFileStore& fs ) {}
    //_KERNEL virtual void Serialize( CByteStore& fs ) {}

    //  
    _KERNEL virtual void FreeData( BYTE* data ) {}

public: // 

    int m_zoneID;          // ID 
    CZoneInstance* m_inst; // 
    bool m_save;           //    

};

//  
class CGroupMember
{

public: // 

    _KERNEL CGroupMember() {}
    _KERNEL CGroupMember( CChar* c, bool group = false ) : m_char( c ), m_group( group ) {}
    ~CGroupMember() {}

public: // 

    _KERNEL bool operator==( const CGroupMember& gm ) { return m_char == gm.m_char; }

public: // 

    // 
    CChar* m_char;
    // 
    bool m_group;

};

//   
class SkillLagData
{

public: //   

    SkillLagData() {}
    SkillLagData( int id, uint lagPulse, byte type = 0 ) :
        m_id( id ), m_lagPulse( lagPulse ), m_type( type )
    {}

public: // 

    //  
    int  m_id;
    //  
    byte m_type;
    //   
    uint m_lagPulse;

};

//   
class SpellLagData
{

public: //   

    SpellLagData() {}
    SpellLagData( int id, uint lagPulse ) : m_id( id ), m_lagPulse( lagPulse ) {}

public: // 

    //  
    int  m_id;
    //   
    uint m_lagPulse;

};

//  
class CAbilValue
{

public: // 

    CAbilValue( int charId = 0, byte type = BT_WEAPON, short abilId = 0, short value = 0 ) :
        m_charId( charId ), m_type( type ), m_abilId( abilId ), m_value( value )
    {}

public: // 

    _KERNEL void Serialize( CFileStore& fs );

public:    // 

    int m_charId;   //  
    byte m_type;    //  
    short m_abilId; // 
    short m_value;  // 

};

//  
class CSeal
{

public: // 

    CSeal* m_next; //   

    CChar* m_char; //  
    uint   m_life; //   
    CSeal* m_pair; // 
    
    bool   m_owner; //  

};

//    
class CCharStats
{

public: //   

    // 
    _KERNEL CCharStats();

    // 
    ~CCharStats()
    {
        ClearAffects();
    }

public: // 

    //  
    _KERNEL void AddCopyAffect( CAffectUnit* affUnit );

    //  
    _KERNEL void ClearAffects();

    //  
    _KERNEL void setCurHP( double hp, bool limits = true );

    //  
    _KERNEL void changeCurHP( double hp );

public: // 

    byte m_type;              //  

    // 
    double m_maxHP;           //  
    double m_curHP;           //   (int)
    double m_dCurHP;          //   (double)
    double m_regHP;           //  
    double m_regHPModifier;   //   

    // 
    double m_maxMana;         //  
    double m_curMana;         //   (int)
    double m_dCurMana;        //   (double)
    double m_regMana;         //  
    double m_regManaModifier; //   

    // 
    double m_maxMV;           //  
    double m_curMV;           //   (int)
    double m_dCurMV;          //   (double)
    double m_regMV;           //  
    double m_regMVModifier;   //   

    // 
    double m_str;  // 
    double m_int;  // 
    double m_wiz;  // 
    double m_dex;  // 
    double m_con;  // 
    double m_size; // 

    double m_hitroll; // 

    double m_damroll; // 

    double m_resist[RESIST_COUNT];   // 

    PArray<CAffectUnit*> m_affects;  // 

    byte m_affBits[AFF_MAX_COUNT/8]; //  

    double* m_weapon;
    double* m_skill;
    double* m_spell;
};

//     
class CCorpseObj
{

public: // 

    CCorpseObj( CorpseObjType type, int value ) :
        m_type( type ), m_value( value )
    {}

    CCorpseObj( CorpseObjType type, CSubject* obj ) :
        m_type( type ), m_obj( obj )
    {}

    ~CCorpseObj() {}

public: // 

    byte m_type;     //  
    int m_value;     // 
    CSubject* m_obj; // 

};

//   
class COblivionNode
{

public: // 

    COblivionNode( AbilityType type, int id ) :
        m_type( type ), m_id( id )
    {}

public: // 

    byte m_type; // 
    int m_id;    // 

};

//   
class CCharAffectInfo
{

public: // 

    _KERNEL CCharAffectInfo() :
        m_turnProt( NULL ), m_turnPower( 0 ), m_drainLevel( 0 ), m_cryChar( NULL ),
        m_oblivion( NULL ), m_anger( NULL ), m_holyCaster( NULL ), m_aims( NULL )
    {}

public: // 

    _KERNEL ~CCharAffectInfo();

public: // 

    CMobPrototype* m_turnProt; //  ,    
    int m_turnPower;           //     (0 - )

    byte m_drainLevel;         //   

    CChar* m_cryChar;          //   

    PArray<COblivionNode*>* m_oblivion; // 

    CChar* m_anger;            //  
    CChar* m_holyCaster;       //   

    PArray<CChar*>* m_aims;    // 

};

//      
class CChar
{

public: //   

    _KERNEL CChar();

    _KERNEL virtual ~CChar();

public: // -

    //  
    _KERNEL virtual bool isMob() = 0;

    //  
    _KERNEL bool isPlayer() { return !isMob(); }

    // 
    _KERNEL virtual bool isImmortal() { return false; }

    //   
    _KERNEL virtual bool isPlayerOrCharm() = 0;

    //  
    _KERNEL virtual bool isHotelOwner() { return false; }

    // 
    _KERNEL virtual bool isBanker() { return false; }

    // 
    _KERNEL virtual bool isStorer() { return false; }

    // 
    _KERNEL virtual bool isSeller() { return false; }

    //  
    _KERNEL virtual bool isRepairman() { return false; }

    //   
    _KERNEL virtual bool isMobForcing() { return false; }

    //   
    _KERNEL virtual bool isPlayerForcing() { return false; }

    // 
    _KERNEL virtual bool isPeaceful() { return false; }

    //   
    _KERNEL bool isFarProf() { return prof()->m_farFight; }

    //  
    _KERNEL virtual bool noCorpse() { return false; }

    //   
    _KERNEL virtual bool isFriend( CChar* c ) = 0;

    //   (NULL  )
    _KERNEL virtual CBaseMobPrototype* prot() = 0;

    //   
    _KERNEL bool inArena();

    //   6 
    _KERNEL const char* name( int i = 0, bool global = false );

    //   6  ( )
    _KERNEL virtual const char* realName( int i = 0 ) = 0;

    //   6    
    _KERNEL const char* nameI( CChar* c, int i = 0, bool global = false );

    //  ""  6    
    _KERNEL const char* meI( CChar* c, int i = 0 );

    // 
    _KERNEL const char* mask( bool global = false );

    //  ( )
    _KERNEL virtual const char* realMask() = 0;

    // 
    _KERNEL const char* text( bool global = true );

    //  ( )
    _KERNEL virtual const char* realText() = 0;

    // 
    _KERNEL const char* desc( bool global = false );

    //  ( )
    _KERNEL virtual const char* realDesc() = 0;

    // 
    _KERNEL Gender gender( bool global = false );

    //  ( )
    _KERNEL virtual Gender realGender() = 0;

    //    
    _KERNEL Gender genderI( CChar* c );

    //   (  )
    _KERNEL virtual int realLevel() = 0;

    // 
    _KERNEL int level();

    //  
    _KERNEL virtual int immInvis() { return 0; }

    //  
    _KERNEL bool hide( PkFlag pkFlag = APF_NOPK ) { return !m_fight && !holded() && !stoned() && hasAffect( AFF_HIDE, pkFlag ); }

    // 
    _KERNEL virtual CProfession* prof() = 0;

    // 
    _KERNEL virtual CRace* race() = 0;

    //  
    _KERNEL virtual CCharPlayer* mentalOwner() { return NULL; }

    // 
    _KERNEL virtual CCharMob* mentalChild() { return NULL; }

    //  ,      (    )
    _KERNEL virtual CCharPlayer* playerContext() = 0;

    //   -
    _KERNEL virtual bool isPK() = 0;

    //  
    _KERNEL int getTurnPower( PkFlag pkFlag ) { return pkFlag == APF_NOPK ? getTurnPowerNopk() : getTurnPowerPk(); }

    // 
    _KERNEL virtual int poltroon( PkFlag pkFlag = APF_NOPK ) = 0;

    //  
    _KERNEL bool CheckPoltroon();

    //   
    _KERNEL virtual bool takeObjects() = 0;

    //     
    _KERNEL int carryCount() { return ListCount( m_objList ); }

    //   ,    
    _KERNEL int allCarryCount() { return (int)m_stats[APF_NOPK]->m_str + level(); }

    //    
    _KERNEL int carryWeight();

    //  ,    
    _KERNEL int allCarryWeight();

    //   
    _KERNEL bool inventoryFull();

    //     
    _KERNEL bool visible( CChar* c );

    //      (   ) 
    _KERNEL virtual bool visibleMe( CChar* c ) { return true; }

    //      '' 
    _KERNEL virtual bool visibleWho( CCharPlayer* player ) = 0;

    //     
    _KERNEL bool visible( CSubject* obj );

    //    
    _KERNEL bool visible( CRoom* r );

    //   
    _KERNEL bool seeExits();

    //     
    _KERNEL bool sleep() { return m_pos <= POS_SLEEPING; }

    //   
    _KERNEL virtual bool dropLink() { return false; }

    //    
    _KERNEL virtual bool inGame() = 0;

    //   
    _KERNEL virtual bool away() { return false; }

    //   
    _KERNEL virtual const char* awayStr( bool color = true ) { return ""; }

    //  
    _KERNEL virtual int resistMax( int type ) = 0;

    //    
    _KERNEL bool hasWeaponSkill( int id );

    //    ( ) 
    _KERNEL virtual bool realHasWeaponSkill( int id ) = 0;

    //    
    _KERNEL bool hasSkill( int id );

    //    ( ) 
    _KERNEL virtual bool realHasSkill( int id ) = 0;

    //   
    _KERNEL bool hasSpell( int id );

    //   ( ) 
    _KERNEL virtual bool realHasSpell( int id ) = 0;

    //     
    _KERNEL bool hasAnyWeaponSkill();

    //     ( ) 
    _KERNEL virtual bool realHasAnyWeaponSkill() = 0;

    //    
    _KERNEL bool hasAnySkill();

    //     ( )
    _KERNEL virtual bool realHasAnySkill() = 0;

    //   
    _KERNEL bool hasAnySpell();

    //    ( )
    _KERNEL virtual bool realHasAnySpell() = 0;

    //  
    _KERNEL virtual int weaponSkill( int id, PkFlag pkFlag = APF_NOPK ) = 0;

    // 
    _KERNEL virtual int skill( int id, PkFlag pkFlag = APF_NOPK ) = 0;

    //  
    _KERNEL virtual int skillGrade( int id, PkFlag pkFlag = APF_NOPK ) = 0;

    // 
    _KERNEL virtual int spell( int id, PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual int skillHitroll( int i, PkFlag pkFlag = APF_NOPK ) = 0;

    //  
    _KERNEL bool checkOblivion( AbilityType type, int id );

    //  
    _KERNEL virtual int specRes( int sr ) { return 0; }

    //   
    _KERNEL virtual double specResK( int sr ) { return 1.0; }

    //   
    _KERNEL virtual double specResistsK( int sr1, int sr2 = SR_NONE, int sr3 = SR_NONE ) { return 1.0; }

    //   
    _KERNEL virtual CDamageType* handDamType() = 0;

    //   
    _KERNEL virtual CDamageStr* handDamStr() = 0;

    //  
    _KERNEL CWalkMethod* walkMethod();

    //   ( )
    _KERNEL virtual CWalkMethod* realWalkMethod() = 0;

    //   - /
    _KERNEL virtual bool attackIsNear() = 0;

    //   - /
    _KERNEL virtual bool weaponIsNear() = 0;

    //   
    _KERNEL virtual int primeHitNumb( bool& shot, PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL int apathyCheck( int count, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL virtual int offHitNumb( PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual int staffHitNumb( PkFlag pkFlag = APF_NOPK ) = 0;

    //     
    _KERNEL int warCryHitNumb( PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL virtual int stunPerc( PkFlag pkFlag = APF_NOPK ) = 0;

    //      
    _KERNEL virtual bool CheckHeavyHand( PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL int ThrowHitrollDice( int hitroll, CChar* target, PkFlag pkFlag );

    //    
    _KERNEL virtual bool ThrowBlockDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual bool ThrowParryDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual bool ThrowEvadeDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual int ThrowDamageDice( CWeaponSkill* wSkill, PkFlag pkFlag = APF_NOPK ) = 0;

    //    ,    
    _KERNEL int ThrowDamageSlotDice();

    //   
    _KERNEL bool criticalFrag( CChar *target, PkFlag pkFlag = APF_NOPK );

    //      
    _KERNEL int ThrowSpellDice( CBaseFragSpell* spell, int power, PkFlag pkFlag );

    //     
    _KERNEL int ThrowHealDice( CBaseHealSpell* spell, int power, CChar* target, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL int ThrowFreshDice( int power, CChar* target );

    //     
    _KERNEL int ThrowFreshManaDice( int power, CChar* target );

    //    
    _KERNEL int ThrowSaturationDice( int power );

    //    
    _KERNEL int ThrowBandagingDice( int power, CChar* target, PkFlag pkFlag );

    //  
    _KERNEL void Loot( CSubject* corpse );

    //     
    _KERNEL bool IsNear( CChar* c );

    //     
    _KERNEL bool inGroup( const CChar* c );

    //     
    _KERNEL bool inClan( CChar* c );

    //    ,   
    _KERNEL bool inWars( CCharPlayer* c );

    //    ,    
    _KERNEL bool inAllies( CCharPlayer* c );

    //  -  
    _KERNEL bool isGroupLeader();

    //   
    _KERNEL bool beFollowed();

    //  
    //_KERNEL virtual bool isPersonalOwner( int id ) { return false; }

    //   
    template <typename T>
    _KERNEL void EnumObjects( T&& func );

    //     
    _KERNEL bool hasObject( CObjPrototype* objProt, CZoneInstance* zoneInst = NULL, bool key = false );

    //     
    _KERNEL bool hasObject( int id, CZoneInstance* zoneInst = NULL );

    //     
    _KERNEL bool hasObject( CSubject* obj );

    //    
    _KERNEL void FindObject( int objId, bool recursive, CSubject*& obj, CSubject*& cont, int& slotId, CZoneInstance* zoneInst = NULL );

    //   
    _KERNEL bool hasEquipment();

    //  
    _KERNEL virtual bool canMove();

    //   
    _KERNEL virtual bool canFear();

    //   
    _KERNEL bool canBackstab();

    //   
    _KERNEL bool canThrowAttack( bool msg = true );

    //    
    _KERNEL int moveCost( CRoom* room );

    //      
    _KERNEL virtual bool letToMove( RoomExit exit ) { return true; }

    //  
    _KERNEL virtual bool canChat() { return true; }

    // 
    _KERNEL virtual uint muted() { return 0; }

    //    
    _KERNEL virtual bool noCry() { return false; }

    // 
    _KERNEL virtual bool noLegs() { return false; }

    //       ( )
    _KERNEL bool canNearFight( CChar* c, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL virtual int handlerFlags() { return 0; }

    // - 
    _KERNEL virtual int handler( CMobTriggerData* data ) { return 0; }

    //  
    _KERNEL virtual bool CanTrigger( int flag, void* param = NULL, BOOL bSkipExec = FALSE ) { return false; }

    //  
    _KERNEL virtual int ProcessTrigger( CMobTriggerData* data ) { return 0; }

    //    
    _KERNEL const char* ConditionStr( PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL const char* HealthStr( PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL virtual double expD() { return 10; }

    //    
    _KERNEL virtual double dsuExpD() { return 0; }

    //   
    _KERNEL virtual uint64 money() = 0;

    //   
    _KERNEL virtual void setMoney( uint64 m ) = 0;

    //  
    _KERNEL void moneyInc( uint64 m );

    //  
    _KERNEL void moneyDec( uint64 m );

    // 
    Position pos() { return (Position)m_pos; }
    _KERNEL void setPos( Position p, bool myCmd = false );

    //  -  
    _KERNEL virtual CObjPrototype* corpseProt() = 0;

    //  -  
    _KERNEL virtual CObjPrototype* statueProt() = 0;

    //   
    _KERNEL virtual void Send( const char* text ) = 0;

    //    
    _KERNEL virtual void SendWithPages( char* text, bool autoDelete, bool color = true ) = 0;

    //    ( )
    _KERNEL virtual void SendFormat( const char* text, ... ) = 0;

    //     
    _KERNEL virtual void SendFormatDmg( int damage, const char* text, ... ) = 0;

    //    ( ) (  )
    _KERNEL virtual void SendFormatU( const char* text, ... ) = 0;

    //    ( ) (  )
    _KERNEL virtual void SendFormatUDmg( int damage, const char* text, ... ) = 0;

    //  ,    
    _KERNEL virtual void SendCharView( CChar* c ) = 0;

    //    
    _KERNEL virtual void SendStatus() = 0;

    //    
    _KERNEL virtual void SendMissStr( int hitType, HitCheck check, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, CWeaponSubject* weapon = NULL ) = 0;

    //    
    _KERNEL virtual void SendHitStr( int hitType, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, int damage, int causedDamage, bool critical, CWeaponSubject* weapon = NULL ) = 0;

    //   
    _KERNEL virtual void SendFightMsg( byte msgType, CChar* hiter, CChar* target, int nCausedDamage = 0, CDamageStr* damStr = NULL ) = 0;

    //   
    _KERNEL bool trustHim( CChar* c );

    // 
    _KERNEL void Equip( CSubject* obj, int slot );

    //  
    _KERNEL CSubject* Unequip( int slot );

    //    
    _KERNEL void PutIntoInv( CSubject* obj, bool hideAgro = false );

    //       
    _KERNEL CSubject* PutIntoInv( int objID );

    //    
    _KERNEL void GetFromInv( CSubject* obj );

    //    
    _KERNEL void GetFromChar( CSubject* obj );

    //   
    _KERNEL bool CheckCarry( CSubject* obj );

    //  
    _KERNEL void TakeObject( CSubject* obj );

    //    
    _KERNEL void TakeFromContainer( CSubject* cont, bool fromInv, char* purpose, bool myCont = true );

    //    
    _KERNEL void TakeObject( CSubject* obj, CSubject* cont, bool fromInv, bool myCont );

    //  
    _KERNEL void ThrowObject( CSubject* obj );

    //  
    _KERNEL void ThrowMoney( int money );

    //  
    _KERNEL void GiveObject( CSubject* obj, CChar* c );

    //  ,   
    _KERNEL void GiveObjectSafe( CSubject* obj, CChar* c );

    //  
    _KERNEL void GiveMoney( uint64 money, CChar* c );

    //   
    _KERNEL int CheckUseFlags( CSubject* obj );

    //  
    _KERNEL void DressObject( CSubject* obj );

    //  
    _KERNEL void ArmObject( CSubject* obj, bool dual = false );

    //  
    _KERNEL void KeepObject( CSubject* obj );

    //  
    _KERNEL void RemoveObject( int slotID );

    //    
    _KERNEL void PutObject( CSubject* obj, CSubject* cont );

    //    
    _KERNEL void PutMoney( int m, CSubject* cont );

    //    
    _KERNEL void DeleteByTimer( CSubject* obj, ObjDeletePlace odp );

    //        
    _KERNEL virtual void DeleteByTimerMail( CSubject* obj, ObjDeletePlace odp ) {}

    //   !
    _KERNEL void DeleteNoRentObjects();

    //   
    _KERNEL CObjVirtual* GetVirtualObj();

    //  
    _KERNEL virtual void UpdateStats( PkFlag pkFlag = APF_NOPK ) = 0;

    //    
    _KERNEL virtual void ApplyEquipMod( int type, CModifier* mod, int value, PkFlag pkFlag, bool fullPk = false ) = 0;

    //    
    _KERNEL virtual void ApplyEquipMods( PkFlag pkFlag ) {}

    //  
    _KERNEL void UpdateAllStats();

    // 
    _KERNEL void Lag( uint pulses );

    //  
    _KERNEL uint LagTime();

    //  
    _KERNEL bool inLag() { return m_lagPulse > g_db.m_pulse || m_holdPulse > g_db.m_pulse || m_stonePulse >= g_db.m_pulse; }

    //   
    _KERNEL void RightLag( uint pulses );
    //    
    _KERNEL bool inRightLag() { return m_rightLagPulse > g_db.m_pulse; }

    //   
    _KERNEL void LeftLag( uint pulses );
    //    
    _KERNEL bool inLeftLag() { return m_leftLagPulse > g_db.m_pulse; }

    //  
    _KERNEL void EvadeLag( uint pulses );
    //   
    _KERNEL bool inEvadeLag() { return m_evadeLagPulse > g_db.m_pulse; }

    //     
    _KERNEL void ShowObjectProtInfo( CObjPrototype* prot, CSubject* obj = NULL, int power = 0, bool admin = false, bool look = false, bool otherChar = false );

    //    
    _KERNEL void ShowMobPrototypeInfo( CMobPrototype* prot, CCharMob* mob = NULL, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL void ShowCharPlayerInfo( CCharPlayer* target, int power, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL virtual const char* clr( int itemID ) { return ""; }

    //    
    _KERNEL virtual void ClearCharReference( CCharPlayer* c );

    //    
    _KERNEL void ClearInstReference( CZoneInstance* zoneInst );

public: // 

    //   
    _KERNEL int instIdx();

public: //  -

    // -   
    _KERNEL PkFlag agroAct( CChar* target );

    // -     
    _KERNEL PkFlag checkNonGroupAgroAct( CChar* target );

    // -   
    _KERNEL PkFlag checkSupportAgroAct( CChar* target );

public: // 

    //     
    _KERNEL bool HasAddon( CZoneInstance* zoneInst );

    //    
    _KERNEL CCharAddon* GetAddon( CZoneInstance* zoneInst );

public: //  

    //  
    _KERNEL void PerformPulse( PkFlag pkFlag = APF_NOPK );

public: //  

    //  
    _KERNEL void ExecCmd( const char* text );

    //  
    _KERNEL void ExecSocial( const CSocialInfo* si, const char* args );

    // 
    _KERNEL void CmdNews( const char* args ) { News( args ); }
    _KERNEL virtual void News( const char* args ) {}

    // 
    _KERNEL void CmdZones( const char* args ) { Zones( args ); }
    _KERNEL virtual void Zones( const char* args ) {}

    // 
    _KERNEL void CmdLegend( const char* args ) { Legend( args ); }
    _KERNEL virtual void Legend( const char* args ) {}

    // 
    _KERNEL void CmdMode( const char* text );
    _KERNEL virtual void Mode( char* param1, char* param2 ) {};

    // 
    _KERNEL void CmdMove( CChar* leader, RoomExit exit, CPortalSubject* portal = NULL );

    // 
    _KERNEL virtual void CmdPath( const char* args ) {}

    //  
    _KERNEL bool CheckDTRoom();

    //   
    _KERNEL virtual bool anonimeRoom() { return false; }

    //     
    _KERNEL bool CheckNoFlyRoom();

    //    
    _KERNEL void ComeIntoRoom( RoomExit fromExit, bool groupMove, bool noTriggers = false );

    //     
    _KERNEL void ComeIntoRoomTriggers( RoomExit fromExit, bool noTriggers );

    //    
    _KERNEL void ExitFromRoom( CRoom* room, RoomExit exit );

    // 
    _KERNEL void CmdNorth( const char* args );

    // 
    _KERNEL void CmdEast( const char* args );

    // 
    _KERNEL void CmdSouth( const char* args );

    // 
    _KERNEL void CmdWest( const char* args );

    // 
    _KERNEL void CmdUp( const char* args );

    // 
    _KERNEL void CmdDown( const char* args );

    // 
    _KERNEL void CmdEnter( const char* args );

    // 
    _KERNEL void CmdStand( const char* args );

    // 
    _KERNEL void CmdRest( const char* args );

    // 
    _KERNEL void CmdSit( const char* args );

    // 
    _KERNEL void CmdSleep( const char* args );

    // 
    _KERNEL void CmdWake( const char* args );

    // 
    _KERNEL void CmdToWake( const char* args );

    // 
    _KERNEL void CmdLook( const char* args );

    //   
    _KERNEL virtual void LookToRoom( bool forceFull = false, bool forceSmall = false );

    //   
    _KERNEL void LookToChar( CChar* c );

    //   
    //_KERNEL virtual void LookToObject( CSubject* obj, bool otherChar = false ) {}

    // 
    _KERNEL void CmdScan( const char* args );

    // 
    _KERNEL void CmdExits( const char* args );

    // 
    _KERNEL void CmdRent( const char* args ) { Rent( args ); }
    _KERNEL virtual void Rent( const char* args, bool forse = false ) {};

    // 
    _KERNEL void CmdRerent( const char* args ) { Rerent( args ); }
    _KERNEL virtual void Rerent( const char* args ) {};

    //  
    _KERNEL void CmdQuitFail( const char* args ) { QuitFail( args ); }
    _KERNEL virtual void QuitFail( const char* args ) {};

    // 
    _KERNEL void CmdQuit( const char* args ) { Quit( args ); }
    _KERNEL virtual void Quit( const char* args ) {};

    // 
    _KERNEL void CmdForce( const char* args );
    _KERNEL virtual void Force( CChar* who, const char* cmd ) = 0;

    //  
    _KERNEL void CmdCharForce( const char* args ) { CharForce( args ); }
    _KERNEL virtual void CharForce( const char* args ) {}

    // 
    _KERNEL void CmdSpeak( const char* args );

    //  
    _KERNEL void CmdGroupSpeak( const char* args );
    _KERNEL void GroupSpeak( CChar* fromWho, const char* text );

    //  
    _KERNEL virtual void MuteFilter( const char* text ) {}

    //  
    _KERNEL bool CheckMute();

    // 
    _KERNEL void CmdChat( const char* args );

    // 
    _KERNEL void CmdYell( const char* args );

    // 
    _KERNEL void CmdHoller( const char* args );

    // 
    _KERNEL void CmdTell( const char* args ) { Tell( args ); }
    _KERNEL virtual void Tell( const char* args ) {}

    // 
    _KERNEL void CmdAnswer( const char* args ) { Answer( args ); }
    _KERNEL virtual void Answer( const char* args ) {}

    //  
    _KERNEL void CmdNoobTell( const char *args ) { NoobTell( args ); }
    _KERNEL virtual void NoobTell( const char *args ) {}

    // 
    _KERNEL void CmdWhisper( const char* args );

    // 
    _KERNEL void CmdEmotion( const char* args );

    // 
    _KERNEL void CmdTake( const char* args );

    // 
    _KERNEL void CmdPut( const char* args );

    // 
    _KERNEL void CmdThrow( const char* args );

    // 
    _KERNEL void CmdGive( const char* args );

    // 
    _KERNEL void CmdInventory( const char* args ) { Inventory( args ); }
    _KERNEL virtual void Inventory( const char* args ) {}

    // 
    _KERNEL void CmdEquipment( const char* args );

    // 
    _KERNEL void CmdDress( const char* args );

    // 
    _KERNEL void CmdKeep( const char* args );

    // 
    _KERNEL void CmdArm( const char* args );

    // 
    _KERNEL void CmdRemove( const char* args ) { Remove( args ); }
    _KERNEL virtual void Remove( const char* args );

    // 
    _KERNEL void CmdReplace( const char* args );

    // 
    _KERNEL void CmdWho( const char* args ) { Who( args, true ); }
    _KERNEL void CmdWhoOffline( const char* args ) { Who( args, false ); }
    _KERNEL virtual void Who( const char* args, bool online ) {}

    // 
    _KERNEL void CmdWhere( const char* args ) { Where( args ); }
    _KERNEL virtual void Where( const char* args ) {}

    // 
    _KERNEL void CmdArea( const char* args ) { Area( args ); }
    _KERNEL virtual void Area( const char* args ) {}

    // 
    _KERNEL void CmdInvisList( const char* args ) { InvisList( args ); };
    _KERNEL virtual void InvisList( const char* args ) {}

    // 
    _KERNEL void CmdVisList( const char* args ) { VisList( args ); }
    _KERNEL virtual void VisList( const char* args ) {}

    // 
    _KERNEL void CmdIgnoreList( const char* args ) { IgnoreList( args ); }
    _KERNEL virtual void IgnoreList( const char* args ) {}

    // 
    _KERNEL void CmdTrustList( const char* args ) { TrustList( args ); }
    _KERNEL virtual void TrustList( const char* args ) {}

    // 
    _KERNEL void CmdAffects( const char* args );

    //     
    _KERNEL void ShowAffects( CChar* c, PkFlag setFlag = APF_NOPK );

    // 
    _KERNEL void CmdSnoop( const char* args ) { Snoop( args ); }
    _KERNEL virtual void Snoop( const char* args ) {}

    //  
    _KERNEL void CmdSnoopStop( const char* args ) { SnoopStop( args ); }
    _KERNEL virtual void SnoopStop( const char* args ) {}

    // 
    _KERNEL void CmdBug( const char* args ) { Bug( args ); }
    _KERNEL virtual void Bug( const char* args ) {}

    // 
    _KERNEL void CmdMisprint( const char* args ) { Misprint( args ); }
    _KERNEL virtual void Misprint( const char* args ) {}

    // 
    _KERNEL void CmdIdea( const char* args ) { Idea( args ); }
    _KERNEL virtual void Idea( const char* args ) {}

    //  
    _KERNEL void CmdCommands( const char* args ) { Commands( args ); }
    _KERNEL virtual void Commands( const char* args ) {}

    //  
    _KERNEL void CmdSocials( const char* args ) { Socials( args ); }
    _KERNEL virtual void Socials( const char* args ) {}

    // 
    _KERNEL void CmdBan( const char* args ) { Ban( args ); }
    _KERNEL virtual void Ban( const char* args ) {}

    // 
    _KERNEL void CmdFill( const char* args );

    // 
    _KERNEL void CmdPourOut( const char* args );

    // 
    _KERNEL void CmdDescription( const char* args ) { Description( args ); }
    _KERNEL virtual void Description( const char* args ) {}

    // 
    _KERNEL void CmdMail( const char* args ) { Mail( args ); }
    _KERNEL virtual void Mail( const char* args ) {}

    // 
    _KERNEL void CmdWrite( const char* args ) { Write( args ); }
    _KERNEL virtual void Write( const char* args ) {}

    // 
    _KERNEL void CmdTakeAway( const char* args ) { TakeAway( args ); }
    _KERNEL virtual void TakeAway( const char* args ) {}

    // 
    _KERNEL void CmdRead( const char* args ) { Read( args ); }
    _KERNEL virtual void Read( const char* args ) {}

    // 
    _KERNEL void CmdAdmin( const char* args ) { Admin( args ); }
    _KERNEL virtual void Admin( const char* args ) {}

    // 
    _KERNEL void CmdEcho( const char* args ) { Echo( args ); }
    _KERNEL virtual void Echo( const char* args ) {}

    //  
    _KERNEL void CmdLocalEcho( const char* args ) { LocalEcho( args ); }
    _KERNEL virtual void LocalEcho( const char* args ) {}

    // 
    _KERNEL void CmdSplit( const char* args );
    _KERNEL void SplitMoney( uint moneyCount );

    // 
    _KERNEL void CmdAway( const char* args ) { Away( args ); }
    _KERNEL virtual void Away( const char* args ) {}

    // 
    _KERNEL void CmdAppear( const char* args );

    // 
    _KERNEL void CmdCompare( const char* args );

    // 
    _KERNEL void CmdDrink( const char* args ) { Drink( args ); }
    _KERNEL virtual void Drink( const char* args ) {}

    // 
    _KERNEL void CmdEat( const char* args ) { Eat( args ); }
    _KERNEL virtual void Eat( const char* args ) {}

    // 
    _KERNEL void CmdDiagnosis( const char* args );

    // 
    _KERNEL void CmdQuaff( const char* args );
    _KERNEL void QuaffObject( CSubject* obj );

    // 
    _KERNEL void CmdBerserk( const char* args );

    //  
    _KERNEL void CmdCry( const char* args );

    // 
    _KERNEL void CmdRepair( const char* args );

    //  
    _KERNEL void CmdNeatShot( const char* args );
    _KERNEL void NeatShot( CChar** targets, PkFlag pkFlag, CDamageStr* damStr );
    _KERNEL void NeatShotTarget( CChar* target, CDamageStr* damStr );

    //   
    _KERNEL void CheckAutoNeatShot( CChar* target );

    //  
    _KERNEL void CmdAreaShot( const char* args );

    // 
    _KERNEL void CmdTrack( const char* args );

    //  
    _KERNEL void RepairObject( CSubject* obj, CChar* who );

    //  
    _KERNEL void SkillRepair( const char* args );

    // 
    _KERNEL void CmdThrowAttack( const char* args );
    _KERNEL void ThrowAttack( CWeaponSubject* weapon, int slot, int power, CChar** targets, CCharExtDataNode::ThrowType type = CCharExtDataNode::ttNone );
    _KERNEL bool ThrowAttackTarget( CWeaponSubject* weapon, CChar* target, CCharExtDataNode::ThrowType type, int num = 1 );

    // 
    _KERNEL void CmdCompression( const char* args ) { Compression( args ); }
    _KERNEL virtual void Compression( const char* args ) {}

    // 
    _KERNEL void CmdFollow( const char* args );

    // 
    _KERNEL void Follow( CChar* c, bool noMsg = false );

    //    
    _KERNEL bool FindFollowLoop( CChar* c );

    // 
    _KERNEL void CmdUnfollow( const char* args );

    //  -  
    _KERNEL CCharPlayer* FindFollowedRace( RaceID raceID );

    //  -   
    _KERNEL CCharMob* FindFollowedMob( int mobID );

    // 
    _KERNEL void CmdGroup( const char* args ) { Group( args ); }
    _KERNEL void CmdGroupSearch( const char* args ) { GroupSearch( args ); }
    _KERNEL virtual void Group( const char* args ) {}
    _KERNEL virtual void GroupSearch( const char* args ) {}
    _KERNEL virtual void ShowGroup( CCharPlayer* forWho, PArray<short>& columns, PkFlag pkFlag = APF_NOPK ) {}
    _KERNEL virtual void ShowGroupChar( CChar* c, PArray<short>& columns, const char* nameFormat, PkFlag pkFlag = APF_NOPK ) {}
    //   
    _KERNEL CChar* Leader() { if( m_leader ) return m_leader; return this; }
    //  ()
    _KERNEL CChar* groupLeader() { if( m_leader && m_leader->inGroup( this ) ) return m_leader; else return this; }
    //_KERNEL CChar* myLeader() { return m_leader ? m_leader : this; }
    _KERNEL CChar* getGroup( int i );
    _KERNEL CChar* getGroupChar( int i );
    _KERNEL CChar* getGroupedChar( int i );
    _KERNEL CChar* GetNextGroupChar( int& i );

    //  
    _KERNEL void CmdPGroup( const char* args ) { PGroup( args ); }
    _KERNEL virtual void PGroup( const char* args ) {}

    // 
    _KERNEL void CmdCondition( const char* args );
    _KERNEL void Condition( CChar* fromWho, PkFlag pkFlag );

    //  
    _KERNEL RoomExit FindDoor( const char* args );

    // 
    _KERNEL void CmdOpen( const char* args );
    _KERNEL void OpenContainer( CSubject* obj );

    // 
    _KERNEL void CmdClose( const char* args );
    _KERNEL void CloseContainer( CSubject* obj );

    // 
    _KERNEL void CmdUnlock( const char* args );
    _KERNEL void UnlockContainer( CSubject* obj );

    // 
    _KERNEL void CmdLock( const char* args );
    _KERNEL void LockContainer( CSubject* obj );

    // 
    _KERNEL void CmdScore( const char* args ) { Score( args ); }
    _KERNEL virtual void Score( const char* args ) {}

    // 
    _KERNEL void CmdAccount( const char* args ) { Account( args ); }
    _KERNEL virtual void Account( const char* args ) {}

    // 
    _KERNEL void CmdWorld( const char* args );

    // 
    _KERNEL void CmdTime( const char* args );

    // 
    _KERNEL void CmdWeather( const char* args );

    // 
    _KERNEL void CmdMagic( const char* args );

    // 
    _KERNEL void CmdQuests( const char* args ) { ShowQuests( args ); }
    _KERNEL virtual void ShowQuests( const char* args ) {}

    //  
    _KERNEL void CmdDailyTasks( const char* args ) { ShowDailyTasks( args ); }
    _KERNEL virtual void ShowDailyTasks( const char* args ) {}

    // 
    _KERNEL void CmdInvest( const char* args ) { Invest( args ); }
    _KERNEL virtual void Invest( const char* args ) {}

    // 
    _KERNEL void CmdBank( const char* args ) { Bank( args ); }
    _KERNEL virtual void Bank( const char* args ) {}

    // 
    _KERNEL void CmdTransfer( const char* args ) { Transfer( args ); }
    _KERNEL virtual void Transfer( const char* args ) {}

    // 
    _KERNEL void CmdClanBank( const char* args ) { ClanBank( args ); }
    _KERNEL virtual void ClanBank( const char* args ) {}

    // 
    _KERNEL void CmdClanInvest( const char* args ) { ClanInvest( args ); }
    _KERNEL virtual void ClanInvest( const char* args ) {}

    // 
    _KERNEL void CmdClanRemove( const char* args ) { ClanRemove( args ); }
    _KERNEL virtual void ClanRemove( const char* args ) {}

    // 
    _KERNEL void CmdClanTransfer( const char* args ) { ClanTransfer( args ); }
    _KERNEL virtual void ClanTransfer( const char* args ) {}

    // 
    _KERNEL void CmdStockage( const char* args ) { Stockage( args ); }
    _KERNEL virtual void Stockage( const char* args ) {}

    // 
    _KERNEL void CmdStore( const char* args ) { Store( args ); }
    _KERNEL virtual void Store( const char* args ) {}

    // 
    _KERNEL void CmdWithdrew( const char* args ) { Withdrew( args ); }
    _KERNEL virtual void Withdrew( const char* args ) {}

    // 
    _KERNEL void CmdSend( const char* args ) { SendObj( args ); }
    _KERNEL virtual void SendObj( const char* args ) {}

    // 
    _KERNEL void CmdClanStockage( const char* args ) { ClanStockage( args ); }
    _KERNEL virtual void ClanStockage( const char* args ) {}

    // 
    _KERNEL void CmdClanStore( const char* args ) { ClanStore( args ); }
    _KERNEL virtual void ClanStore( const char* args ) {}

    // 
    _KERNEL void CmdClanWithdrew( const char* args ) { ClanWithdrew( args ); }
    _KERNEL virtual void ClanWithdrew( const char* args ) {}

    // 
    _KERNEL void CmdClanSend( const char* args ) { ClanSendObj( args ); }
    _KERNEL virtual void ClanSendObj( const char* args ) {}

    // 
    _KERNEL void CmdExchange( const char* args ) { Exchange( args ); }
    _KERNEL virtual void Exchange( const char* args ) {}

    // 
    _KERNEL void CmdAuction( const char* args ) { Auction( args ); }
    _KERNEL virtual void Auction( const char* args ) {}

    // 
    _KERNEL void CmdDonateInvest( const char* args ) { DonateInvest( args ); }
    _KERNEL virtual void DonateInvest( const char* args ) {}

    //    
    _KERNEL void CmdOperateSoul( const char* args ) { OperateSoul( args ); }
    _KERNEL virtual void OperateSoul( const char* args ) {}

    // 
    _KERNEL void CmdColor( const char* args ) { Color( args ); }
    _KERNEL virtual void Color( const char* args ) {}

    // 
    _KERNEL void CmdSystem( const char* args ) { System( args ); }
    _KERNEL virtual void System( const char* args ) {}

    // 
    _KERNEL void CmdBuy( const char* args ) { Buy( args ); }
    _KERNEL virtual void Buy( const char* args ) {}

    // 
    _KERNEL void CmdList( const char* args ) { List( args ); }
    _KERNEL virtual void List( const char* args ) {}

    // 
    _KERNEL void CmdLookGoods( const char* args ) { LookGoods( args ); }
    _KERNEL virtual void LookGoods( const char* args ) {}

    // 
    _KERNEL void CmdStats( const char* args ) { Stats( args ); }
    _KERNEL virtual void Stats( const char* args ) {}

    // 
    _KERNEL void CmdPrice( const char* args ) { Price( args ); }
    _KERNEL virtual void Price( const char* args ) {}

    // 
    _KERNEL void CmdSell( const char* args ) { Sell( args ); }
    _KERNEL virtual void Sell( const char* args ) {}

    // 
    _KERNEL void CmdLevels( const char* args ) { Levels( args ); }
    _KERNEL virtual void Levels( const char* args ) {}

    // 
    _KERNEL void CmdWeapons( const char* args ) { Weapons( args ); }
    _KERNEL virtual void Weapons( const char* args ) {}

    // 
    _KERNEL void CmdSkills( const char* args ) { Skills( args ); }
    _KERNEL virtual void Skills( const char* args ) {}

    // 
    _KERNEL void CmdSpells( const char* args ) { Spells( args ); }
    _KERNEL virtual void Spells( const char* args ) {}

    // 
    _KERNEL void CmdTrain( const char* args ) { Train( args ); }
    _KERNEL virtual void Train( const char* args ) {}

    // 
    _KERNEL void CmdLearn( const char* args ) { Learn( args ); }
    _KERNEL virtual void Learn( const char* args ) {}

    // 
    _KERNEL void CmdLearnBook( const char* args ) { LearnBook( args ); }
    _KERNEL virtual void LearnBook( const char* args ) {}

    // 
    _KERNEL void CmdGlory( const char* args ) { Glory( args ); }
    _KERNEL virtual void Glory( const char* args ) {}

    //   
    _KERNEL bool CheckManaCost( const CSpell* spl, int power, PkFlag pkFlag );

    //  
    _KERNEL PkFlag CheckCurse( PkFlag pkFlag = APF_ALL );

    //  
    _KERNEL PkFlag PayManaCost( CastType castType, const CSpell* spl, int value, PkFlag pkFlag );

    //  
    _KERNEL bool CheckNoMagic();

    //   
    _KERNEL PkFlag CheckAstralShield( CChar* target, const CSpell* spell, PkFlag pkFlag );

    //   
    _KERNEL bool CheckParryFrag( CChar* target, PkFlag pkFlag );

    // 
    _KERNEL void CmdCast( const char* args );
    _KERNEL bool Cast( CSpell* spell, int power, CastType castType, const char* args, PkFlag setFlag = APF_NOPK );

    //    
    _KERNEL bool hasManaToSpell( CSpell* spell, int power, PkFlag pkFlag );
    _KERNEL bool hasManaToSpell( int id, int power, PkFlag pkFlag );

    // 
    _KERNEL void CmdRecite( const char* args );
    _KERNEL void CmdRecite( CSubject* ol, const char* args );

    //  
    _KERNEL bool CheckBerserk();

    // 
    _KERNEL void CmdHelp( const char* args ) { Help( args ); }
    _KERNEL virtual void Help( const char* args ) {}

    // 
    _KERNEL void CmdRules( const char* args ) { Help( "" ); }

    // 
    _KERNEL void CmdInformation( const char* args ) { Help( "" ); }

    // 
    _KERNEL void CmdAnnounces( const char* args ) { Announces( args ); }
    _KERNEL virtual void Announces( const char* args ) {}

    // 
    _KERNEL void CmdTimer( const char* args );

    // 
    _KERNEL void CmdClan( const char* args ) { Clan( args ); }
    _KERNEL virtual void Clan( const char* args ) {}

    // -
    _KERNEL void CmdClanSpeak( const char* args ) { ClanSpeak( args ); }
    _KERNEL virtual void ClanSpeak( const char* args, const char* name = NULL ) {}

    // -
    _KERNEL void CmdAllySpeak( const char* args ) { AllySpeak( args ); }
    _KERNEL virtual void AllySpeak( const char* args, const char* name = NULL ) {}

    // 
    _KERNEL void CmdFraction( const char* args ) { Fraction( args ); }
    _KERNEL virtual void Fraction( const char* args ) {}

    // 
    _KERNEL void CmdTournament( const char* args ) { Tournament( args ); }
    _KERNEL virtual void Tournament( const char* args ) {}

    // 
    _KERNEL void CmdImmSpeak( const char* args ) { ImmSpeak( args ); }
    _KERNEL virtual void ImmSpeak( const char* args ) {}

    // 
    _KERNEL void CmdPray( const char* args ) { Pray( args ); }
    _KERNEL virtual void Pray( const char* args ) {}

    // 
    _KERNEL void CmdRemember( const char* args ) { Remember( args ); }
    _KERNEL virtual void Remember( const char* args ) {}

    // 
    _KERNEL void CmdJournal( const char* args ) { Journal( args ); }
    _KERNEL virtual void Journal( const char* args ) {}

    // 
    _KERNEL void CmdRecords( const char* args ) { Records( args ); }
    _KERNEL virtual void Records( const char* args ) {}

    // 
    _KERNEL void CmdGAffects( const char* args ) { GAffects( args ); }
    _KERNEL virtual void GAffects( const char* args ) {}

    // 
    _KERNEL void CmdColumns( const char* args ) { Columns( args ); }
    _KERNEL virtual void Columns( const char* args ) {}

    // 
    _KERNEL void CmdGrind( const char* args ) { Grind( args ); }
    _KERNEL virtual void Grind( const char* args ) {}

    // 
    _KERNEL void CmdSetSpec( const char* args ) { SetSpec( args ); }
    _KERNEL virtual void SetSpec( const char* args ) {}

    // 
    _KERNEL void CmdTote( const char* args ) { Tote( args ); }
    _KERNEL virtual void Tote( const char* args ) {}

    // 
    _KERNEL void CmdLoto( const char* args ) { Loto( args ); }
    _KERNEL virtual void Loto( const char* args ) {}

    // 
    _KERNEL void CmdInterview( const char* args ) { Interview( args ); }
    _KERNEL virtual void Interview( const char* args ) {}

    // 
    _KERNEL void CmdHide( const char* args );

    // 
    _KERNEL void CmdSneak( const char* args );

    // 
    _KERNEL void CmdDisarm( const char* args );
    _KERNEL void CmdDisarm( CChar* target = NULL );

    //  
    _KERNEL int disarmPercent( CChar* c, PkFlag pkFlag = APF_NOPK, SlotID slot = SLOT_COUNT );

    // 
    _KERNEL void CmdStab( const char* args );
    _KERNEL void CmdStab( CChar* target = NULL, bool noFight = true );

    //  
    _KERNEL int stabPercent( CChar* target, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdTrip( const char* args );
    _KERNEL bool CmdTrip( CChar* target = NULL );

    //  
    _KERNEL int tripPercent( CChar* target, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdSteal( const char* args );
    _KERNEL void CmdSteal( CChar* target, SlotID slot, CSubject* obj );

    //  
    _KERNEL int stealPercent( CChar* target, SlotID slot, CSubject* obj, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdPickLock( const char* args );
    _KERNEL void CmdPickLock( CExit& target );

    //  
    _KERNEL int picklockPercent( CObjPrototype* key );

    // 
    _KERNEL void CmdPocket( const char* args );
    _KERNEL void CmdPocket( CChar* target );

    // 
    _KERNEL void CmdSweep( const char* args );

    //  
    _KERNEL int pocketPercent( CChar* target, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdMeditation( const char* args );

    // 
    _KERNEL void CmdCampFire( const char* args );

    // 
    _KERNEL void CmdRelax( const char* args );

    // 
    _KERNEL void CmdStatus( const char* args ) { Status( args ); }
    _KERNEL virtual void Status( const char* args ) {}

    // 
    _KERNEL void CmdHome( const char* args ) { Home( args ); }
    _KERNEL virtual void Home( const char* args ) {}

    // 
    _KERNEL void CmdPoison( const char* args ) { Poison( args ); }
    _KERNEL virtual void Poison( const char* args ) {}

    //  
    _KERNEL void CmdBlood( const char* args );

    // 
    _KERNEL void CmdTurn( const char* args );

    // 
    _KERNEL void CmdConcentration( const char* args );

    //  
    _KERNEL bool CheckConcentration();

    // 
    _KERNEL void CmdUngrind( const char* args ) { Ungrind( args ); }
    _KERNEL virtual void Ungrind( const char* args ) {}

    //  
    _KERNEL void CmdRage( const char* args );
    _KERNEL void Rage( byte tGod );

    //     
    _KERNEL double RageCorrection( double tCost, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdSqueeze( const char *args );
    _KERNEL void Squeeze( CChar *tTarget, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL void CmdGetPain( const char *args );

    //  
    //_KERNEL void CmdHolyHunting( const char *args );

    //  
    _KERNEL void CmdStance( const char *args );

    // 
    _KERNEL void CmdScatter( const char *args ) { Scatter( args ); }
    _KERNEL virtual void Scatter( const char* args ) {}

    //  
    _KERNEL void CmdAssemble( const char* args ) { Assemble( args ); }
    _KERNEL virtual void Assemble( const char* args ) {}

    //  
    _KERNEL void CmdInsert( const char* args ) { Insert( args ); }
    _KERNEL virtual void Insert( const char* args ) {}

    //  
    _KERNEL void CmdExtract( const char* args ) { Extract( args ); }
    _KERNEL virtual void Extract( const char* args ) {}

    // 
    _KERNEL void CmdStaff( const char* args ) { Staff( args ); }
    _KERNEL virtual void Staff( const char* args ) {}

public: // 

    // 
    _KERNEL void CmdKill( const char* args );
    _KERNEL bool Kill( CChar* c, CChar* whomAssist = NULL );

    //  
    //_KERNEL void Knock( CChar* c, PkFlag pkFlag = APF_NOPK, bool prime = true, int knockNum = 0, int farType = 0, bool warCry = false );
    _KERNEL void Knock( CChar* c, PkFlag pkFlag = APF_NOPK, int knockNum = 0, int kFlag = KF_NONE );

    //    
    _KERNEL int weaponDamage( CWeaponSubject* weapon, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL int knockDamage( CChar* c, PkFlag pkFlag, CWeaponSubject* weapon, bool& critical );

    //   
    _KERNEL void MagicStaffKnock( CChar* c, PkFlag pkFlag = APF_NOPK, int knockNum = 0 );

    //      
    _KERNEL int magicStaffDamage( CChar* c, PkFlag pkFlag, CWeaponSubject* weapon );

    //    
    _KERNEL int stabDamage( CChar* c, PkFlag pkFlag, CWeaponSubject* weapon );

    //   
    _KERNEL virtual bool PerformCustomHit( CChar* c, int knockNum, bool nearFight ) = 0;

    //    
    _KERNEL virtual void AutoAgroModeOn( CCharPlayer* cp ) {}

    //    
    _KERNEL void AgroToMe( CChar* who, bool agroModeOn = true, bool direct = true, bool self = true );

    //    
    _KERNEL void SupportToMe( CChar* who );

    //  ,    
    _KERNEL void CauseFight( CChar* c, bool nearFight, PkFlag pkFlag = APF_NOPK );

    //       
    _KERNEL int DamageModifier( int damage, PkFlag pkFlag = APF_NOPK, MagicSphere magicSphere = MS_NONE, bool critical = false, int destructionSkill = 0 );

    //    
    _KERNEL bool IsFatalDamage( int damage, CChar* who, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL int TryCauseDamage( CChar* c, int damage, bool magic, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool CauseDamage( CChar* c, int damage, bool magic, bool nearFight, PkFlag pkFlag = APF_NOPK, CWeaponSubject* weapon = NULL, bool causeFight = true, bool warCry = false, bool toEquip = true );

    //   
    _KERNEL void CausePoison( CChar* c, PkFlag pkFlag = APF_NOPK, CWeaponSubject* weapon = NULL, bool useThrow = false );

    //  
    _KERNEL void CauseMagicArrows( CChar* c, PkFlag pkFlag, CDamageStr* damStr, int shotFlag, bool firstShot );

    //   
    _KERNEL void CauseMagicStaff( CChar* c, PkFlag pkFlag, int knockNum );

    //  
    _KERNEL void CauseHolyHunting( CChar* c, CWeaponSubject* weapon, PkFlag pkFlag );

    //    
    _KERNEL void CauseManaDamage( CChar* c, int damage, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL void CauseMVDamage( CChar* c, int damage, PkFlag pkFlag = APF_NOPK );

    //     
    _KERNEL void CauseSlotDamage( int damageSlot, int damage, CChar* who );

    // 
    _KERNEL void Heal( int hp, PkFlag pkFlag = APF_NOPK, bool noMsg = false );

    //   
    _KERNEL int TryChangeHP( double hp, PkFlag pkFlag = APF_NOPK, uint uFlags = CHF_NONE );

    //  HP
    _KERNEL bool ChangeHP( double hp, CChar* who, PkFlag pkFlag = APF_NOPK, double* pCausedDamage = NULL, uint uFlags = CHF_NONE );

    //  
    _KERNEL void FreshMana( int mana, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL void FreshMV( int mv, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL virtual void Saturation( int hours ) {}

    //    
    _KERNEL ObjCorpseChance objCorpseChance( CSubject* obj );

    //  
    _KERNEL virtual void RIP( CChar* who = NULL, bool toStone = false, bool noSignsOfDeath = false, PkFlag pkFlag = APF_NOPK, bool dtRip = false );

    //    
    _KERNEL virtual void LoadOnCorpse( CSubject* corpse, CChar *who = NULL ) {}

    // 
    _KERNEL void CmdFlee( const char* args );
    _KERNEL void Flee( bool noDT = false, RoomExit re = RE_NONE );

    // 
    _KERNEL void CmdRetreat( const char* args );

    // 
    _KERNEL void CmdComeIn( const char* args );

    // 
    _KERNEL void CmdComeOut( const char* args );

    // 
    _KERNEL void CmdAssist( const char* args );

    // 
    _KERNEL void CmdKick( const char* args );
    _KERNEL void CmdKick( CChar* target = NULL );

    // 
    _KERNEL void CmdSwitch( const char* args );
    _KERNEL void CmdSwitch( CChar* target );

    //   
    _KERNEL bool canRescue( CChar* target );

    // 
    _KERNEL void CmdRescue( const char* args );
    _KERNEL bool CmdRescue( CChar* target = NULL );

    // 
    _KERNEL void CmdPush( const char* args );

    //  
    _KERNEL void CmdDrow( const char* args );

    // 
    _KERNEL void CmdBash( const char* args );
    _KERNEL void CmdBash( CChar* target = NULL );

    //  
    _KERNEL int bashPercent( CChar* c, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL void CmdBandaging( const char* args );
    _KERNEL void CmdBandaging( CChar* target, PkFlag setFlag );

    // 
    _KERNEL void CmdShelter( const char* args ) { Shelter( args ); }
    _KERNEL virtual void Shelter( const char* args ) = 0;

    //   
    _KERNEL void FinishFight();

    //     ,    
    _KERNEL void UpdateFightTarget();

    //   
    _KERNEL void GroupGainExpD( double exp, double limitK = 1 );

    //     
    _KERNEL void GroupGainExpD( CCharMob* mob, double limitK = 1 );

    //  -
    _KERNEL virtual bool CheckAgro( CChar* c, bool noMsg = false );

    //   
    _KERNEL bool CheckSupport( CChar* c, bool noMsg = false );

public: //   

    //    
    _KERNEL void CastAffect( uint id, uint pulses, PkFlag pkFlag = APF_NOPK, int value = 80, bool doubl = false, bool noUpdate = false );

    //    
    _KERNEL void DispelAffect( AffectID id, PkFlag pkFlag = APF_NOPK, bool noUpdate = false );

    //   
    _KERNEL void RemoveCastAffect( int num );

    //   
    _KERNEL void ClearCastAffects();

    //    
    _KERNEL CAffectData* FindAffect( AffectID id, PkFlag pkFlag, byte count = 0 );

    //    
    _KERNEL bool hasAffect( byte id, PkFlag pkFlag );

    //    
    _KERNEL bool hasAffects( PkFlag pkFlag, AffectID id1 = AFF_COUNT, AffectID id2 = AFF_COUNT, AffectID id3 = AFF_COUNT, AffectID id4 = AFF_COUNT, AffectID id5 = AFF_COUNT, AffectID id6 = AFF_COUNT, AffectID id7 = AFF_COUNT, AffectID id8 = AFF_COUNT, AffectID id9 = AFF_COUNT );

    //     
    _KERNEL bool hasCastAffect( AffectID id, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL CAffectUnit* FindAffectUnit( byte id, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL void AddAffectUnit( byte id, PkFlag type = APF_NOPK, int value = 80, bool doubl = true );

    //  - 
    _KERNEL PkFlag GetAffectPkFlag( AffectID id );

    //   
    _KERNEL bool getAffectExist( byte id, PkFlag pkFlag );

    //   
    _KERNEL void DegradeAffect( AffectID id, PkFlag pkFlag = APF_NOPK, bool noUpdate = false );

    //    
    _KERNEL CAffectUnit* FindAffProtect( MagicSphere sphere, PkFlag pkFlag = APF_NOPK );

public: //  

    //  -
    _KERNEL CChar* FindCharTarget( CMaskManager& mm );

    //  -
    _KERNEL CSubject* FindObjectTarget( CMaskManager& mm );

    //  -
    _KERNEL CSubject* FindContainerTarget( CMaskManager& mm );

    //  
    _KERNEL virtual void ProcessTarget( char* purpose ) {}

public: // 

    //     
    _KERNEL virtual int percModifier( CChar* c, int power ) { return power; }

    //    
    _KERNEL bool CheckCastOnlyMe( int id );

    //    
    _KERNEL bool CheckSkillOnlyMe( int id );

    //    
    _KERNEL void CastMessage( const CSpell* spell, CChar* target, CastType castType, PkFlag pkFlag, int power = 0, const char* spellName = NULL, ColorID color = COLOR_0 );

    //  
    _KERNEL bool CastEmpty( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastFrag( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastFrag( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target = NULL );

    // 
    _KERNEL bool CastHeal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastHeal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastMassHeal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastMassAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // -
    _KERNEL bool CastAgroAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastAgroAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastDrainLevel( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastDrainLevel( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //   
    _KERNEL bool CastResistMagic( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastResistMagic( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastChainLighting( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastChainLighting( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastCallAnimal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastCallElements( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastCallUndead( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastCallSpirit( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastDrain( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastDrain( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastWordOfRecall( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastWordOfRecall( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //   
    _KERNEL bool CastMassWordOfRecall( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool Recall( int power = 0 );

    // 
    _KERNEL bool CastReplace( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastAppeal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastAppealZone( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args, bool zoneOnly );
    _KERNEL bool CastAppeal( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CCharPlayer* target, bool zoneOnly = false );

    //  
    _KERNEL void TeleportTriggers( CRoom* from );

    // 
    _KERNEL bool CastIdentyty( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastFresh( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastFresh( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastMassFresh( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastFreshMana( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastFreshMana( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    // 
    _KERNEL bool CastSaturation( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastSaturation( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastCreateLight( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastCreateWater( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastCreateFood( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //    
    _KERNEL void SendLocateString( CSubject* ol, int type, int power, CZone* zone, const char* place, CChar* owner );

    //  
    _KERNEL bool CastFindObject( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastFindCharacter( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //     
    _KERNEL void SendLocateCharacterInfo( CChar* cl, int power, CZone* zone );

    //    
    _KERNEL void FillAutoExits( char *buf );

    //  
    _KERNEL bool CastUpWeapon( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastUpCloth( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastHold( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastHold( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL void Hold( uint pulses, bool noMsg = false );

    //  
    _KERNEL void Unhold( bool noMsg = false );

    //  
    _KERNEL int holdPercent( CChar* c, CBaseHoldSpell* holdSpell, int power, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool holded() { return m_holdPulse > g_db.m_pulse; }

    //   
    _KERNEL void Stone( uint pulses, bool noMsg = false );

    //   
    _KERNEL void Unstone( bool noMsg = false );

    //    
    _KERNEL bool stoned() { return m_stonePulse > g_db.m_pulse; }

    //   
    _KERNEL void DrainLevel( int drainLevel, int pulses, int power, PkFlag pkFlag );

    // 
    _KERNEL bool CastFear( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastFear( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //   
    _KERNEL int fearPercent( CChar* c, int power, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool CastCureBlindness( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastCureBlindness( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastRemoveCurse( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastRemoveCurse( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastReleaseMental( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastReleaseMental( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastCurePoison( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastCurePoison( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastWordOfPowerSpell( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastWordOfPowerSpell( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastDispelMagic( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastDispelMagic( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL void PerformDispel( CChar* who, const CSpell* spell, int* power, PkFlag pkFlag, bool agro, bool dispelAlly = true );

    // 
    _KERNEL bool CastLowSize( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastLowSize( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastWordOfLight( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastWordOfLife( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastFreeWill( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastClearingRitual( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastWordOfBalance( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastMajesty( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastDegeneration( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastInvisibleEye( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastStoneCurse( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastStoneCurse( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //   
    _KERNEL int stoneCursePercent( CChar* c, int power, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool CastLifeTouch( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastLifeTouch( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL bool CastColorRay( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastPolymorph( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastPolymorph( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //  
    _KERNEL int polymorphPercent( CChar* c, int power, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool CastGuardCircle( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastSealOfFriendship( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastSealOfFriendship( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

    //   
    _KERNEL int sealOfFriendshipPercent( CChar* c, int power, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL bool CastLivingRoots( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL bool CastMassSpell( SpellMassType massType, const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    // 
    _KERNEL bool CastMassFrag( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //  
    _KERNEL void MassFragTarget( const CSpell* spell, CChar* target, PkFlag pkFlag, CastType castType, int power, double koef );

    //  
    _KERNEL void PerformHarm( CChar* who, const CSpell* spell, int power, PkFlag pkFlag );

    //  -
    _KERNEL bool CastMassAgroAffect( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //   
    _KERNEL void MassAgroAffectTarget( const CSpell* spell, CChar* target, PkFlag pkFlag, int power, double koef );

    // 
    _KERNEL bool CastMassHold( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //   
    _KERNEL void MassHoldTarget( const CSpell* spell, CChar* target, PkFlag pkFlag, int power, double koef );

    //  
    _KERNEL bool CastMassFear( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );

    //   
    _KERNEL void MassFearTarget( const CSpell* spell, CChar* target, PkFlag pkFlag, int power, double koef );

    //  
    _KERNEL bool CastOblivion( const CSpell* spell, int power, CastType castType, PkFlag setFlag, const char* args );
    _KERNEL bool CastOblivion( const CSpell* spell, int power, CastType castType, PkFlag setFlag, CChar* target );

public: //    

    //   
    _KERNEL bool CastAddImp( int power, CChar* target, PkFlag pkFlag = APF_NOPK );

public: //   

    //   
    _KERNEL void SkillLag( int id, uint lag, byte type = 0 );

    //   
    _KERNEL uint skillLagPulse( int id, byte type = 0 );

public: //   

    //   
    _KERNEL void SpellLag( int id, uint lag );

    //   
    _KERNEL uint spellLagPulse( int id );

public: // 

    //  ,   
    _KERNEL int GetIngredientPower( IngrColorID color = INGR_COLOR_NONE );

    //  
    _KERNEL void DestroyIngredient();

public: //    

    //   
    _KERNEL double curHP  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_curHP;   }
    _KERNEL double curMana( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_curMana; }
    _KERNEL double curMV  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_curMV;   }

    _KERNEL double dCurHP  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_dCurHP;   }
    _KERNEL double dCurMana( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_dCurMana; }
    _KERNEL double dCurMV  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_dCurMV;   }

    _KERNEL double maxHP  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_maxHP;   }
    _KERNEL double maxMana( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_maxMana; }
    _KERNEL double maxMV  ( PkFlag pkFlag = APF_NOPK ) { return m_stats[pkFlag]->m_maxMV;   }

    _KERNEL void setCurHP( double val, PkFlag pkFlag = APF_NOPK, bool limits = true );
    _KERNEL void setCurMana( double val, PkFlag pkFlag = APF_NOPK ) { m_stats[pkFlag]->m_dCurMana = val; m_stats[pkFlag]->m_curMana = val; }
    _KERNEL void setCurMV  ( double val, PkFlag pkFlag = APF_NOPK ) { m_stats[pkFlag]->m_dCurMV   = val; m_stats[pkFlag]->m_curMV   = val; }

    //  
    int curHPPerc( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag]->m_curHP*100/m_stats[pkFlag]->m_maxHP ); }
    int curManaPerc( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag]->m_curMana*100/m_stats[pkFlag]->m_maxMana ); }
    int curMVPerc( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag]->m_curMV*100/m_stats[pkFlag]->m_maxMV ); }

    //     
    int needHP( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag] ? m_stats[pkFlag]->m_maxHP - m_stats[pkFlag]->m_curHP : 0 ); }
    int needMana( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag] ? m_stats[pkFlag]->m_maxMana - m_stats[pkFlag]->m_curMana : 0 ); }
    int needMV( PkFlag pkFlag = APF_NOPK ) { return (int)( m_stats[pkFlag] ? m_stats[pkFlag]->m_maxMV - m_stats[pkFlag]->m_curMV : 0 ); }

    //     
    int curHPByDamageType( PkFlag pkFlag );

    //    ( ,   -  )
    _KERNEL double mentalResist( PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL virtual double levelK( CChar* target );

    //    
    _KERNEL void UpdateAIFlags();

public: // 

    //  
    _KERNEL void QuestMessage( const char* text, ... );

    //   
    _KERNEL void QuestMessage( CChar* c, const char* text1, const char* text2, ... );

    //      "  ."
    _KERNEL void InTeleportMessage();

    //      " ."
    _KERNEL void OutTeleportMessage();

    //      
    _KERNEL bool Teleport( CRoom* room, bool noMessage = false, bool toInst = false );

public: //  

    //     
    _KERNEL virtual void SetMentalChild( CCharMob* mob, int charmSpell, int charmControl, int mobPkProtId = 0 ) {}

    //   
    _KERNEL virtual void ClearMentalChild() {}

    //   
    _KERNEL virtual void SetMentalOwner( CCharPlayer* cp, int mobPkProtId = 0 ) {}

    //   
    _KERNEL virtual void ClearMentalOwner() {}

    //    
    _KERNEL int noBotConditions( CRoom* room, int type = 0 );

    //    
    _KERNEL int noBotGroupConditions( bool msg = false );

    //  !
    _KERNEL BOOL checkNoBotRankNew();

public: //  

    //   
    _KERNEL void SetSeal( CChar* fromWho, int pulses );

    //   
    _KERNEL void RemoveSeal( CChar* fromWho, bool msg = true );

    //   
    _KERNEL void RemoveSeal( CSeal* seal, bool msg = true );

    //    
    _KERNEL bool hasSeal( CChar* fromWho );

public: //   

    //   
    void GetExtData();

    //   
    void CheckExtData();

    //  
    _KERNEL CMobPrototype* getTurnProt() { return m_extData ? m_extData->turnProt() : NULL; }
    _KERNEL void setTurnProt( CMobPrototype* prot );

    //  
    _KERNEL int getTurnPowerNopk() { return m_extData ? m_extData->turnPower() : 0; }
    _KERNEL void setTurnPowerNopk( int power );

    // - 
    _KERNEL int getTurnPowerPk() { return m_extData ? m_extData->turnPowerPk() : 0; }
    _KERNEL void setTurnPowerPk( int power );

    //  
    _KERNEL int getDrainLevel() { return m_extData ? m_extData->drainLevel() : 0; }
    _KERNEL void setDrainLevel( int level );

    //   
    _KERNEL CChar* getCryChar() { return m_extData ? m_extData->cryChar() : NULL; }
    _KERNEL void setCryChar( CChar* c );

    //  
    _KERNEL PArray<COblivionNode*>* getOblivion() { return m_extData ? m_extData->oblivion() : NULL; }
    _KERNEL void setOblivion( PArray<COblivionNode*>* oblivion );

    //  
    _KERNEL CChar* getAngerChar() { return m_extData ? m_extData->anger() : NULL; }
    _KERNEL void setAngerChar( CChar* anger );

    //   
    _KERNEL CChar* getHolyCaster() { return m_extData ? m_extData->holyCaster() : NULL; }
    _KERNEL void setHolyCaster( CChar* caster );

    // 
    _KERNEL PArray<CChar*>* getAims() { return m_extData ? m_extData->aims() : NULL; }
    _KERNEL void setAims( PArray<CChar*>* aims );

    // - 
    _KERNEL CMobPrototype* getCharmPk() { return m_extData ? m_extData->charmPk() : NULL; }
    _KERNEL void setCharmPk( CMobPrototype* prot );

    //   
    _KERNEL CShelterZonesData* getShelterZones() { return m_extData ? m_extData->shelterZones() : NULL; }
    _KERNEL void setShelterZones( CShelterZonesData* zonesData );

    //  
    _KERNEL CCharExtDataNode::ThrowType getThrowType() { return m_extData ? m_extData->throwType() : CCharExtDataNode::ttNone; }
    _KERNEL void setThrowType( CCharExtDataNode::ThrowType type );

    //  
    _KERNEL void* getExtZoneData( int zoneId ) { return m_extData ? m_extData->getZoneData( zoneId ) : nullptr; }
    _KERNEL void setExtZoneData( int zoneId, void* pData );

    //  
    _KERNEL int getBonusReagent() { return m_extData ? m_extData->bonusReagent() : 0; }
    _KERNEL void setBonusReagent( int bonus );

    //  
    _KERNEL int getBonusExp() { return m_extData ? m_extData->bonusExp() : 0; }
    _KERNEL void setBonusExp( int bonus );

    //  
    _KERNEL int getBonusAbility() { return m_extData ? m_extData->bonusAbility() : 0; }
    _KERNEL void setBonusAbility( int bonus );

public: // 

    //   
    _KERNEL void ArenaResetStats( bool lags = true );

public: // 

    CChar* m_next;  //   

    CChar* m_leader;             // ,   
    PArray<CGroupMember> m_group;  // ,    
    bool m_follow;               //    

    CSubject* m_objList;           //   
    CSubject* m_equip[SLOT_COUNT]; //  
    CObjVirtual* m_virtualObj;     //  

    PArray<CAffectData*> m_castAffects; //  
    byte m_castAffBits[AFF_MAX_COUNT/8];

    uint m_lagPulse;                    //   

    PArray<SkillLagData> m_skillLags; //    
    PArray<SpellLagData> m_spellLags; //    

    //   
    uint m_rightLagPulse;  //    
    uint m_leftLagPulse;   //  
    uint m_evadeLagPulse;  // 

    uint m_disarmLagPulse; //   

    CChar* m_fight;    // ,   
    bool m_nearFight;  //   
    int m_fightOrder;  //    

    CSeal* m_sealList; //   

public: //  

    CRoom* m_room;    // 
    byte m_pos;       // 
    uint m_holdPulse;  //      
    uint m_stonePulse; //       

    CCharStats* m_stats[APF_COUNT]; //  

    PArray<CCharAddon*> m_addons; //   

    bool m_light;      //    

    CCharExtData* m_extData;    //  

    //  
    bool polymorphed() { return getTurnProt() && !getTurnPowerNopk(); }

    //  
    bool turned() { return getTurnProt() && getTurnPowerNopk(); }

    //   
    BYTE m_scan;

    //     
    ushort m_aiFlags;

    static BOOL m_curNoBot;

};
