/*  -

    Copyright (c) 2001 - 2004. All Rights Reserved.
    :  ..
*/
#pragma once
#include "DB/Char/Char.h"
#include "DB/DBObject.h"
#include "DB/Journal.h"
#include "DB/Char/Clan.h"

//   
class CCharStoreSection
{

public: // 

    CCharStoreSection() :
        m_itemCount( 0 ), m_hotel( 0 )
    {}

public: // 

    //  
    virtual byte id() = 0;

    //  
    virtual const char* name() const = 0;

    //   
    virtual void Serialize( CFileStore& fs ) = 0;

public: // 

    int m_itemCount; //  
    byte m_hotel;    //  

};

//    
class CCharStoreMainSection : public CCharStoreSection
{

public: // 

    //  
    byte id() { return 0; }

    //  
    const char* name() const { return ""; }    

    //   
    void Serialize( CFileStore& fs );

};

//    
class CCharStoreAddSection : public CCharStoreSection
{

public: // 

    CCharStoreAddSection() :
        m_id( 0 )
    {}

public: // 

    //  
    byte id() { return m_id; }

    //  
    const char* name() const { return m_name; }    

    //   
    void Serialize( CFileStore& fs );

public: // 

    byte m_id;                        // 
    char m_name[MAX_SECTION_NAME+1];  // 

};

//    
class CCharStoreNode
{

public: // 

    CCharStoreNode() :
        m_obj( NULL ), m_section( NULL ), m_next( NULL )
    {}

    ~CCharStoreNode() {}

public: // 

    CSubject*          m_obj;     // 
    CCharStoreSection* m_section; // 
    CCharStoreNode*    m_next;    // 

};

//   (  )
class CTell
{

public: // 

    CCharPlayer* m_char; //  
    uint m_pulse;        //  
    char m_text[1];      //  
};

//  
class CRestrictNode
{

public: // 

    const char* m_str; // 
    int m_type;        // 
    int m_id;          // 

};

//  
class CObjFilter
{

public: // 

    CObjFilter()
    {
        // <<<<<<<<<<<
        ::memset( this, 0, sizeof(*this) );
    }

public: // 

    //   
    _KERNEL void Reset();

    //   
    _KERNEL bool SetType( const char* args );

    //   
    _KERNEL void AddNameMask( const char* args );

    //   
    _KERNEL void AddTypeMask( const char* args );

    //   
    _KERNEL bool AddAffMod( char* args );

    //  
    _KERNEL bool Accept( CSubject* obj, CConnection* conn, COffer* offer );

public: // 

    byte m_mode;              //  

    byte m_type;              //  

    int m_idx;                //   

    bool m_byName;            //   
    char* m_nameMask;         //  
    CMaskManager m_nameMM;    //  

    bool m_byType[2];         //   
    char* m_typeMask[2];      //  
    CMaskManager m_typeMM[2]; //  

    uint m_hours;             //   

    int m_appraise;           //   

    PriceType m_priceType;    //  
    bool m_byRestrict;        //   
    CCharRestrict m_restrict; //   

    CAffect* m_affect;        //  
    CModifier* m_modifier;    //  
    int m_bonusVal;           //  

};

//   
class CDTSubject
{

public: //   

    CDTSubject( CSubject* obj = NULL, double exp = 0, uint pulse = 0 ) :
        m_obj( obj ), m_exp( exp ), m_pulse( pulse )
    {}

    ~CDTSubject() { g_db.destroyObj( m_obj ); }

public: // 

    _KERNEL void Serialize( CFileStore& fs ); //   

public: // 

    CSubject* m_obj; // 
    double m_exp;    //  
    uint m_pulse;    // 

};

//   
class CFightBuffer
{

public: // 

    _KERNEL CFightBuffer();

    ~CFightBuffer() {}

public: // 

    //  
    _KERNEL void Reset();

    //  
    _KERNEL void Init( CRoom* room, CChar* hiter, CChar* target, CDamageStr* damStr, int hitType = BHT_ATTACK, CWeaponSubject* weapon = NULL );

    //  
    _KERNEL bool Compare( CChar* hiter, CChar* target );

    //  
    _KERNEL void AddMissStr( CRoom* room, HitCheck check, CChar* hiter, CChar* target, CDamageStr* damStr, int hitType = BHT_ATTACK, CWeaponSubject* weapon = NULL );

    //  
    _KERNEL void AddHitStr( CRoom* room, CChar* hiter, CChar* target, CDamageStr* damStr, int damage, int causedDamage, int hitType = BHT_ATTACK, CWeaponSubject* weapon = NULL );

    //   
    _KERNEL void AddFightMsg( byte msgType, CRoom* room, CChar* hiter, CChar* target, int nCausedDamage = 0, CDamageStr* damStr = NULL );

    //    
    _KERNEL void AddBufferStr( CRoom* room, char* str );

    //   
    _KERNEL bool IsCorrectData();

    //   
    _KERNEL int* FightMsgDamageValue(uint uMsgType);

    //   
    enum FightMsgDamage
    {
        FMD_NONE = 0,
        FMD_FIRESHIELD,
        FMD_FIREARROWS,
        FMD_HOLYHUNT,
        FMD_COUNT
    };

public: // 

    bool m_active;        //  
    CChar* m_hiter;       // 
    CChar* m_target;      // 
    CDamageStr* m_damStr; //  
    int m_hitType;        //  
    CWeaponSubject* m_weapon; // 

    int m_acCount;        //  
    int m_evadeCount;     //  
    int m_blockCount;     //  
    int m_parryCount;     //  
    int m_fireCount;      //    

    int m_hitCount;       //  
    int m_damage;         // 
    int m_damageMin;      //  
    int m_damageMax;      //  
    int m_causedDamage;   //  

    bool m_flags[FMT_COUNT]; //   
    int m_flagsDamage[FMD_COUNT]; // ,   

    char m_buffer[SOCK_SND_BUFSIZE]; //   
    char* m_buf;          //   
    bool m_bOverflow;     //  

};

//   
class CCharPlayerStats : public CCharStats
{

public: //   

    // 
    CCharPlayerStats()
    {
        m_weapon = new double[WS_COUNT];
        m_skill = new double[SKILL_MAX_COUNT];
        m_spell = new double[SPELL_MAX_COUNT];
    }

    // 
    ~CCharPlayerStats()
    {
        delete[] m_weapon;
        delete[] m_skill;
        delete[] m_spell;
    }

};

//    
class CBotCheckData
{

public: // 

    _KERNEL CBotCheckData();

public: // 

    int m_checkType;       //  
    CCharPlayer* m_char;   // 
    uint m_moveLimit;      //  

    bool m_botChecking;    //    
    uint m_checkInitPulse; //   
    uint m_botCheckPulse;  //    
    uint m_botWaitPulse;   //   

    char* m_checkColor;    //  
    int m_imitType;        //  
    int m_botCheckCount;   //   
    int m_botCheckLimit;   //   

};

//   
class CGloryStats
{

public: // 

    CGloryStats();

public: // 

    // 
    void Serialize( CFileStore& fs );

    //  
    byte AbilityValue( int type, int id );

    //   
    byte* AbilityRef( int type, int id );

    //   
    bool CanIncreaseAbility( int type, int id, CCharPlayer* cp = NULL, bool msg = false );

    //  
    int GetGlorySumm();

public: // 

    byte m_weapon[WS_COUNT];        //   
    byte m_skill[SKILL_MAX_COUNT];  //   
    byte m_spell[SPELL_MAX_COUNT];  //   

    byte m_resist[RESIST_COUNT];    // 
    byte m_resistMax[RESIST_COUNT]; //  

    Array<Pair<ushort,byte> > m_affects; //  (,)

    int m_totalGlory;               //  

};


//    
class CCharZoneProgress
{

public: // 

    CCharZoneProgress( int zoneId, int count ) :
        m_zoneId( zoneId ), m_count( count )
    {}

public: // 

    bool Compare( int nValue, uint uType )
    {
        switch( uType ) {
        case SCT_ID:
            return m_zoneId == nValue;
        default:
            break;
        };
        return false;
    }

public: // 

    int m_zoneId; // 
    int m_count;  //  

};


// -
class CCharPlayer : public CChar, public CDBObject
{

public: // 

    _KERNEL CCharPlayer();

    _KERNEL ~CCharPlayer();

public: //   

    //   
    _KERNEL void InitNew();

    //   
    _KERNEL void Serialize( CFileStore& fs, int restoreAcc = 0 );

    //  
    _KERNEL void Link();

public: // 

    _KERNEL bool operator==( const CCharPlayer& cp );
    _KERNEL bool operator>=( const CCharPlayer& cp );

public: // CChar

    // 
    _KERNEL bool isImmortal();

    //   
    _KERNEL bool isMobForcing();

    //   
    _KERNEL bool isPlayerForcing();

    //   
    _KERNEL bool isFriend( CChar* c );

    //   (NULL  )
    _KERNEL CBaseMobPrototype* prot() { return NULL; }

    //  
    _KERNEL bool isMob() { return false; }

    //   
    _KERNEL bool isPlayerOrCharm() { return true; }

    // 
    _KERNEL CCharMob* mentalChild() { return m_mentalChild; }

    //   6 
    _KERNEL const char* realName( int i = 0 ) { return m_name[i]; }

    // 
    _KERNEL const char* realMask() { return m_mask; }

    // 
    _KERNEL const char* realText();

    // 
    _KERNEL const char* realDesc() { return m_desc; }

    // 
    _KERNEL Gender realGender() { return (Gender)m_gender; }

    // 
    _KERNEL int age();

    //   (  )
    _KERNEL int realLevel() { return m_level; }

    //  
    _KERNEL int immInvis() { return m_immInvis; }

    // 
    _KERNEL CProfession* prof() { return m_prof; }

    // 
    _KERNEL CRace* race() { return m_race; }

    //  ,      (    )
    _KERNEL CCharPlayer* playerContext() { return this; }

    //   -
    _KERNEL bool isPK() { return true; }

    // 
    _KERNEL int poltroon( PkFlag pkFlag = APF_NOPK ) { return (int)( maxHP()/100*m_cVars[CMV_POLTROON] ); }

    // 
    _KERNEL double expD() { return m_expD; }

    //    
    _KERNEL double dsuExpD();

    //    
    _KERNEL double usedExpD();

    //    
    _KERNEL double trainExpD();

    //   
    _KERNEL double LostSkillPercD();

    //   
    _KERNEL uint64 money() { return m_money; }
    _KERNEL void setMoney( uint64 m ) { m_money = m; }

    //   
    _KERNEL void bankInc( uint64 m );

    //   
    _KERNEL void bankDec( uint64 m );

    //  -  
    _KERNEL CObjPrototype* corpseProt();

    //  -  
    _KERNEL CObjPrototype* statueProt();

    //  
    _KERNEL bool canMove();

    //  
    _KERNEL bool canChat();

    // 
    _KERNEL uint muted() { return m_account->m_mute; }

    //   
    _KERNEL int usedTrains();

    //   
    _KERNEL int TrainedStat( uint uStat );

    //  
    _KERNEL void ChangeStat( uint nStat, int nVal );

    //  
    _KERNEL bool dropLink() { return m_conn == NULL; }

    //   
    _KERNEL bool inGame() { return m_room != NULL && !m_dropLink; }

    //  
    _KERNEL virtual int resistMax( int type );

    //  ,   
    _KERNEL void ShowDropLink();

    //  
    _KERNEL bool away() { return m_account->m_awayPulse != 0; }

    //  
    _KERNEL const char* awayStr( bool color = true );

    //   
    _KERNEL bool inGuild();

    //   
    bool inHell() { return m_preHellRoom != NULL; }

    //    
    _KERNEL bool defaultClr( int itemID );

    //    
    _KERNEL const char* clr( int itemID );

    //     
    _KERNEL void clrID( int itemID, ColorID& tColor, ColorID& bColor );

    //     
    _KERNEL void setClrID( int itemID, ColorID tColorID, ColorID bColorID );

    //   -
    _KERNEL bool inPkList( CCharPlayer* cp );

    //  
    _KERNEL bool nameAgreed() { return m_agreed == 1; }

    // 
    _KERNEL virtual void CmdPath( const char* args );

public: //  

    //   
    _KERNEL bool realHasWeaponSkill( int i );
    _KERNEL void setHasWeaponSkill( int i, bool has );

    //   
    _KERNEL bool realHasSkill( int i );
    _KERNEL void setHasSkill( int i, bool has );

    //  
    _KERNEL bool realHasSpell( int i );
    _KERNEL void setHasSpell( int i, bool has );

    //    
    _KERNEL bool realHasAnyWeaponSkill();

    //    
    _KERNEL bool realHasAnySkill();

    //   
    _KERNEL bool realHasAnySpell();

public: //  

    //  
    _KERNEL int weaponSkill( int i, PkFlag pkFlag = APF_NOPK );
    _KERNEL void setWeaponSkill( int i, int value ) { m_baseWeapon[i] = value; }

    // 
    _KERNEL int skill( int i, PkFlag pkFlag = APF_NOPK );
    _KERNEL void setSkill( int i, int value ) { m_baseSkill[i] = value; }

    //  
    _KERNEL int skillGrade( int id, PkFlag pkFlag = APF_NOPK );

    // 
    _KERNEL int spell( int i, PkFlag pkFlag = APF_NOPK );
    _KERNEL void setSpell( int i, int value ) { m_baseSpell[i] = value; }

    //    
    _KERNEL virtual int skillHitroll( int i, PkFlag pkFlag = APF_NOPK ) { return weaponSkill( i, pkFlag ); }

    //    
    _KERNEL int skillDamroll( int i, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL int strDamroll( PkFlag pkFlag = APF_NOPK );

    //     
    _KERNEL int percModifier( CChar* c, int power );

public: //    

    //    
    _KERNEL double weaponSkillExpD( int i, byte percent );

    //   
    _KERNEL double skillExpD( int i, byte percent );

    //   
    _KERNEL double spellExpD( int i, byte percent );

public:

    //   
    _KERNEL CDamageType* handDamType();

    //   
    _KERNEL CDamageStr* handDamStr();

    //  
    _KERNEL CWalkMethod* realWalkMethod();

    //   - /
    _KERNEL bool attackIsNear();

    //   - /
    _KERNEL bool weaponIsNear();

    //   
    _KERNEL int primeHitNumb( bool& shot, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL int offHitNumb( PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL int staffHitNumb( PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL int stunPerc( PkFlag pkFlag = APF_NOPK );

    //      
    _KERNEL bool CheckHeavyHand( PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL bool ThrowBlockDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL bool ThrowParryDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL bool ThrowEvadeDice( int hitroll, CChar* hiter, PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL int ThrowDamageDice( CWeaponSkill* wSkill, PkFlag pkFlag = APF_NOPK );

    //   
    _KERNEL bool PerformCustomHit( CChar* c, int knockNum, bool nearFight ) { return false; }

    //   
    _KERNEL bool CheckProtectedRecall();

    //    
    _KERNEL bool CheckStoneCharmer();

    //    
    _KERNEL bool CheckScrollPanacea(BOOL bTry = FALSE);

    //   
    _KERNEL bool inRent() { return !m_room; }

    //   
    _KERNEL bool takeObjects() { return m_cVars[CMV_TAKEOBJS] != 0; }

    //     
    _KERNEL bool resiveToStore() { return m_cVars[CMV_STORE] != 0; }

    //    
    _KERNEL bool ignore( CCharPlayer* cp );

    //      ''
    _KERNEL bool visibleWho( CCharPlayer* player );

    //  
    _KERNEL bool agroFlag() { return m_agroFlagPulse > g_db.m_pulse; }

    //   
    _KERNEL void SetAgroFlag( bool hide = false, bool newFlag = true );

    //   
    _KERNEL void Send( const char* text );

    //    
    _KERNEL void SendWithPages( char* text, bool autoDelete, bool color = true );

    //   
    _KERNEL void SendFormat( const char* text, ... );

    //     
    _KERNEL void SendFormatDmg( int damage, const char* text, ... );

    //    ( ) (  )
    _KERNEL void SendFormatU( const char* text, ... );

    //    ( ) (  )
    _KERNEL void SendFormatUDmg( int damage, const char* text, ... );

    //  ,    
    _KERNEL void SendCharView( CChar* c );

    //  ,    
    _KERNEL void SendAffectView( const char* format, int itemID, const char* defaultClr, PkFlag affFlag );

    //    
    _KERNEL void SendStatus();

    //   
    _KERNEL void CreateFightMsg( byte msgType, char* msg, CChar* hiter, CChar* target, int nCausedDamage = 0, CDamageStr* damStr = NULL );

    //   
    _KERNEL void SendFightMsg( byte msgType, CChar* hiter, CChar* target, int nCausedDamage = 0, CDamageStr* damStr = NULL );

    //    
    _KERNEL void CreateMissStr( int hitType, HitCheck check, char* msg, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, CWeaponSubject* weapon = NULL, int missCount = 0 );

    //    
    _KERNEL void SendMissStr( int hitType, HitCheck check, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, CWeaponSubject* weapon = NULL );

    //    
    _KERNEL void CreateHitStr( int hitType, char* msg, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, int damage, int causedDamage, bool critical, CWeaponSubject* weapon = NULL, int hitCount = 0, int damMin = 0, int damMax = 0 );

    //    
    _KERNEL void SendHitStr( int hitType, CChar* hiter, CChar* target, char* warCryName, CDamageStr* damStr, int damage, int causedDamage, bool critical, CWeaponSubject* weapon = NULL );

    //    
    _KERNEL void ReleaseFightCompress();

    //    
    _KERNEL void SendValue( int i );

    //    
    _KERNEL void SendValue( uint i );

    //    
    _KERNEL void SendValue( uint64 i );

    //    
    _KERNEL void SendValue( double i );

    //    
    _KERNEL void SendValue( const char* str );

    //    
    _KERNEL void SendValue( time_t t );

    //    
    _KERNEL void SendValue( IpAddr ip );

    //   -  
    _KERNEL void DropObject( CSubject* obj );

    //  
    _KERNEL void UpdateStats( PkFlag pkFlag = APF_NOPK );

    //    
    _KERNEL void ApplyEquipMod( int type, CModifier* mod, int value, PkFlag pkFlag, bool fullPk = false );

    //    
    _KERNEL virtual void ApplyEquipMods( PkFlag pkFlag );

    //    
    _KERNEL double AbilCut( double value );

    //  
    _KERNEL void Force( CChar* who, const char* cmd );

    //  
    _KERNEL void CharForce( const char* args );

public: //  

    //     
    _KERNEL void PutIntoMailBox( CLetterSubject* letter );

    //     
    _KERNEL void GetFromMailBox( CLetterSubject* letter );

public: // 

    //    
    _KERNEL void PutIntoStore( CSubject* obj, CCharStoreSection* section = NULL );

    //    
    _KERNEL void GetFromStore( CCharStoreNode* node );

    //    
    _KERNEL static int StoreMax( byte storeRank );

    //    
    _KERNEL static bool CanRaiseStore( int gloryLevel, byte storeRank );

public: //  

    //     
    _KERNEL void EnterIntoGame();

    //   !
    _KERNEL bool CheckNoRentAffects();

    //   !
    _KERNEL bool CheckNoRentObjects();

    // 
    _KERNEL void Rent( const char* args, bool forse = false );

    // 
    _KERNEL void Rerent( const char* args );

    //    
    _KERNEL void RentOnReboot();

    //   !
    _KERNEL void DeleteNoRentAffects();

    //     
    _KERNEL void RentObjects( bool store, bool inv );

    //     
    _KERNEL void UnRentObjects( bool store, bool inv );

    //   
    _KERNEL double CalcRentSumm( bool inrent );

    //   
    _KERNEL void UpdateRentPulse( bool inrent );

    //  
    _KERNEL void QuitFail( const char* args );

    // 
    _KERNEL void Quit( const char* args );

    //    
    _KERNEL void QuitDropObj( CSubject* obj );

    // 
    _KERNEL void Inventory( const char* args );

    //   
    _KERNEL void ShowModeVar( char*& buf, ModeVar mv );

    // 
    _KERNEL void Mode( char* param1, char* param2 );

    //     
    _KERNEL bool NoBotMode( char* param );

    //   
    _KERNEL void ModeMapper( char* param );

    //   
    _KERNEL void ModeGroupSearch( char* param );

    //   
    _KERNEL bool ModeMapperScan( bool set = false );

    //   
    _KERNEL bool anonimeRoom() { return m_room && (m_room->m_flags & RF_ANONIME); }

    //   
    _KERNEL void LookToRoom( bool forceFull = false, bool forceSmall = false );

    //   
    //_KERNEL void LookToObject( CSubject* obj, bool otherChar = false );

    //   
    _KERNEL void List( const char* args, PArray<CCharPlayer*>& list, const char* name, const char* name2, int size, bool readOnly = false );

    //    
    _KERNEL void ListPK( const char* args, PArray<PkInfo*>& list, const char* name, const char* name2, int size, bool readOnly = false );

    // 
    _KERNEL void InvisList( const char* args );

    // 
    _KERNEL void VisList( const char* args );

    // 
    _KERNEL void IgnoreList( const char* args );

    // 
    _KERNEL void TrustList( const char* args );

    // 
    _KERNEL void Snoop( const char* args );

    //  
    _KERNEL void SnoopStop( const char* args );

    // 
    _KERNEL void Bug( const char* args );

    // 
    _KERNEL void Misprint( const char* args );

    // 
    _KERNEL void Idea( const char* args );

    //  
    _KERNEL void Commands( const char* args );

    //  
    _KERNEL void Socials( const char* args );

    // 
    _KERNEL void Ban( const char* args );

    // 
    _KERNEL void Description( const char* args );
    _KERNEL void DescriptionComplete( const char* text, const char* args );

    // 
    _KERNEL void Mail( const char* args );

    //   
    _KERNEL void SendMail( const char* author, const char* text, bool fromChar = false );

    //   
    _KERNEL void CheckMail( CCharPlayer* mess = NULL );

    //    
    _KERNEL void rebootChangeLetter1();

    //  
    _KERNEL void CheckNews();

    //  
    _KERNEL void LetterComplete( const char* text, const char* args );

    //        
    _KERNEL void DeleteByTimerMail( CSubject* obj, ObjDeletePlace odp );

    // 
    _KERNEL void News( const char* args );

    // 
    _KERNEL void Zones( const char* args );

    // 
    _KERNEL void Legend( const char* args );

    //  
    _KERNEL CBoard* GetBoard( CBoardSubject* board );

    // 
    _KERNEL void Write( const char* args );

    //   
    _KERNEL void BoardMessageComplete( const char* text, const char* args );

    // 
    _KERNEL void TakeAway( const char* args );

    // 
    _KERNEL void Read( const char* args );

    // 
    _KERNEL void Away( const char* args );

public: //  

    // 
    _KERNEL void Admin( const char* args );

    // 
    _KERNEL void Echo( const char* args );

    //  
    _KERNEL void LocalEcho( const char* args );

    //  
    _KERNEL void MuteFilter( const char* text );

    //  
    _KERNEL void AdminBanList( const char* args );

    //  
    _KERNEL void AdminAccounts( const char* args );

    //   
    _KERNEL void AccountInfo( CAccount* account, bool forAdmin );

    //    
    _KERNEL void AccountSetPermission( CAccount* account, const char* args );

    // 
    _KERNEL void AdminCharacters( const char* args );

    // 
    _KERNEL void Degeneration();

    //   
    _KERNEL void AdminCharacterInfo( CCharPlayer* c );

    // 
    _KERNEL void AdminMobs( const char* args );

    //     
    _KERNEL CMobPrototype* AdminFindMobProt( char* param );

    //    
    _KERNEL void AdminMobInfo( CMobPrototype* prot );

    // 
    _KERNEL void AdminObjs( const char* args );

    //     
    _KERNEL CObjPrototype* AdminFindObjProt( char* param );

    //    
    _KERNEL void AdminObjInfo( CObjPrototype* prot );

    //  
    _KERNEL void AdminKick( const char* args );

    // 
    _KERNEL void AdminStatistics( const char* args );

    //  
    _KERNEL void AdminRestart( const char* args );

    //  
    _KERNEL void AdminValue( const char* args );

    //  
    _KERNEL void AdminSet( const char* args );

    //  
    _KERNEL void AdminInvisible( const char* args );

    //  
    _KERNEL void AdminGoTo( const char* args );

    //  
    _KERNEL void AdminClans( const char* args );

    //  
    _KERNEL void AdminFractions( const char* args );

    //  
    _KERNEL void AdminArena( const char* args );

    //  
    _KERNEL void AdminTote( const char* args );

    //  
    _KERNEL void AdminLoto( const char* args );

    //  
    _KERNEL void AdminShops( const char* args );

    //   
    _KERNEL void AdminShopSellers( const char* args );

    //   
    _KERNEL void AdminShopGoods( const char* args );

    //  
    _KERNEL void AdminExec( const char* args );

    //  
    _KERNEL void AdminHell( const char* args );

    //  
    _KERNEL void AdminNames( const char* args );

    //  
    _KERNEL void AdminMute( const char* args );

    //  
    _KERNEL void AdminTitle( const char* args );

    //     
    _KERNEL void CharHellInfo( CCharPlayer* c );

    //    
    _KERNEL void PutIntoHell( CCharPlayer* c, int hours, const char* reason );

    //    
    _KERNEL void GetFromHell( CCharPlayer* c );

    //   
    _KERNEL void SendCheckBotString();

    //    
    _KERNEL void InitCheckBot();

    //     
    _KERNEL void InitNoBotCheck( CCharPlayer* cp );

    //    
    _KERNEL bool CanCheckBot( bool nobot = false );

    //    
    _KERNEL void StartCheckBot();

    //     
    _KERNEL void PerformRoundCheckBot();

    //   
    _KERNEL void BotCheckFailed();

    //     
    _KERNEL bool isBotChecking() { return m_botData ? m_botData->m_botChecking : false; }

    //  
    _KERNEL uint getMoveLimit() { return m_botData ? m_botData->m_moveLimit : 0; }
    _KERNEL void setMoveLimit( uint val );

    //  
    _KERNEL uint botCheckPulse() { return m_botData ? m_botData->m_botCheckPulse : 0; }
    _KERNEL uint botWaitPulse() { return m_botData ? m_botData->m_botWaitPulse : 0; }

    //    
    //_KERNEL bool noBotConditions( CRoom* room );

public: // 

    // 
    _KERNEL void Drink( const char* args );

    // 
    _KERNEL void Eat( const char* args );

    // 
    _KERNEL void Compression( const char* args );

    // 
    _KERNEL void Tell( const char* args );

    // 
    _KERNEL void Answer( const char* args );

    //  
    _KERNEL void NoobTell( const char* args );

    // 
    _KERNEL void Group( const char* args );

    //  
    _KERNEL void SortGroup( bool mode );

    //  
    _KERNEL void GroupNumbers( const char* purpose );

    //  
    _KERNEL void GroupLeader( char* purpose );

    //    
    _KERNEL void ShowGroup( CCharPlayer* forWho, PArray<short>& columns, PkFlag pkFlag = APF_NOPK );

    //     
    _KERNEL void ShowGroupChar( CChar* c, PArray<short>& columns, const char* nameFormat, BYTE *spaces, PkFlag pkFlag = APF_NOPK );

    //  
    _KERNEL void PGroup( const char* args );

    //  
    _KERNEL void GroupSearch( const char* args );

    //    
    _KERNEL bool CheckSectionSpace( CStoreSection* sect, CCharPlayer *tMess = NULL );

    // 
    _KERNEL void Score( const char* args );

    // 
    _KERNEL void Account( const char* args );

    // 
    _KERNEL void Invest( const char* args );

    // 
    _KERNEL void Remove( const char* args );

    // 
    _KERNEL void Who( const char* args, bool online );

    // 
    _KERNEL void Where( const char* args );

    // 
    _KERNEL void Area( const char* args );

    // 
    _KERNEL void Bank( const char* args );

    // 
    _KERNEL void Transfer( const char* args );

    // 
    _KERNEL void ClanBank( const char* args );

    // 
    _KERNEL void ClanInvest( const char* args );

    // 
    _KERNEL void ClanRemove( const char* args );

    // 
    _KERNEL void ClanTransfer( const char* args );

    //    
    _KERNEL CCharStoreSection* FindSection( const char* name, bool full = false );

    //  
    _KERNEL void StoreSections( CCharPlayer* cp, const char* args );

    // 
    _KERNEL void Stockage( const char* args );

    // 
    _KERNEL void Store( const char* args );

    // 
    _KERNEL void Withdrew( const char* args );

    // 
    _KERNEL void SendObj( const char* args );

    //   
    _KERNEL bool CheckGetObjs( CCharPlayer* cp );

    //  
    _KERNEL void ClanSection( const char* args );

    // 
    _KERNEL void ClanStockage( const char* args );

    // 
    _KERNEL void ClanStore( const char* args );

    // 
    _KERNEL void ClanWithdrew( const char* args );

    // 
    _KERNEL void ClanSendObj( const char* args );

    // 
    _KERNEL void ClanRent( const char* args );

    // 
    _KERNEL void Exchange( const char* args );

    //  
    _KERNEL void ExchangeExpose( const char* args, bool group, bool clan );

    //  
    _KERNEL void ExchangePrice( const char* args );

    //  
    _KERNEL void ExchangeRemove( const char* args );

    //  
    _KERNEL void ExchangeInfo( const char* args );

    // 
    _KERNEL void ExchangeStats( const char* args );

    //  
    _KERNEL void ExchangeBuy( const char* args );

    //  
    _KERNEL void ExchangeOffers( const char* args );

    //  
    _KERNEL void ExchangeFilter( const char* args );

    //  
    _KERNEL int EvaluateObject( CSubject *tObj );

    //  
    _KERNEL void ExchangeEvaluate( const char *args );

    //  
    _KERNEL void ExchangeAutoExpose( const char *args, bool group, bool clan );

    // 
    _KERNEL void Auction( const char* args );

    // 
    _KERNEL void DonateInvest( const char* args );

    //    
    _KERNEL void OperateSoul( const char* args );

    //  
    _KERNEL void AuctionExpose( const char* args, bool group );

    //  
    _KERNEL void AuctionInfo( const char* args );

    //  
    _KERNEL void AuctionStats( const char* args );

    //  
    _KERNEL void AuctionStake( const char* args );

    //  
    _KERNEL void AuctionOffers( const char* args );

    // 
    _KERNEL void Color( const char* args );

    // 
    _KERNEL void System( const char* args );

    // 
    _KERNEL void Buy( const char* args );

    // 
    _KERNEL void List( const char* args );

    // 
    _KERNEL void LookGoods( const char* args );

    // 
    _KERNEL void Stats( const char* args );

    // 
    _KERNEL void Price( const char* args );

    // 
    _KERNEL void Sell( const char* args );

    // 
    _KERNEL void Levels( const char* args );

    // 
    _KERNEL void Weapons( const char* args );
    _KERNEL void ShowWeapons( char*& buf, bool showExp, bool showInvent, PkFlag pkFlag = APF_NOPK );
    _KERNEL int ShowWeaponsCount();

    // 
    _KERNEL void Skills( const char* args );
    _KERNEL void ShowSkills( char*& buf, bool showExp, bool showInvent, PkFlag pkFlag = APF_NOPK );
    _KERNEL int ShowSkillsCount();

    // 
    _KERNEL void Spells( const char* args );
    _KERNEL void ShowSpells( char*& buf, bool showExp, bool showInvent, PkFlag pkFlag = APF_NOPK );
    _KERNEL int ShowSpellsCount();

    // 
    _KERNEL void Train( const char* args );
    _KERNEL void TrainStat( const char* name, ushort& baseVal, byte raceVal );

    // 
    _KERNEL void Learn( const char* args );

    // 
    _KERNEL void LearnBook( const char* args );

    // 
    _KERNEL void Glory( const char* args );

    //  
    _KERNEL void GloryAbilityShop( const char* args );

    //  
    _KERNEL void GloryUpShop( const char* args );

    //    
    _KERNEL void ClearGloryStats();

    //    
    _KERNEL void AutoAgroModeOn( CCharPlayer* cp );

    // 
    _KERNEL void Saturation( int hours );

    //  
    _KERNEL void RIP( CChar* who = NULL, bool toStone = false, bool noSignsOfDeath = false, PkFlag pkFlag = APF_NOPK, bool dtRip = false );

    //  
    _KERNEL void GainExpD( double exp, double limitK = 1 );

    //  
    _KERNEL double LostExpD( double exp );

    // 
    _KERNEL void Help( const char* args );

    //   
    _KERNEL void ShowTopic( CTopic* topic, char* szFooterText = NULL );

    // 
    _KERNEL void Announces( const char* args );

    //  -
    _KERNEL bool CheckAgro( CChar* c, bool noMsg = false );

    // 
    _KERNEL void Clan( const char* args );

    //  
    _KERNEL void ClanPolitics( const char* args );

    // -
    _KERNEL void ClanSpeak( const char* args, const char* name = NULL );

    // -
    _KERNEL void AllySpeak( const char* args, const char* name = NULL );

    // 
    _KERNEL void Fraction( const char* args );

    // 
    _KERNEL void Tournament( const char* args );

    //   
    _KERNEL void TournamentSchedule( CTrnZone* trnZone, const char* args );

    // 
    _KERNEL void ImmSpeak( const char* args );

    // 
    _KERNEL void Pray( const char* args );

    // 
    _KERNEL void Remember( const char* args );

    // 
    _KERNEL void Journal( const char* args );

    // 
    _KERNEL void Records( const char* args );

    // 
    _KERNEL void GAffects( const char* args );

    // 
    _KERNEL void Columns( const char* args );

    // 
    _KERNEL void Grind( const char* args );

    // 
    _KERNEL void SetSpec( const char* args );

    // 
    _KERNEL void Tote( const char* args );

    // 
    _KERNEL void Loto( const char* args );

    // 
    _KERNEL void Interview( const char* args );

    // 
    _KERNEL void Status( const char* args );

    // 
    _KERNEL void Home( const char* args );
    _KERNEL void HomeDescriptionComplete( const char* text, const char* args );
    _KERNEL void CreateHomeRoom( int roomId, const char* desc, byte dir, CRoom* exitRoom );

    // 
    _KERNEL void ShowQuests( const char* args );
    //  
    _KERNEL CQuestStatus* FindQuest( int questId );

    //  
    _KERNEL void ShowDailyTasks( const char* args );

    // 
    _KERNEL void Poison( const char* args );

    // 
    _KERNEL void Ungrind( const char* args );

    //  
    _KERNEL void Assemble( const char* args );

    //  
    _KERNEL void Insert( const char* args );

    //  
    _KERNEL void Extract( const char* args );

    // 
    _KERNEL void Scatter( const char* args );
    _KERNEL void ScatterUniq( CSubject* obj, int power );
    _KERNEL void ScatterObject( CSubject* obj, int power );

    // 
    _KERNEL void Staff( const char* args );

    // 
    _KERNEL virtual void Shelter( const char* args );
    _KERNEL void DispelShelter();
    _KERNEL void AddShelterZone();

    //    
    _KERNEL virtual void ClearInstReference( CZoneInstance* zoneInst );

    //    
    _KERNEL void ClearMobReference( CCharMob* mob );

    //   
    _KERNEL void SetHerald( int nDay, int nMonth, int nYear );
    _KERNEL void ResetHerald();

public: //  

    //   
    _KERNEL void MaxTableAdd( CMobPrototype* prot );

    //    
    _KERNEL int MaxTableGet( CBaseMobPrototype* prot );

    //   (  )
    _KERNEL void Unmax();

public: //  

    //  
    _KERNEL void PerformPulse( PkFlag pkFlag = APF_NOPK );

public: // CDBObject

    //   
    _KERNEL bool GetField( char* field, char* param, CCharPlayer* cp );

    //   
    _KERNEL bool SetField( char* field, char* param, const char* text );

public: //  

    //     
    _KERNEL void SetMentalChild( CCharMob* mob, int charmSpell, int charmControl, int mobPkProtId = 0 );

    //   
    _KERNEL void ClearMentalChild();

    //   
    CCharZoneProgress* FindZoneProgress( int nZoneId );

    //   
    void AddZoneProgress( CZoneInstance* inst );

public:

    //  
    void StartEquip( Array<int>& equip );

public: //  

    //  
    _KERNEL void ProcessTarget( char* purpose );

    //  
    _KERNEL void CreateStatus( CString* status, const char* statusStr );

    //   
    _KERNEL void ProcessStatusVar( char*& buf, const char*& var );

    //    
    _KERNEL void ShowFightStatus( char*& buf, CChar* c, PkFlag pkFlag = APF_NOPK );

public: //   

    _KERNEL void ImprisonSoul(); //  
    _KERNEL void RebirthSoul();  // 
    _KERNEL int CalcInvents( int invent, BookType bType, int num );  //  

    //  
    _KERNEL void ResetStats();

    //  
    _KERNEL void ResetAbilities();

    //  
    _KERNEL void ResetInvent();

    //  
    _KERNEL void ResetGlory();

    //  
    _KERNEL void ResetFull();

public: //   

    _KERNEL void CmdDTList( const char* args ); // 
    _KERNEL void CmdDTBuy( const char* args );  // 
    _KERNEL void CmdDTInfo( const char* args ); // 
    _KERNEL void CmdDTLore( const char* args ); // 

public: // 

    int     m_id;                        // 
    char    m_name[6][CHAR_NAME_SIZE+1]; //    
    char    m_mask[CHAR_NAME_SIZE+1];    // 
    CString m_title;                     // 
    CCharPlayer *m_titleSet;             //   
    CString m_desc;                      //  
    byte    m_gender;                    // 
    byte    m_agreed;                    //   
    uint    m_birthPulse;                //  
    uint    m_freezePulse;               // ,   

    time_t m_lastIn;    //   
    time_t m_lastOut;   //   
    uint   m_lastPulse; //   

    CClan* m_clan;   //   
    byte m_rank;     //   
    short m_getObjs; //  

    byte m_level;    //  (   m_exp  m_immLevel)
    byte m_immLevel; //   (  - )
    byte m_immInvis; //   
    double m_expD;   //  

    byte m_maxLevel;  // ,     
    int m_trains;     //  
    int m_overTrains; //  

    int m_allRipCount;   //  
    double m_expLost;    //   
    CString m_ripReason; //   

    CProfession* m_prof;    // 
    CRace*       m_race;    // 
    byte         m_align;   // 

    uint64 m_money;  //   
    uint64 m_bank;   //   

    int m_food;  //  
    int m_water; //  

    byte m_hasWeaponSkill[2]; //      ( )
    byte m_baseWeapon[12];    //   

    byte m_hasSkill[SKILL_BYTE_COUNT]; //     ( )
    byte m_baseSkill[SKILL_MAX_COUNT];  //  

    byte m_hasSpell[SPELL_BYTE_COUNT]; //     ( )
    byte m_baseSpell[SPELL_MAX_COUNT];  //  

    byte m_soulType;         //  
    CCharPlayer *m_soulLink; //  
    uint m_soulPulse;        //   
    short m_rebirthNum;      //  
    bool m_rebirthExist;     //  
    short m_rebirthStore;    //  
    uint m_rebirthPulse;     //  

    int m_inventNum;                     // : ,   
    int m_inventRemain;                  // :  
    int m_inventTotal;                   // :  ( )

    byte m_inventWeapon[WS_COUNT];       //    
    byte m_inventSkill[SKILL_MAX_COUNT]; //    
    byte m_inventSpell[SPELL_MAX_COUNT]; //    

    byte m_remainWeapon[WS_COUNT];       //    
    byte m_remainSkill[SKILL_MAX_COUNT]; //    
    byte m_remainSpell[SPELL_MAX_COUNT]; //    

    CGloryStats* m_gloryStats;           //   

    PArray<CCharStoreSection*> m_sections; //   
    CCharStoreNode* m_storeList; //   
    int m_receiveCount;     //  
    byte m_storeRank;       //  

    //  
    ushort m_baseStr;     // 
    ushort m_baseDex;     // 
    ushort m_baseInt;     // 
    ushort m_baseWiz;     // 
    ushort m_baseCon;     // 
    byte m_baseSize;    // 

    CAccount* m_account;  //  
    int m_accId;          // Id  
    CRoom* m_homeRoom;    // 
    bool m_dropLink;      //   

    //byte m_hotel;         //  
    CRoom* m_rentRoom;    //  
    double m_rentSumm;    //  
    int m_rentMoney;      //    
    uint m_rentPulse;     //  

    CZone* m_tempRent;      //     
    CRoom* m_returnRoom;    //    

    CRoom* m_preHellRoom;   // ,      
    uint m_hell;            // ,     
    CCharPlayer* m_hellImm; //    
    CString m_reason;       // 

    CCharPlayer* m_agreedImm; //   

    CConnection* m_conn;  //  

    PArray<CCharPlayer*> m_invisList;  //  ,    
    PArray<CCharPlayer*> m_visList;    //  ,    
    PArray<CCharPlayer*> m_ignoreList; //  
    PArray<CCharPlayer*> m_trustList;  //  

    PArray<CCharPlayer*> m_grpTrustList;  //   
    uint m_grpTrustPulse;                 // ,     

    CLetterSubject* m_letters; //  

    CCharPlayer* m_lastTell; //  

    CObjPrototype* m_corpseProt; //  -
    CObjPrototype* m_statueProt; //  -

    PArray<uint> m_maxTable; //    (3 - id , 1  -  )

    CCharMob* m_mentalChild;  // ,     
    uint m_mentalPulse;       // ,     
    int m_charmSpell;         //   
    int m_charmControl;       //  

    uint m_agroFlagPulse;     // ,     
    uint m_agroStartPulse;    //  
    bool m_agroHide;          //   

    CJournal m_tellsJournal;  //   

    PArray<short> m_columns;        //   ''

    CString m_status;               //  
    CString m_statuses[AMT_COUNT];  //  
    bool m_bothParam[3];            //   

    byte m_cVars[CMV_COUNT];        //   
    int m_exchangeFilter;           //    
    PArray<byte> m_exchangeTypes;   //    

    CInvisibleEyeMob* m_eye[3];     //  

    PArray<CChar*> m_nums;          //    

    bool m_allyBoard;               //   

    CCharPlayer* m_letterAuthor;    //     

    int m_aukSigns;                 //    
    int m_lotoNum;                  //   

    PArray<CQuestStatus*> m_quests; //   
    PArray<CDailyTaskProgress*> m_dailyTasks; // 
    PArray<CCharZoneProgress*> m_zoneProgress; //  

    double m_glory;                 // 
    double m_totalGlory;            //  
    byte m_hallGloryLevel;          //    
    uint m_heraldPulse;             //  
    SYSTEMTIME* m_pHeraldTime;      //   

    PArray<CDTSubject*> m_dtList;   //    

    CFightBuffer* m_fightBuf;       //   

    CFraction* m_fraction;          // 
    byte m_fractionRank;            //   

    int m_donateToExp;              //     
    double m_donateMax;             //  
    double m_donateExp;             //  
    int m_donateToMoney;            //    
    int m_donateMoney;              //  
    int m_donateToIngr;             //  
    int m_donateIngr;               //  
    int m_donateHeraldDays;         //   
    int m_donateToHerald;           //   

    uint m_instPulse;               //   

    uint m_fightPoints;             //    

    uint m_moves[DAY_HOURS];        //   
    bool m_hasMoves;                //  

    uint m_moveCount;               //  
    CBotCheckData* m_botData;       //    

    //   
    static short m_equipMods[MDT_COUNT][EMT_COUNT];
};
