/*  

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#include "DB/Subject/Subject.h"
#include "DB/Board.h"
#include "DB/Journal.h"

//   -
class PkInfo
{

public: // 

	PkInfo( CCharPlayer* cp, const char* reason ) :
		m_char( cp ),
		m_reason( reason )
	{}

	~PkInfo() {}

public: // 

	CCharPlayer* m_char; // 
	CString m_reason;    // 

};

//    
class CGetObjInfo
{

public: // 

	CGetObjInfo( CCharPlayer* cp, CSubject* obj, uint pulse ) :
		m_char( cp ), m_obj( obj ), m_pulse( pulse )
	{}

	~CGetObjInfo() {}

public: // 

	CCharPlayer* m_char;  // 
	CSubject*    m_obj;   // 
	uint         m_pulse; // 

};

//  
class CStoreSection
{

public: // 

	_KERNEL CStoreSection() :
		m_journal( CI_WEATHER, 50 )
	{}

	_KERNEL CStoreSection( int id, const char *name, int min, int max, bool mode ) :
		m_id( id ),
		m_minSize( min ),
		m_maxSize( max ),
		m_slotCount( min ),
		m_itemCount( 0 ),
		m_mode( mode ),
		m_rent( 0 ),
		m_rank( 3 ),
		m_journal( CI_WEATHER, 50 ),
		m_pass( NULL )
	{
		strcpy( m_name, name );
	}

	~CStoreSection() {}

public: // 

    //   
	_KERNEL void Serialize( CFileStore &fs, CClan* clan );

    //  
	_KERNEL void ClearCharReference( CCharPlayer *tChar );

public: // 

	int m_id;                        // 
	char m_name[MAX_SECTION_NAME+1]; // 

	CString* m_pass;                // 

	int m_minSize;                  //  
	int m_maxSize;                  //  
	int m_slotCount;                //  
	int m_itemCount;                //  

	int m_rent;                     //  

	bool m_mode;                    //  
	int m_rank;                     //  
	PArray<CCharPlayer*> m_list;    //  
	PArray<CGetObjInfo*> m_getObjs; //  

	CJournal m_journal;             //  

};

//   
class CStoreNode
{

public: // 

	_KERNEL CStoreNode( CStoreSection *ss, CSubject *obj ) :
		m_sect( ss ),
		m_obj( obj )
	{}

	_KERNEL CStoreNode()
	{}

	~CStoreNode() {}

public: // 

	CSubject*      m_obj;  // 
	CStoreSection* m_sect; // 
	CStoreNode*    m_next; //  

};

// 
class CClan
{

public: // 

	_KERNEL CClan();

	~CClan() {}

public: //   

	//  
	_KERNEL void InitNew();

	//   
	_KERNEL void Serialize( CFileStore& fs );

	//  
	_KERNEL void Link();

	//  
	_KERNEL void PerformPulse();

public: // 

    //  
    _KERNEL static int StoreMax( byte storeRank );

    //     
    _KERNEL static bool CanRaise( int gloryLevel, byte storeRank );

	//    
	_KERNEL void LoadToStore( CStoreSection* sect, CSubject* obj );

	//    
	_KERNEL void PutIntoStore( CStoreSection* sect, CSubject* obj );

	//    
	_KERNEL void GetFromStore( CStoreNode* tNode );

	//     
	_KERNEL void RentObjects( int hotel );

	//     
	_KERNEL void UnRentObjects();

	//  
	_KERNEL CStoreSection* FindSection( CMaskManager& mm );

public: // 

	//    ,   
	_KERNEL bool inWars( const CCharPlayer* c ) const;

	//    ,    
	_KERNEL bool inAllies( const CCharPlayer* c ) const;

	//  
	_KERNEL const char* politicStr( CClan* clan ) const;

public: // 

	bool operator==( const CClan& c ) const { return ::strcmp( m_name, c.m_name ) == 0; }
	bool operator>=( const CClan& c ) const { return ::strcmp( m_name, c.m_name ) >= 0; }

public: // 

	int  m_id;                     // 
	char m_name[MAX_CLAN_NAME+1];  // 
	char m_mask[MAX_CLAN_NAME+1];  // 

	char m_rankName[5][MAX_CLAN_RANKNAME+1]; //  
	short m_rankAccessTime[5];               //   
	short m_rankObjLimit[5];                 //    
	byte m_getObjTime;                       //   

	CCharPlayer*    m_leader;  //  
	PArray<PkInfo*> m_pkList;  // -

	PArray<CClan*> m_wars;     // ,   
	PArray<CClan*> m_allies;   // ,    

	PArray<CStoreSection*> m_sections; //   
	CStoreNode *m_storeList;           //   
    byte m_storeRank;                  //  

	CBoard m_board;         //  
	int m_bank;             // 
	int m_hotel;            //  

	double m_rentSumm;      //  
	uint m_rentPulse;       //  

	CJournal m_journalSpeak;  //   
	CJournal m_journalStore;  //   
	CJournal m_journalBank;   //   

	CCharPlayer* m_owner; //  

	int m_allMembers;     //   
	double m_allExp;      //    ( )

	int m_count;          //   

};
