/*   

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#include "DB/Char/ProfAI.h"
#include "DB/Char/CustomAction.h"
#include "System/TriggerMng.h"

//   
_KERNEL int GradeToPower( int grade );
//   
_KERNEL int PowerToGrade( int power );

//  
class CMobAffectData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs );

public: // 

	byte m_id;      // 
	int  m_power;   // 

};

//   
class CMobAffectRandData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs );

public: // 

	byte m_id;      // 
	int m_power;    // 
	byte m_percent; // 

};

//   
class CMobSkillData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs );

public: // 

	byte m_id;    // 
	byte m_value; // 

};

//   
class CMobSpellData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs );

public: // 

	byte m_id;    // 
	byte m_value; // 

};

//    
class CObjSellData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs, int zoneID );

	//  
	_KERNEL void Link();

public: // 

	CObjPrototype* m_prot;   //  
	int m_price;             // 

};

//  
class CMobFactory
{

public: //   

	CMobFactory( int instIdx, CCharMob* (*factory)( CMobPrototype*, CZoneInstance*, CMobPopData* ) ) :
		m_instIdx( instIdx ), m_factory( factory )
	{}

	~CMobFactory() {}

public: // 

	int m_instIdx; //  

	// 
	CCharMob* (*m_factory)( CMobPrototype*, CZoneInstance*, CMobPopData* );

};

//   
class CBaseMobPrototype
{

public: // 

    // 
    _KERNEL virtual int id() = 0;

    // 
    _KERNEL virtual byte level() = 0;

	//  
	_KERNEL virtual CMobPrototype* prot() = 0;

    // 
    _KERNEL virtual byte complex() = 0;

    // 
    _KERNEL virtual int hp() = 0;

    // 
    _KERNEL virtual int mana() = 0;

    // 
    _KERNEL virtual int mv() = 0;

    // 
    _KERNEL virtual byte StatStr() = 0;

    // 
    _KERNEL virtual byte StatInt() = 0;

    // 
    _KERNEL virtual byte StatWiz() = 0;

    // 
    _KERNEL virtual byte StatDex() = 0;

    // 
    _KERNEL virtual byte StatCon() = 0;

    // 
    _KERNEL virtual int hitroll() = 0;

    // 
    _KERNEL virtual int damroll() = 0;

    //  
    _KERNEL virtual byte hitNumb() = 0;

    //  
    _KERNEL virtual byte damDice() = 0;

    // 
    _KERNEL virtual int flags() = 0;

    //   
    _KERNEL virtual int exp() = 0;

    //  
    _KERNEL virtual double moneyK() = 0;

    // 
    _KERNEL virtual byte poison() = 0;

    //  
	_KERNEL virtual byte oldBookPerc() = 0;

    //   
	_KERNEL virtual int symbolPerc() = 0;

    //   
	_KERNEL virtual int reagPerc() = 0;

    // 
    _KERNEL virtual short resist( int type ) = 0;

    //  
    _KERNEL virtual bool hasSkill( int id ) = 0;

	//    
	_KERNEL virtual bool realHasAnySkill() = 0;

	// 
	_KERNEL virtual int skill( int id ) = 0;

    //  
    _KERNEL virtual bool hasSpell( int id ) = 0;

	//   
	_KERNEL virtual bool realHasAnySpell() = 0;

	// 
	_KERNEL virtual int spell( int id ) = 0;

	//  
	_KERNEL virtual int specRes( int sr ) = 0;

    //   
    _KERNEL virtual bool hasAffects() = 0;

    //  
	_KERNEL virtual CMobAffectData* getAffect( int i ) = 0;

    //   
	_KERNEL virtual int getAffectPower( int i ) = 0;

    //   
    _KERNEL virtual bool hasRandAffects() = 0;

    //  
	_KERNEL virtual CMobAffectRandData* getRandAffect( int i ) = 0;

    //   
	_KERNEL virtual int getRandAffectPower( int i ) = 0;

    //   AI
    _KERNEL virtual ushort aiFlags() = 0;

};

//  
class CMobPrototype : public CBaseMobPrototype
{

public: // 

	_KERNEL CMobPrototype();

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs, int zoneID );

	//  
	_KERNEL void Link();

public: // 

    // 
    _KERNEL int id() { return m_id; }

	//  
	_KERNEL CMobPrototype* prot() { return this; }

    // 
    _KERNEL byte complex() { return 0; }

	// - 
	_KERNEL int handler( CMobTriggerData* data ) { return m_handler( data ); }

	//    
	_KERNEL bool realHasAnySkill() { return m_skills != 0; }

	//    
	_KERNEL bool realHasAnySpell() { return m_spells != 0; }

	//  
	_KERNEL int specRes( int sr ) { return m_specResist ? m_specResist[sr] : 0; }

    // 
    _KERNEL int hp() { return m_hp; }

    // 
    _KERNEL int mana() { return m_mana; }

    // 
    _KERNEL int mv() { return m_mv; }

    // 
    _KERNEL byte StatStr() { return m_str; }

    // 
    _KERNEL byte StatInt() { return m_int; }

    // 
    _KERNEL byte StatWiz() { return m_wiz; }

    // 
    _KERNEL byte StatDex() { return m_dex; }

    // 
    _KERNEL byte StatCon() { return m_con; }

    // 
    _KERNEL int hitroll() { return m_hitroll; }

    // 
    _KERNEL int damroll() { return m_damroll; }

    //  
    _KERNEL byte hitNumb() { return m_hitNumb; }

    //  
    _KERNEL byte damDice() { return m_dam[0]; }

    // 
    _KERNEL int flags() { return m_flags; }

    // 
    _KERNEL short resist( int type ) { return m_resist[type]; }

    // 
    _KERNEL byte poison() { return m_poison; }

    // 
    _KERNEL byte level() { return m_level; }

    //   
    _KERNEL int exp() { return m_exp; }

    //  
	_KERNEL byte oldBookPerc() { return m_oldBookPerc; }

	//  -  
	_KERNEL CObjPrototype* corpseProt();

	//  -  
	_KERNEL CObjPrototype* statueProt();

	//   
	_KERNEL bool hasSkill( int id );

	//  
	_KERNEL bool hasSpell( int id );

	// 
	_KERNEL int skill( int id );

	// 
	_KERNEL int spell( int id );

    //   
    _KERNEL bool hasAffects() { return m_affects != 0; }

    //  
	_KERNEL CMobAffectData* getAffect( int i ) { return m_affects ? &m_affects[i] : 0; }

    //   
	_KERNEL int getAffectPower( int i ) { return m_affects ? m_affects[i].m_power : 0; }

    //   
    _KERNEL bool hasRandAffects() { return m_affectsRand != 0; }

    //  
	_KERNEL CMobAffectRandData* getRandAffect( int i ) { return m_affectsRand ? &m_affectsRand[i] : 0; }

    //   
	_KERNEL int getRandAffectPower( int i ) { return m_affectsRand ? m_affectsRand[i].m_power : 0; }

    //   
	_KERNEL int symbolPerc() { return m_symbolPerc; }

    //   
	_KERNEL int reagPerc() { return m_reagPerc; }

    //  
    _KERNEL double moneyK() { return 1; }

    //   AI
    _KERNEL ushort aiFlags() { return m_aiFlags; }

    //  
    _KERNEL void Weaken( CWeakenParams* params );

	//   
	double ApplyPower( int version, double value, double power, int type );

	//   
	double ApplyBoss( int version, double value, int boss, int bType );

	//   
	_KERNEL int ApplyHard( int version, double value, double power, int pType, int boss, int bType );

	// 
	Gender gender() const { return (Gender)m_gender; }

	//  
	_KERNEL CProfAI* profAI() { return m_profAI ? m_profAI : m_prof->m_profAI; }

	//   
	_KERNEL void AddMobFactory( int instIdx, CCharMob* (*factory)( CMobPrototype*, CZoneInstance*, CMobPopData* ) );

    //    AI
    _KERNEL void SetAIFlags();

    //   
    _KERNEL void AddCustomAbility( CCustomAbility* abil );

public: // 

	int          m_id;           // 

	CString      m_name[6];      //    
	CString      m_mask;         // 
	CString      m_text;         //  
	CString      m_desc;         //  
	byte         m_gender;       // 
	byte         m_pos;          // 

	byte         m_walk;         // 
	CWalkMethod* m_walkMethod;   //  
	byte         m_poltroonPerc; //  %  

	CString      m_greetings;    // 

	byte         m_level;        // 
	CProfession* m_prof;         // 
	CRace*       m_race;         // 
	byte         m_align;        // 

	int          m_exp;          // 
	byte         m_max;          // 
	int          m_mon[3];       //   A+BdC
	int          m_flags;        // 

	int          m_power;        // 
	int          m_boss;         //  

	byte         m_str;          // 
	byte         m_dex;          // 
	byte         m_int;          // 
	byte         m_wiz;          // 
	byte         m_con;          // 

	byte         m_weight;       // 
	byte         m_oldBookPerc;  //  
	byte         m_size;         // 

	byte         m_reagType;     //  
	byte         m_reagSize;     //  
	int          m_reagId;       // Id 
	int          m_reagPerc;     //   

	byte         m_symbolSize;   //  
	int          m_symbolPerc;   //   

	int          m_hp;           // 
	int          m_mana;         // 
	int          m_mv;           // 

	short        m_hitroll;      // 
	short        m_damroll;      // 
	byte         m_dam[2];       //  

	byte         m_poison;       //  

	CDamageType* m_damType;      //  
	CDamageStr*  m_damStr;       //  
	byte         m_hitNumb;      //  

	short m_resist[RESIST_COUNT]; // 

	short*       m_specResist;   //  

	CMobAffectData*     m_affects;     //  
	CMobAffectRandData* m_affectsRand; //  
	bool                m_onlyOne;     //    

	CMobSkillData* m_skills;     // 
	CMobSpellData* m_spells;     // 

	CObjSellData* m_goods;       // 
	int           m_goodsCount;

	int m_handlerFlags;                        //  
	int (*m_handler)( CMobTriggerData* data ); // - 

	CTriggerHandler* m_trgs;     // 

	PArray<CQuestStatus*> m_questStats; //  

	int m_inGame; //      

	CObjPrototype* m_corpseProt; //  -
	CObjPrototype* m_statueProt; //  -

	//   (  AI)
	PArray<CMobFactory*>* m_factories;

	//  ,   
	PArray<CMobPrototype*> m_enemyMobs;

    //  
    PArray<CCustomAbility*>* m_customAbilities;

	int    m_playerRips;         //   
	double m_playerExp;          //   

	PArray<CActionData*>* m_actions; //  
	bool m_hasHits;                  //   

    ushort m_aiFlags;  //   AI

	CProfAI* m_profAI; //  

    bool m_noWeaken;   //  

};
