/*  

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once

// 
class CSpell
{

public: // 

	_KERNEL CSpell( int id, const char* name, PKType pkType, bool fight, MagicSphere sphere, int mana, bool (CChar::*handler) ( const CSpell*, int, CastType, PkFlag, const char* ), SpecialResist sr = SR_NONE );

public: // 

	//  
	_KERNEL static void InitTable( CSpell* spells[] );

	//    
	//_KERNEL int mana( int power ) const { return power > 300 ? 0 : m_mana - m_mana*power/300; }
	_KERNEL int mana( int power ) const { return m_mana + m_mana*power/300; }

public: // 

	int m_id;            // 
	const char* m_name;  //  
	const char* m_small; //     
	const char* m_mask;  // 
	int m_profFlag;      //   

	byte m_pkType;       // K-
	bool m_fight;        //  

	byte m_sphere;       //  
	int m_mana;          //  

	short m_resist;      //  

	// -
	bool (CChar::*m_handler) ( const CSpell* spell, int power, CastType castType, PkFlag, const char* args );

};

//    
class CBaseFragSpell : public CSpell
{

public: // 

	_KERNEL CBaseFragSpell( int id, const char* name, MagicSphere sphere, int mana, int dice0, int dice1, bool (CChar::*handler) ( const CSpell*, int, CastType, PkFlag, const char* ), SpecialResist sr = SR_NONE );

public:

	//   
	_KERNEL virtual void Show( CChar* caster, CChar* target, bool critical, int damage = 0 ) = 0;

	//      
	_KERNEL virtual bool Influence( CChar* target ) { return true; }

	//    
	_KERNEL static const char* spellClr( CChar* toChar, CChar* caster, CChar* target, bool critical );

public: // 

	int m_dice[2];   //  

};

// -
class CFragSpell : public CBaseFragSpell
{

public: // 

	_KERNEL CFragSpell( int id, const char* name, MagicSphere sphere, int mana, int dice0, int dice1, SpecialResist sr = SR_NONE );

public:

	//   
	_KERNEL virtual void ApplyToTarget( CChar* caster, CChar* target, CastType castType, int power, PkFlag pkFlag = APF_NOPK ) {}

};

//   
class CMassFragSpell : public CBaseFragSpell
{

public: // 

	_KERNEL CMassFragSpell( int id, const char* name, MagicSphere sphere, int mana, int dice0, int dice1, SpecialResist sr = SR_NONE );

public:

	//    
	_KERNEL virtual void ShowToMe( CChar* caster, bool critical ) = 0;

	//   
	_KERNEL virtual void ApplyToTarget( CChar* caster, CChar* target, CastType castType, int power, PkFlag pkFlag = APF_NOPK ) {}

};

//  - 
class CDrainSpell : public CBaseFragSpell
{

public: // 

	_KERNEL CDrainSpell( int id, const char* name, MagicSphere sphere, int mana, int dice0, int dice1, SpecialResist sr = SR_NONE );

public:

	//   
	_KERNEL virtual void ApplyToTarget( CChar* caster, CChar* target, int damage, PkFlag pkFlag = APF_NOPK ) = 0;

};

//  
class CChainLightingSpell : public CBaseFragSpell
{

public: // 

	CChainLightingSpell();

public:

	//   
	void Show( CChar* caster, CChar* target, bool critical, int damage = 0 );

	//   
	void ApplyToTarget( CChar* caster, CChar* target, CastType castType, int power, PkFlag pkFlag );

};

// -
class CAffectSpell : public CSpell
{

public: // 

	_KERNEL CAffectSpell( int id, const char* name, PKType pkType, MagicSphere sphere, int mana, AffectID affectID, int durationK, bool onlyGroup, SpecialResist sr = SR_NONE, IngrColorID ingrColor = INGR_COLOR_NONE, int lagPulses = 0 );

public: // 

    //   
    _KERNEL virtual void ApplyAffect( CChar* caster, CChar* target, int power, PkFlag pkFlag ) {}

public: // 

	CAffect* m_affect; // 
	int m_durationK;   //   (  1% )
	bool m_onlyGroup;  //    
	byte m_ingrColor;  //  
	int m_lagPulses;   //   

};

//  " "
class CSeeMagicAffectSpell : public CAffectSpell
{

public: // 

	_KERNEL CSeeMagicAffectSpell() :
        CAffectSpell( SPELL_SEE_MAGIC, " ", PK_NONE, MS_ASTRAL, 6, AFF_SEE_MAGIC, 150, false, SR_SPELL_SEE_MAGIC )
    {}

public: // 

    //   
    _KERNEL virtual void ApplyAffect( CChar* caster, CChar* target, int power, PkFlag pkFlag );

};

// - 
class CMassAffectSpell : public CSpell
{

public: // 

	_KERNEL CMassAffectSpell( int id, const char* name, PKType pkType, MagicSphere sphere, int mana, AffectID affectID, int durationK, IngrColorID ingrColor = INGR_COLOR_NONE, int lagPulses = 0, SpecialResist sr = SR_NONE );

public: // 

	CAffect* m_affect; // 
	int m_durationK;   //   (  1% )
	byte m_ingrColor;  //  
	int m_lagPulses;   //   

};

//   
class CAgroAffectSpell : public CSpell
{

public: // 

	_KERNEL CAgroAffectSpell( int id, const char* name, MagicSphere sphere, int mana, AffectID affectID, int durationK, SpecialResist sr = SR_NONE );

public: // 

	//  
	_KERNEL virtual int percent( CChar* who, CChar* target, int power, PkFlag pkFlag ) = 0;

public: // 

	CAffect* m_affect; // 
	int m_durationK;   //   (  1% )

};

//    
class CMassAgroAffectSpell : public CSpell
{

public: // 

	_KERNEL CMassAgroAffectSpell( int id, const char* name, MagicSphere sphere, int mana, AffectID affectID, int durationK, SpecialResist sr = SR_NONE );

public: // 

	//  
	_KERNEL virtual int percent( CChar* who, CChar* target, int power, PkFlag pkFlag ) = 0;

public: // 

	CAffect* m_affect; // 
	int m_durationK;   //   (  1% )

};

//  -
class CBaseHealSpell : public CSpell
{

public: // 

	_KERNEL CBaseHealSpell( int id, const char* name, int mana, int dice0, int dice1, bool (CChar::*handler) ( const CSpell*, int, CastType, PkFlag, const char* ), SpecialResist sr = SR_NONE );

public: // 

	//     
	int healHP( CChar* target, int power, PkFlag pkFlag = APF_NOPK ) const;

public: // 

	int m_dice[2];  //  

};

// -
class CHealSpell : public CBaseHealSpell
{

public: // 

	_KERNEL CHealSpell( int id, const char* name, int mana, int dice0, int dice1, SpecialResist sr = SR_NONE );

};

// - 
class CMassHealSpell : public CBaseHealSpell
{

public: // 

	_KERNEL CMassHealSpell( int id, const char* name, int mana, int dice0, int dice1, SpecialResist sr = SR_NONE );

};

//  -
class CBaseHoldSpell : public CSpell
{

public: // 

	_KERNEL CBaseHoldSpell( int id, const char* name, int mana, bool (CChar::*handler) ( const CSpell*, int, CastType, PkFlag, const char* ), SpecialResist sr = SR_NONE );

public: // 

	//     
	virtual double HoldPower( CChar* c ) const = 0;

};

// -
class CHoldSpell : public CBaseHoldSpell
{

public: // 

	_KERNEL CHoldSpell( int id, const char* name, int mana, SpecialResist sr = SR_NONE );

};

// -
class CMassHoldSpell : public CBaseHoldSpell
{

public: // 

	_KERNEL CMassHoldSpell( int id, const char* name, int mana, SpecialResist sr = SR_NONE );

};
