/*  

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#include "DB/Char/Char.h"

//      
class CObjPopOnMobData
{

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs, int zoneID );

	//  
	_KERNEL void Link();

public: // 

	CObjPrototype* m_objProt; //  
	byte m_slot;              // 
	byte m_percent;           //  

};

//      
class CObjPopInContData
{

public: //   

	//  
	_KERNEL void Link();

public: // 

	CObjPrototype* m_objProt; //  
	byte m_percent;           //  

};

//    
class CObjPopInRoomData
{

public: //   

	CObjPopInRoomData() :
		m_data( NULL ), m_obj( NULL )
	{}

	~CObjPopInRoomData();

public: // 

	CObjPopInRoomBaseData* m_data; //  

	CSubject* m_obj; //  

};

//    
class CMobPopData
{

public: //   

	// 
	_KERNEL CMobPopData();

	// 
	_KERNEL ~CMobPopData();

public: //   

	//   
	_KERNEL void Serialize( CFileStore& fs, int zoneID );

	//  
	_KERNEL void Link();

	//    
	_KERNEL void LinkKeys();

public: // 

	CMobPopBaseData* m_data; //  

	CObjPopInRoomData** m_objLinks; //   
	int m_objLinkCount;             //  

	CCharMob* m_mob;  //  
	CRoom* m_room;    //  

    bool m_multiply;  //  

	PArray<CCharAddon*> m_addons; //    

	CSubject* m_savedObjList; //    

};

// 
class CExit
{

public: //   

	//  
	_KERNEL void Link( CRoom* room );

public: // 

	//  
	_KERNEL bool closed() const { return (m_flags & EF_DOOR_CLOSED) != 0; }
	_KERNEL void setClosed( bool val ) { if( val ) m_flags |= EF_DOOR_CLOSED; else { m_flags &= ~EF_DOOR_CLOSED; setLocked( false ); } }

	//  
	_KERNEL bool locked() const { return (m_flags & EF_DOOR_LOCKED) != 0; }
	_KERNEL void setLocked( bool val ) { if( val ) m_flags |= EF_DOOR_LOCKED; else m_flags &= ~EF_DOOR_LOCKED; }

	//   
	_KERNEL bool canLock() const { return m_keyProt != NULL; }

	//    
	_KERNEL bool repopLock() const { return !(m_flags & EF_NO_LOCK); }

	// 
	_KERNEL CDoorSubject* door() const { return m_baseData ? m_baseData->doorObj() : NULL; }

	//  
	_KERNEL PArray<int>* walkers() const { return m_baseData ? m_baseData->walkers() : nullptr; }

public: // 

	CExitBaseData* m_baseData; //   

	CRoom* m_room;             // 
	int m_flags;               // 
	CObjPrototype* m_keyProt;  //  

};

//   
class CTrackInfo
{

public: // 

	uint m_pulse;    //  ,   
	CChar* m_char;   // ,   
	byte m_exit;     //  

};

//    
class CPointListNode
{

public: // 

	_KERNEL CPointListNode( int fractionId = 0, int squadId = 0 ) :
		m_fractionId( fractionId ), m_squadId( squadId )
	{}

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

public: // 

	int m_fractionId; // 
	int m_squadId;    // 

};

//   
class CPointMode
{

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

	//  
	_KERNEL bool CheckChar( CPointListNode* node, PArray<int>* numbers );

public: // 

	byte m_type;                    //  
	int m_fraction;                 // 
	PArray<CPointListNode*> m_list; // 

};

//   
class CTrnRoom
{

public: // 

	_KERNEL CTrnRoom();

	~CTrnRoom() {}

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

	//     
	_KERNEL bool CheckCharGather( CPointListNode* node, PArray<int>* numbers );

	//     
	_KERNEL bool CheckCharEnter( CPointListNode* node, PArray<int>* numbers );

public: // 

	bool m_beginFlag;         //  

	bool m_gatherFlag;        //  
	CPointMode* m_gatherMode; //   

	bool m_enterFlag;         //  
	CPointMode* m_enterMode;  //   

};

//   
class CRoomAddData
{
public:

    CRoomAddData( short type ) :
        m_type( type )
    {}

    short m_type;
};

//   :  
class CRoomAddZoneData : public CRoomAddData
{
public:
    CRoomAddZoneData( int zoneId, void* data ) :
        CRoomAddData( (short)radtZone ), m_zoneId( zoneId ), m_data( data )
    {}

public:
    int m_zoneId;   //  
    void* m_data;   //   
};

//   :  
class CRoomAddCustomIntData : public CRoomAddData
{
public:
    CRoomAddCustomIntData( int data ) :
        CRoomAddData( (short)radtCustomInt ), m_data( data )
    {}

public:
    int m_data;   //  
};

// 
class CRoom
{

public: //   

	// 
	_KERNEL CRoom();

	// 
	_KERNEL ~CRoom();

public: //   

	//  
	_KERNEL void Link();

	//  
	_KERNEL void Prepare();

	//  
	_KERNEL void PerformPulse();

	//  
	_KERNEL void Repop( bool init = false );

public: //    

	//    
	_KERNEL void EnterIntoRoom( CChar* c, bool toInst = false );

	//     (    )
	_KERNEL void LeaveRoom( CChar* c, RoomExit exit );

	//    
	_KERNEL CChar* FindCharacter( CChar* c ) { return ListFind( m_charList, c ); }

	//    
	_KERNEL bool noPlayersInRoom();

    //     
	_KERNEL bool noMobsInRoom();

public: //    

	//    
	_KERNEL void PutIntoRoom( CSubject* obj, bool last = false );

	//       
	_KERNEL CSubject* PutIntoRoom( int objID, bool last = false );

	//    
	_KERNEL void GetFromRoom( CSubject* obj );

	//    
	_KERNEL CSubject* FindObject( CSubject* obj ) { return ListFind( m_objList, obj ); }

	//    
	_KERNEL CSubject* FindObject( int objId, bool charsearch = false );

public: // 

	//    ( )
	_KERNEL void RoomMessage( const char* text, ... );

	//       ( )
	_KERNEL void RoomMessageU( const char* text, ... );

	//    ( )
	_KERNEL void RoomMessageU( CChar* exept, const char* text, ... );

	//    ( )
	_KERNEL void RoomMessageU( CChar* exept1, CChar* exept2, const char* text, ... );

	//       
	_KERNEL void RoomMessageIU( CChar* exept, const char* text, ... );

	//       
	_KERNEL void RoomMessageIUDmg( int damage, CChar* exept, const char* text, ... );

    //         
	_KERNEL void RoomMessageCIU( int colorID, CChar* exept, const char* text, ... );

	//     
	_KERNEL void RoomAffectMessageCIU( int colorID, CChar* exept, PkFlag pkFlag, const char* text, ... );

	//  
	_KERNEL void NatureMessage( const char* text );

	//    
	_KERNEL void RoomMessageF( ShowType show, int color, const char* text, ... );
	_KERNEL void RoomMessageF( CChar* exept1, ShowType show, int color, const char* text, ... );
	_KERNEL void RoomMessageF( CChar* exept1, CChar* exept2, ShowType show, int color, const char* text, ... );
	_KERNEL void RoomMessageF( CChar* exept1, CChar* exept2, CChar* exept3, ShowType show, int color, const char* text, ... );

    //   
    void RoomParamMessage( const char* msg, CChar* c1 = NULL, CChar* c2 = NULL );

public: // 

	//   
	_KERNEL void OpenDoor( CChar* who, RoomExit e );

	//   
	_KERNEL void CloseDoor( CChar* who, RoomExit e );

	//   
	_KERNEL void RepopDoor( RoomExit e, bool lock );

    //   
    _KERNEL void ExitAvail( int exit, bool set = true );

public: // 

	// 
	_KERNEL bool light();

    //  
    _KERNEL int flags() const { return m_flags; }

	//  
	_KERNEL int isRestRoom() const;

	// 
	_KERNEL bool isPeaceful() const { return (m_flags & RF_PEACEFUL) != 0; };

    // 
	_KERNEL bool isArena() const { return (m_flags & RF_ARENA) != 0; };

	// 
	_KERNEL bool isNoPK() const { return (m_flags & (RF_NOPK|RF_PEACEFUL) ) != 0; };

	// 
	_KERNEL bool isNoRecall() const { return (m_flags & (RF_NORECALL|RF_ARENA)) != 0; };

	// 
	_KERNEL bool isSafe() const { return (m_flags & RF_SAFE) != 0; };

	// 
	_KERNEL bool hasRoad() const { return (m_flags & RF_ROAD) != 0; };

	// 
	_KERNEL bool noMagic() const { return (m_flags & RF_NO_MAGIC) != 0; }

	//   
	_KERNEL bool noYell() const { return (m_flags & RF_NO_YELL) != 0; };

    //   
    _KERNEL bool HasDetail();

	//   
	_KERNEL int guardCircleStr() const;

	//    
	_KERNEL int spellMod( int power, MagicSphere ms, bool neg = false );

	//   
	_KERNEL int availableExits( bool mob = false ) const;

	//  
	_KERNEL bool CanTrigger( int flag, void* param = NULL ) { return m_data->m_trgs->CanTrigger( flag, this, param ); }

	//  
	_KERNEL int ProcessTrigger( CRoomTriggerData* data ) { return m_data->m_trgs->ProcessTrigger( data ); }

	//    
	_KERNEL void ReleaseFightCompress();

	//  
	_KERNEL bool CheckKey( int dir, CSubject* key );

public: //  

	// 
	_KERNEL const char* title() { return m_title ? m_title->m_str : m_data->m_title.m_str; }

	// 
	_KERNEL const char* desc() { return m_desc ? m_desc->m_str : m_data->m_desc.m_str; }

	//  
	_KERNEL CZone* zone() const { return m_data->m_zone; }

	//  
	_KERNEL int id() const { return m_data->m_id; }

	//  . 
	_KERNEL CDescAddon** descAddons() { return m_descAddons ? m_descAddons : m_data->descAddons(); }

public: //   

	//   
	_KERNEL CZoneInstance* zoneInst() { return m_zoneInst; }

	//  
	_KERNEL int instIdx() const { return m_zoneInst->m_idx; }

	//  
	_KERNEL CRoom* GetExitRoom( CChar* c, byte dir );

	//   
	_KERNEL CRoom* GetExitRoom( CChar* c, CPortalSubject* portal );

	//   
	_KERNEL bool TryVisitInst( CChar* c, CCharPlayer* cp, bool noMsg = false );

	//      
	_KERNEL void ClearInstReference( CZoneInstance* inst );

	//    
	_KERNEL void RemoveTracks( CCharMob* mob );

	//   
	_KERNEL void ClearRepopData();

public: //   

	//     
	_KERNEL CCharMob* FindMob( int mobId, int count = 1 );

	//     
	_KERNEL void ClearFromMobMemory( CCharPlayer* cp );

	//      
	_KERNEL void ClearCharReference( CCharPlayer* cp );

    //    
    _KERNEL int GetMobNum();

    //   
    _KERNEL ushort aiFlags() { return m_aiFlags; }

    //    
    _KERNEL void UpdateAICharFlags();

    //    
    _KERNEL void UpdateAIObjFlags();

    //   
    _KERNEL void FillRepopMobArray( PArray<CCharMob*>* mobArray );

public: //   

	//  
	_KERNEL void AddTrack( CChar* c, RoomExit exit );

	//  
	_KERNEL void RemoveTrack( CChar* c );

public: //  

    _KERNEL int GetCustomIntData();
    _KERNEL void SetCustomIntData( int data );

    _KERNEL void* GetAddZoneData( int zoneId );
    _KERNEL void SetAddZoneData( int zoneId, void* data );

public: // 

	CRoomBaseData* m_data; //   

	CString* m_title;  // 
	CString* m_desc;   // 

	CExit m_exits[6];   // 
	int m_flags;        // 
	int m_guilds;       // 
	byte m_landType;    //  
	byte m_hotel;       //  

	CDescAddon** m_descAddons; //   (NULL-terminated)

	CChar*    m_charList;  //   
	CSubject* m_objList;   //   

	CMobPopData* m_mobPopDatas; //    
	int m_mobPopDataCount;

	CObjPopInRoomData* m_objPopDatas; //    
	int m_objPopDataCount;

	CZoneInstance* m_zoneInst; //  
	//bool m_inst;               //  

	//PArray<CQuestStatus*> m_questStats; //  

	CTrackInfo m_tracks[TRACK_COUNT];  // 

	bool m_fightCompress;     //   

    //     
    enum UpdateAIFlagsType
    {
        UAFT_NONE = 0, // 
        UAFT_CHAR,     // 
        UAFT_OBJ,      // 
        UAFT_COUNT
    };

    PArray<CRoomAddData*>* m_addData;

    ushort m_aiFlags;         //   
    byte m_aiUpdate;          //    

};

