/*   

	Copyright (c) 2011. All Rights Reserved.
	: 
*/

#pragma once

#include "DB/Subject/RuneSlot.h"

//   
class CReagentInfo
{

public: // 

	int m_id;    //  
	int m_objId; //  
	CObjPrototype* m_prot; //  

};

//  
class CReagentLine
{

public: // 

	int m_count; //  
	CReagentInfo** m_line; // 

};

//    
class CRInfo
{

public: // 

	CRInfo();
	CRInfo( int value, int level );

public: // 

	int m_value;          // 
	int m_level;          // 

	double m_symbolPrice; //   
	uint m_symbolSellNum; //   

	double m_runePrice;   //      
	uint m_runeSellNum;   //   

};

//  
class CRuneModifier
{

public: // 

	CRuneModifier( RuneType type );

public: // 

	virtual void name( char* buf, int value ); //  
	virtual void mask( char* buf, int value ); //  

	virtual void ApplyToObject( CSubject* obj, int value ) const {} //   

public: // 

	byte m_type; //  

};

//   - 
class CSimpleRuneModifier : public CRuneModifier
{

public: // 

	CSimpleRuneModifier( ModifierType type );

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

	CModifier* m_mod; // 

};

//   - 
class CArmorRuneModifier : public CRuneModifier
{

public: // 

	CArmorRuneModifier();

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

};

//   - 
class CAffectRuneModifier : public CRuneModifier
{

public: // 

	CAffectRuneModifier( AffectID aff );

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

	CAffect* m_affect; // 

};

//   - 
class CWeightRuneModifier : public CRuneModifier
{

public: // 

	CWeightRuneModifier( bool inc );

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

	bool m_inc; // 

};

//   - 
class CFlagRuneModifier : public CRuneModifier
{

public: // 

	CFlagRuneModifier( ObjectFlag flag );

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

	int m_flag; // 

};

//   - 
class CTimerRuneModifier : public CRuneModifier
{

public: // 

	CTimerRuneModifier();

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

};

//   - 
class CMaterialRuneModifier : public CRuneModifier
{

public: // 

	CMaterialRuneModifier();

public: // 

	void name( char* buf, int value ); //  
	void mask( char* buf, int value ); //  

	void ApplyToObject( CSubject* obj, int value ) const; //   

public: // 

};

//   
class	CRuneInfo
{

public: // 

	//  
	void runeLevel( char* buf, RuneNode node ) const;
	//  
	void runeInfo( char* buf, RuneNode node ) const;

public: // 

	short m_id;            //  
	int m_perc;            // 
	CRuneModifier* m_mod;  // 
	CRInfo* m_nodes[RNN_COUNT]; //  
	int m_plan[RNP_COUNT]; //  

};

//  
class CRuneLineNode
{

public: // 

	int m_id;    //  
	byte m_node; //  

};

//  
class CRuneLine
{

public: // 

	int m_count; //  
	int m_perc;  //  
	CRuneLineNode* m_line; // 

};

//  
class CRuneBase
{

public: // 

	// 
	_KERNEL void Init();
	// 
	_KERNEL void Link();

	//   
	_KERNEL int nodeToSize( int node );
	//  
	_KERNEL RuneSize planToSize( RunePlan plan );
	//  
	_KERNEL const char* sizeToName( int size, CaseType caseType, bool male = true );
	//    
	_KERNEL bool sizeToObj( int size, CSubject* obj );
	//    
	_KERNEL const char* sizeToColor( int size );
	//  
	_KERNEL uint sizeToRuneLife( RuneNode node );
	//    
	_KERNEL bool nodeExist( RuneNode node, RunePlan size );
	//    
	_KERNEL int FindRuneByBonus( RuneType rType, int id );

	//    
	_KERNEL CSubject* createSymbol( const CRuneInfo* runeInfo, RuneNode node );
	//    
	_KERNEL CSubject* createRune( const CRuneInfo* rune, RuneNode node, uint life = 0 );
	//   
	_KERNEL void createRuneSlot( const CRuneInfo* rune, RuneNode node, uint life, CSubject* obj );

	//   
	_KERNEL CRuneLineNode* randomRune( RuneSize rSize );

    //     
    _KERNEL int GetRandomReagent( int size );

	//    
    _KERNEL CSubject* createReagent( int size, int reagPerc, CObjPrototype* reagProt = NULL );
	_KERNEL CSubject* createReagent( CMobPrototype* prot, int reagPerc );
	//    
	_KERNEL CSubject* createSymbol( CMobPrototype* prot );
	//   
	_KERNEL int setLoadPerc( int value, RuneSize size, bool symbol = false );

	//   
	_KERNEL void checkCorrectTimer( CSubject* obj, int hotel );

	//   
	_KERNEL RuneId RuneVersionCorrect( int val, CFileStore& fs );

	//    
	_KERNEL void GenerateNewPlan( CCharPlayer* cp );

public: // 

	CReagentLine* m_reagents[RNS_COUNT]; //  
	CRuneLine* m_runeLines[RNS_COUNT];   //  
	const CRuneInfo* m_runes;            //  
	int m_runeCount;                     //  

	double m_reagentK[RNS_COUNT]; //   
	double m_symbolK[RNS_COUNT];  //   

};
