/*   

	Copyright (c) 2013. All Rights Reserved.
	: 
*/
#pragma once

//     
const int TOURNAMENT_ZONE_BEFORE_MSG = 60*600;
//      
const int TOURNAMENT_ZONE_BEFORE_TIME = 15*600;
//   
const int TOURNAMENT_GATHER_PORTAL_ID = 20440;
//   
const int TOURNAMENT_EXIT_PORTAL_ID = 20441;
//   
const int TOURNAMENT_ENTER_PORTAL_ID = 20442;
//     
const int TRN_OWNER_MIN_DAYS = 21;

//  
class CFractionSquad
{

public: //   

	_KERNEL CFractionSquad( int max = 0 ) :
		m_max( max )
	{}

	~CFractionSquad();

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

	//  
	_KERNEL void Link();

	//  
	_KERNEL void SetMax( int max );

public: // 

	int m_max;                    //  
	PArray<CCharPlayer*> m_squad; //  

};


//   
class CFractionZoneDefence
{
public:
    CFractionZoneDefence();
public:
	// 
	_KERNEL void Serialize( CFileStore& fs );
public:
    CWeekSchedule   m_schedule; // 
    SYSTEMTIME      m_setTime;  //  
};


//     
class CFractionZoneSettings
{

public: //   

	CFractionZoneSettings( CFraction* fraction = NULL ) :
		m_fraction( fraction ),
        m_defence( NULL )
	{}

	~CFractionZoneSettings();

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

	// 
	_KERNEL void Link();

	//    
	_KERNEL void ClearCharReference( CCharPlayer* cp );

public: // 

	CFraction*              m_fraction; // 
	PArray<CFractionSquad*> m_squads;   // 
    CFractionZoneDefence*   m_defence;  // 

};

//   
class CTrnZoneAffectBonus
{

public: // 

	// 
	void Serialize( CFileStore& fs );

	//  
	void Apply( CChar* c );

public: // 

	short m_affectId; // 
	short m_value;    // 

};

// -  
class CTrnZoneModifierBonus
{

public: // 

	// 
	void Serialize( CFileStore& fs );

	//  -
	void Apply( CChar* c );

public: // 

	short m_modifierId; // 
	short m_value;      // 

};

//   
class CTrnZoneOwnership
{

public: // 

	CTrnZoneOwnership() :
		m_ownerId( 0 )
	{}

public: // 

	// 
	void Serialize( CFileStore& fs );

	//  
	void SerializeCurrent( CFileStore& fs );

    //  
    void SetOwner( byte ownerId );

	//   
	void ShowInfo( CCharPlayer* cp );

public: // 

	byte m_ownerId;                             // 
    CTime m_ownerStartTime;                     //   
	PArray<CTrnZoneAffectBonus*> m_affects;     // 
	PArray<CTrnZoneModifierBonus*> m_modifiers; // 

};


//  /  
class CTrnZoneDefence
{

public:

    // 
    CTrnZoneDefence();

    // 
    _KERNEL void Serialize( CFileStore& fs );

    //    
    void SetBasicDefence();

    //   
    void SetAttackTime( CTime& time, byte attackFraction );

    CWeekSchedule m_defenceSchedule;        //  
    CWeekSchedule m_tempSchedule;           //  
    CTime m_defenceSetTime;                 //    

    byte m_attackerFraction;                //  
    CTime m_attackTime;                     //  
    CTime m_attackSetTime;                  //   

    static int MIN_HOUR;                    //      
    static int MAX_HOUR;                    //      

    static int MIN_HOURS_DAY;               //     
    static int MIN_HOURS_WEEK;              //     

    static CWeekSchedule BASIC_SCHEDULE;    //  
};


//  
class CTrnZone
{

public: // 

    //   
    enum RepopPulseType
    {
        RPT_NONE = 0, // 
        RPT_MSG,      //   
        RPT_PREPARE,  // 
        RPT_START,    // 
        RPT_COUNT
    };

    // 
    _KERNEL CTrnZone();

	// 
	_KERNEL void InitNew( CZone* zone );

	// 
	_KERNEL void Serialize( CFileStore& fs );

	// 
	_KERNEL void Link();

	//    
	_KERNEL void ClearCharReference( CCharPlayer* cp );

	//    
	_KERNEL CFractionZoneSettings* FindFraction( CFraction* fraction );

	//   
	_KERNEL void ShowInfo( CCharPlayer* cp, bool admin );

    //  
    _KERNEL void PerformPulse();

    //    
    _KERNEL uint CheckRepopPulse();

	//  
	_KERNEL void SetActive( bool main = true );

	//  
	_KERNEL void SetInactive();

	//   
	_KERNEL void EndInactive();

	//    
	_KERNEL bool CheckCharRequirement( CCharPlayer* cp, CCharPlayer* actor );

	//     
	_KERNEL bool CheckCharList( CCharPlayer* cp, CPointListNodeLoadData* node );

	//   
	_KERNEL void CheckCharEquip( CCharPlayer* cp );

	//  
	_KERNEL bool CheckEquip( CSubject* obj );

	//    
	_KERNEL bool CheckLeader( CChar* who, CChar* leader );

	//   
	_KERNEL void AddCharRip( CCharPlayer* cp );

	//   
	_KERNEL uint GetCharRip( CCharPlayer* cp );

	//   
	_KERNEL CRoom* FindGatherRoom( CCharPlayer* cp );

	//   
	_KERNEL void ClearTransitObjs();

	//    
	_KERNEL void CalcNextRepop( bool main = true );

	//  
	_KERNEL void SetOwner( int ownerId );

    //   
    _KERNEL CFractionZoneSettings* FindFractionByRealPlayer( CCharPlayer* cp );

	//     
	_KERNEL CFractionZoneSettings* FindRealFraction( CFraction* fraction );

    //    
    _KERNEL void SetBasicDefence();

    //     
    _KERNEL void CheckSetLateAttack();

    //   
    _KERNEL void SetAttackTime( CTime& time, byte attackFraction );

public: // 

	CZone* m_zone;             //   
	CTrnGroup* m_group;        //  

	byte m_stage;              //  

	int m_repopTimeMin;        //   
	int m_repopTimeMax;        //   
	tm m_repopTime;            //   
	tm m_nextRepopTime;        //   
	uint m_repopPulse;         //   

	int m_activeTime;          //   
	uint m_activeStartPulse;   //   
	uint m_activeEndPulse;     //   

	int m_ripTime;             //    

	PArray<CRoomBaseData*> m_beginRooms;  //  
	PArray<CRoomBaseData*> m_gatherRooms; //  
	PArray<CRoomBaseData*> m_enterRooms;  //  
	bool m_oneGather;                     //     

	bool m_mass;               //  
	int m_massLimit;           //   

	int m_ingrLimit;           //  
	int m_potionLimit;         //  

	byte m_restrictType;         //   
	PArray<int> m_restrictList;  //  

	PArray<CFractionZoneSettings*> m_fractions;  //  
	PArray<int> m_fractionIds;        //  
	PArray<CFractionZoneSettings*> m_realSquads; //  

	PArray<CCharPulseUnit*> m_ripChars;    //  
	PArray<CObjPrototype*> m_transitProts; //  

	bool m_prepareMsg; //   

	void (*m_activeFunc)();   //   
	void (*m_inactiveFunc)(); //   
	void (*m_clearRefFunc)( CCharPlayer* ); //   
	void (*m_settingFunc)( CCharPlayer*, const char* cmd ); //  
	void (*m_changeOwnerFunc)(); //   

	CTrnZoneOwnership* m_ownership; //  
    CTrnZoneDefence* m_defence; //  /

};

//  
class CTrnGroup
{

public: // 

	CTrnGroup() :
		m_id( 0 ), m_name( "" ), m_repopZone( NULL )
	{}

public: // 

	// 
	void Serialize( CFileStore& fs );

	//   
	void GetNames( char* buffer );

	//  
	void Synchronize( CTrnZone* zone );

	//    
	void ShowPrepareMessage( CTrnZone* zone );

	//   
	void SetPrepare( CTrnZone* zone );

	//   
	void EndInactive();

	//   
	bool CheckWaiting();

	//  
	void ShowInfo( CCharPlayer* cp );

public: // 

	byte m_id;                        // 
	CString m_name;                   //   
	PArray<CTrnZone*> m_zones; //   
	CTrnZone* m_repopZone;     //  

};

//   
class CTrnZoneBase
{

public: // 

	// 
	_KERNEL void Serialize( CFileStore& fs );

	// 
	_KERNEL void Link();

	//  
	_KERNEL void AddZone( CTrnZone* zone );

	//  
	_KERNEL CTrnZone* FindZone( int id );

	//    
	_KERNEL void ClearCharReference( CCharPlayer* cp );

	//    
	_KERNEL void PrepareStuff( CSubject* obj );

	//   
	_KERNEL bool HasActive() { return m_activeZones.getCount() > 0; }

	//    
	_KERNEL void ShowPrepareMessage( CTrnZone* zone );

	//   
	_KERNEL void SetPrepare( CTrnZone* zone, bool main = true );

	//   
	_KERNEL void SetActive( CTrnZone* zone );

	//   
	_KERNEL void SetInactive( CTrnZone* zone );

	//   
	_KERNEL void PerformPulse();

	//    
	_KERNEL void PerformActivePulse();

	//   
	_KERNEL bool IsStandardEquip( CSubject* obj );

public: // 

	PArray<CTrnZone*> m_zones;       //  
	PArray<CTrnZone*> m_activeZones; //   
	PArray<CTrnGroup*> m_groups;     //   

};