/*  

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#include "DB/World.h"
#include "DB/ZoneInstance.h"
#include "DB/ZoneLoadData.h"

class CZoneUser;

//    
class CZoneAddon
{

public: // /

	_KERNEL virtual void Serialize( BYTE **data, bool store ) {}

    //  
    _KERNEL virtual void FreeData( BYTE* data ) {}

public: // 

};

// 
class CZone
{

public: // 

	_KERNEL CZone();

public: //   

	//  
	_KERNEL void LoadFolder( CFileStore& fs, Array<CMobPrototype*>& mobProts, Array<CObjPrototype*>& objProts );

	//   
	_KERNEL void Serialize( CFileStore& fs );

	//  
	_KERNEL void Link();

	//  ,  
	_KERNEL void Link2();

	//  
	_KERNEL void Prepare();

	//  
	_KERNEL void PerformPulse();

	//  
	_KERNEL void Repop( bool init = false );

	//  
	_KERNEL void PerformRound();

	//   
	_KERNEL void ZoneMessage( const char* text, ... );

	//  
	_KERNEL void NatureMessage( const char* text );

	//    
	_KERNEL int GenerateRoomID();

public: // 

	//   
	_KERNEL CZoneInstance* CreateInstance( byte lvl = 0 );

	//   
	_KERNEL void DeleteInst( int idx );

	//   
	_KERNEL CRoom* room( int idx, int num ) { return m_rooms[num]->inst( idx ); }

	//   
	_KERNEL CZoneInstance* baseInst() { return m_insts[0]; }

	//  
	_KERNEL CZoneInstance* inst( int idx );

	//  
	_KERNEL bool CheckInst( int instIdx );

    //  
    _KERNEL bool CheckInst( CZoneInstance* zoneInst );

	//      
	_KERNEL void ClearInstReference( CZoneInstance* inst );

	//   
	_KERNEL CZoneUser* GetUser( int accId );

public: // 

	//    
	_KERNEL CRoom* FindRoom( int id, int instIdx = 0 );

	//     
	_KERNEL CRoomBaseData* FindBaseRoom( int id );

	//     
	_KERNEL CMobPrototype* FindMobPrototype( int id );

	//     
	_KERNEL CMobPrototype* FindMobPrototype( CMaskManager& mm );

	//     
	_KERNEL CObjPrototype* FindObjPrototype( int id );

	//     
	_KERNEL CObjPrototype* FindObjPrototype( CMaskManager& mm, bool full = false );

	//     
	_KERNEL CSubject* FindObject( int id, int idx = 0 );

public: //   

	//     
	_KERNEL CCharMob* FindMob( int id, int idx = 0 );

	//     
	_KERNEL void ClearFromMobMemory( CCharPlayer* cp );

	//      
	_KERNEL void ClearCharReference( CCharPlayer* cp );

public: //  

    //   
    _KERNEL void CalcZoneLevel();

    //  
    _KERNEL void Weaken();

    //     
    _KERNEL CMobPopBaseData* WeakenFindLoadTarget( Pair<ushort,byte>* target );

public: // 

	// 
	_KERNEL bool light() { return m_world->light(); }

public: // 

	int     m_id;       // 
	CString m_name;     // 
	CWorld* m_world;    // 
    int     m_level;    //  

	PArray<CRoomBaseData*> m_rooms; // 

	CMobPrototype** m_mobProts;  //   
	int m_mobProtCount;

	CObjPrototype** m_objProts;  //   
	int m_objProtCount;

	bool m_inst;                    //  
    byte m_maxComplex;              //  
	short m_repop;                  //  
	int m_maxInstIdx;               //   
	uint m_instPulse;               //   
	Array<int> m_trigLibs;          //  
	PArray<CZoneInstance*> m_insts; //  
	PArray<CZoneUser*> m_users;     //  

	CTrnZone* m_trnZone;         //  
	CZoneAddon* m_addon;         //  

    bool m_noWeaken;             //   
    byte m_weakenLevel;           //    
};

//   
class CZoneInfo
{

public: // 

	//  
	_KERNEL void SetInfo( char* src );

public: // 

	short m_count;  //   
	int m_id;       // 

	char* m_name;   // 

	int m_min;      //  
	int m_max;      //  
	float m_mean;   //  

	char* m_author; // 
	char* m_place;  // 

};

//  
class CZoneBase
{

public: // 

	CZoneBase() :
		m_orderId( 0 ), m_orderName( 0 ), m_orderLevel( 0 )
	{}

public: // 

	//    
	_KERNEL void Load();

	//    
	_KERNEL CZoneInfo* GetZoneInfo( int id );

public: // 

	PArray<CZoneInfo*> m_zoneInfo; //  

	short* m_orderId;              //   
	short* m_orderName;            //   
	short* m_orderLevel;           //   

};

//    
class CZoneUser
{

public: // 

	CZoneUser( CCharPlayer* cp, int accId, CZoneInstance* inst, bool owner = false, uint pulse = 0 ) :
		m_accId( accId )
	{
		SetValues( cp, inst->m_idx, pulse ? pulse : inst->m_pulse, owner );
	}

public: // 

	//   
	_KERNEL void SetValues( CCharPlayer* cp, int instIdx, uint pulse, bool owner );

	//  
	_KERNEL void AddChar( CCharPlayer* cp );

public: // 

	int m_instIdx;         //  

	int m_accId;           // 
	PArray<CCharPlayer*> m_chars; // 

	uint m_pulse;          //   
	bool m_owner;          // 

};