/*   

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#include "DB/Zone.h"

//   ( )
class CTriggerData
{

public: // 

	_KERNEL CTriggerData( int type, bool forDelete = true, int (*handler)(CTriggerData*) = NULL );
	_KERNEL virtual ~CTriggerData() {}

public: // 

	//  ,    
	uint wakePulse() const { return m_start + m_pulses; }

public: // 

	CTriggerData* m_next; //   

	int  m_type;    //  
	bool m_delete;  //  

	uint m_start;   //    
	uint m_pulses;  //    
	uint m_step;    //   
	bool m_waiting; //  

	void* m_info;   //  

	int (*m_handler)( CTriggerData* data ); // - 

};

//   
class CMobTriggerData : public CTriggerData
{

public: //   

	CMobTriggerData( MTrgType type, CCharMob* mob, bool forDelete = true, int (*handler)( CMobTriggerData* data ) = NULL ) :
		CTriggerData( type, forDelete, ( int (*)( CTriggerData* ) )handler ),
		m_mob( mob )
	{}

	_KERNEL ~CMobTriggerData();

public: // 

	CCharMob* m_mob;      // ,    

};

//    MTRG_SPEAK
class CSpeakMobTriggerData : public CMobTriggerData
{

public : // 

	CSpeakMobTriggerData( CCharMob* mob, CCharPlayer* c, const char* text ) :
		CMobTriggerData( MTRG_SPEAK, mob ),
		m_char( c ),
		m_text( text )
	{}

public : // 

	CCharPlayer* m_char;  // ,  
	CString m_text; // 

};

//    MTRG_INCOMING_LEADER
class CIncomingLeaderMobTriggerData : public CMobTriggerData
{

public : // 

	CIncomingLeaderMobTriggerData( CCharMob* mob, CChar* c, bool group, RoomExit direction, int (*handler)( CMobTriggerData* data ) = NULL ) :
		CMobTriggerData( MTRG_INCOMING_LEADER, mob, true, handler ),
		m_char( c ),
		m_group( group ),
		m_direction( direction )
	{}

public : // 

	CChar* m_char;    // ,   
	bool m_group;     //   
	byte m_direction; // ,   

};

//    MTRG_INCOMING_ANY
class CIncomingAnyMobTriggerData : public CMobTriggerData
{

public : // 

	CIncomingAnyMobTriggerData( CCharMob* mob, CChar* c, RoomExit direction, int (*handler)( CMobTriggerData* data ) = NULL ) :
		CMobTriggerData( MTRG_INCOMING_ANY, mob, true, handler ),
		m_char( c ),
		m_direction( direction )
	{}

public : // 

	CChar* m_char;    // ,   
	byte m_direction; // ,   

};

//    MTRG_GIVE_OBJ
class CGiveObjMobTriggerData : public CMobTriggerData
{

public : // 

	CGiveObjMobTriggerData( CCharMob* mob, CCharPlayer* c, CSubject* obj ) :
		CMobTriggerData( MTRG_GIVE_OBJ, mob ),
		m_char( c ),
		m_obj( obj )
	{}

public : // 

	CCharPlayer* m_char;  // ,   
	CSubject* m_obj;      // 

};

//    MTRG_GIVE_MONEY
class CGiveMoneyMobTriggerData : public CMobTriggerData
{

public : // 

	CGiveMoneyMobTriggerData( CCharMob* mob, CCharPlayer* c, uint money ) :
		CMobTriggerData( MTRG_GIVE_MONEY, mob ),
		m_char( c ),
		m_money( money )
	{}

public : // 

	CCharPlayer* m_char;  // ,   
	uint m_money;         // - 

};

//    MTRG_RIP
class CRipMobTriggerData : public CMobTriggerData
{

public : // 

	CRipMobTriggerData( CCharMob* mob, CRoom* room, CChar* who, CCorpseSubject* corpse, bool noSigns = false ) :
		CMobTriggerData( MTRG_RIP, mob ),
		m_room( room ),
		m_who( who ),
		m_corpse( corpse ),
		m_noSigns( noSigns )
	{}

public : // 

	CRoom*  m_room;     // ,    
	CChar*  m_who;      //   ,   NULL,    
	CCorpseSubject* m_corpse; // ,   NULL,     
	bool    m_noSigns;  //   

};

//    MTRG_REPOP
class CRepopMobTriggerData : public CMobTriggerData
{

public: //   

	CRepopMobTriggerData( CCharMob* mob ) :
		CMobTriggerData( MTRG_REPOP, mob )
	{}

	~CRepopMobTriggerData();

};

//    MTRG_AGRO
class CAgroMobTriggerData : public CMobTriggerData
{

public : // 

	CAgroMobTriggerData( CCharMob* mob, CCharPlayer* who ) :
		CMobTriggerData( MTRG_AGRO, mob ),
		m_who( who )
	{}

public : // 

	CCharPlayer* m_who;  //    

};

//    MTRG_AGRO_NOLAG
class CAgroNoLagMobTriggerData : public CMobTriggerData
{

public : // 

	CAgroNoLagMobTriggerData( CCharMob* mob, CCharPlayer* who ) :
		CMobTriggerData( MTRG_AGRO_NOLAG, mob ),
		m_who( who )
	{}

public : // 

	CCharPlayer* m_who;  //     (         )

};

//    MTRG_AGRO_ON
class CAgroOnMobTriggerData : public CMobTriggerData
{

public : // 

	CAgroOnMobTriggerData( CCharMob* mob, CCharPlayer* who ) :
		CMobTriggerData( MTRG_AGRO_ON, mob ),
		m_char( who )
	{}

public : // 

	CCharPlayer* m_char;  //        

};

//    MTRG_AGRO_NOSILENCE
class CAgroNoSilenceMobTriggerData : public CMobTriggerData
{

public : // 

	CAgroNoSilenceMobTriggerData( CCharMob* mob, CCharPlayer* who ) :
		CMobTriggerData( MTRG_AGRO_NOSILENCE, mob ),
		m_who( who )
	{}

public : // 

	CCharPlayer* m_who;  //     (         )

};

//    MTRG_LOOK
class CLookMobTriggerData : public CMobTriggerData
{

public : // 

	CLookMobTriggerData( CCharMob* mob, CChar* who ) :
		CMobTriggerData( MTRG_LOOK, mob ),
		m_who( who )
	{}

public : // 

	CChar* m_who;  //    

};

//    MTRG_SOCIAL
class CSocialMobTriggerData : public CMobTriggerData
{

public : // 

	CSocialMobTriggerData( CCharMob* mob, CChar* who, const CSocialInfo* social ) :
		CMobTriggerData( MTRG_SOCIAL, mob ),
		m_who( who ),
		m_social( social )
	{}

public : // 

	CChar* m_who;  //    
	const CSocialInfo* m_social; //   

};

//    MTRG_SKILL
class CSkillMobTriggerData : public CMobTriggerData
{

public : // 

	CSkillMobTriggerData( CCharMob* mob, CChar* target, SkillID skillID ) :
		CMobTriggerData( MTRG_SKILL, mob ),
		m_target( target ),
		m_skillID( skillID )
	{}

public : // 

	CChar* m_target; // ,    
	SkillID m_skillID; //  

};

//    MTRG_CAUSE_DAMAGE
class CCauseDamageMobTriggerData : public CMobTriggerData
{

public : // 

	CCauseDamageMobTriggerData( CCharMob* mob, int damage, CWeaponSubject* weapon, CChar* c, bool magic = false ) :
		CMobTriggerData( MTRG_CAUSE_DAMAGE, mob ),
		m_damage( damage ),
		m_weapon( weapon ),
		m_char( c ),
		m_magic( magic )
	{}

public : // 

	int m_damage;             //   
	CWeaponSubject* m_weapon; // ,   
	CChar* m_char;            //    ( NULL   )
	bool m_magic;             //   

};

//    MTRG_TO_WAKE
class CToWakeMobTriggerData : public CMobTriggerData
{

public : // 

	CToWakeMobTriggerData( CCharMob* mob, CCharPlayer* who ) :
		CMobTriggerData( MTRG_TO_WAKE, mob ),
		m_who( who )
	{}

public : // 

	CCharPlayer* m_who;  //   

};

//    MTRG_FORCE
class CForceMobTriggerData : public CMobTriggerData
{

public : // 

	CForceMobTriggerData( CCharMob* mob, CChar* who, const char* text ) :
		CMobTriggerData( MTRG_FORCE, mob ),
		m_char( who ),
		m_cmd( text )
	{}

public : // 

	CChar *m_char;  //  
	CString m_cmd;  // 

};

//    MTRG_COMMAND
class CCommandMobTriggerData : public CMobTriggerData
{

public : // 

	CCommandMobTriggerData( CCharMob* mob, CChar* who, const char* text ) :
		CMobTriggerData( MTRG_COMMAND, mob ),
		m_char( who ),
		m_cmd( text )
	{}

public : // 

	CChar *m_char;  //   
	CString m_cmd;  // 

};

//    MTRG_KILL
class CKillMobTriggerData : public CMobTriggerData
{

public : // 

	CKillMobTriggerData( CCharMob* mob, CChar* who, CCorpseSubject* corpse ) :
		CMobTriggerData( MTRG_KILL, mob ),
		m_who( who ),
		m_corpse( corpse )
	{}

public : // 

	CChar* m_who;             //   
	CCorpseSubject* m_corpse; //  ,   NULL,     

};

//   
class CObjTriggerData : public CTriggerData
{

public : // 

	CObjTriggerData( OTrgType type, CSubject* obj, bool forDelete = true, int (*handler)( CObjTriggerData* data ) = NULL ) :
		CTriggerData( type, forDelete, ( int (*)( CTriggerData* ) )handler ),
		m_obj( obj )
	{}

public : // 

	CSubject* m_obj;      // ,    

};

//    OTRG_ON_LAND
class COnLandObjTriggerData : public CObjTriggerData
{

public : // 

	COnLandObjTriggerData( CSubject* obj, CRoom* room ) :
		CObjTriggerData( OTRG_ON_LAND, obj ),
		m_room( room )
	{}

public : // 

	CRoom* m_room; // ,   

};

//    OTRG_INV_COMMAND
class CInvCommandObjTriggerData : public CObjTriggerData
{

public : // 

	CInvCommandObjTriggerData( CSubject* obj, CChar* c, bool equiped, const char* cmd ) :
		CObjTriggerData( OTRG_INV_COMMAND, obj ),
		m_char( c ),
		m_equiped(equiped),
		m_cmd( cmd )
	{}

public : // 

	CChar* m_char;  // 
	bool m_equiped; //   
	CString m_cmd;  // 

};

//    OTRG_INV_COMMAND_ALWAYS
class CInvAlwaysCommandObjTriggerData : public CObjTriggerData
{

public : // 

	CInvAlwaysCommandObjTriggerData( CSubject* obj, CChar* c, bool equiped, const char* cmd ) :
		CObjTriggerData( OTRG_INV_COMMAND_ALWAYS, obj ),
		m_char( c ),
		m_equiped(equiped),
		m_cmd( cmd )
	{}

public : // 

	CChar* m_char;  // 
	bool m_equiped; //   
	CString m_cmd;  // 
											 
};

//    OTRG_DRESS
class CDressObjTriggerData : public CObjTriggerData
{

public : // 

	CDressObjTriggerData( CSubject* obj, CChar* c, SlotID slot ) :
		CObjTriggerData( OTRG_DRESS, obj ),
		m_char( c ),
		m_slot( slot )
	{}

public : // 

	CChar* m_char; // ,   
	byte m_slot;   // ,    

};

//    OTRG_DAMAGEDORREPARED
class CDamagedOrReparedObjTriggerData : public CObjTriggerData
{

public : // 

	CDamagedOrReparedObjTriggerData( CSubject* obj, CChar* c, CChar* who, int hits ) :
		CObjTriggerData( OTRG_DAMAGEDORREPARED, obj ),
		m_char( c ),
		m_who( who ),
		m_hits( hits )
	{}

public : // 

	CChar* m_char; // ,    
	CChar* m_who;  //      ( NULL   )
	int m_hits;    //     ( - ,  - , )

};

//    OTRG_UNDRESS
class CUnDressObjTriggerData : public CObjTriggerData
{

public : // 

	CUnDressObjTriggerData( CSubject* obj, CChar* c ) :
		CObjTriggerData( OTRG_UNDRESS, obj ),
		m_char( c )
	{}

public : // 

	CChar* m_char;  // ,   

};

//    OTRG_CAUSE_DAMAGE
class CCauseDamageObjTriggerData : public CObjTriggerData
{

public : // 

	CCauseDamageObjTriggerData( CChar* who, CChar* whom, int damage, CSubject* weapon, bool magic, bool nearFight ) :
		CObjTriggerData( OTRG_CAUSE_DAMAGE, weapon ),
		m_who( who ),
		m_whom( whom ),
		m_damage( damage ),
		m_weapon( weapon ),
		m_magic( magic ),
		m_nearFight( nearFight )
	{}

public : // 

	CChar* m_who;       // ,  
	CChar* m_whom;      // ,  
	int m_damage;       //      
	CSubject* m_weapon; // ,    (NULL  )
	bool m_magic;       //  
	bool m_nearFight;   // 

};

//    OTRG_TAKE
class CTakeObjTriggerData : public CObjTriggerData
{

public : // 

	CTakeObjTriggerData( CSubject* obj, CChar* c ) :
		CObjTriggerData( OTRG_TAKE, obj ),
		m_char( c )
	{}

public : // 

	CChar* m_char;  // ,   

};

//    OTRG_OPEN
class COpenObjTriggerData : public CObjTriggerData
{

public : // 

	COpenObjTriggerData( CSubject* obj, CChar* c ) :
		CObjTriggerData( OTRG_OPEN, obj ),
		m_char( c )
	{}

public : // 

	CChar* m_char;  // ,   

};

//    OTRG_DRINK
class CDrinkObjTriggerData : public CObjTriggerData
{

public : // 

	CDrinkObjTriggerData( CSubject* obj, CCharPlayer* c ) :
		CObjTriggerData( OTRG_DRINK, obj ),
		m_char( c )
	{}

public : // 

	CCharPlayer* m_char;  // ,  

};

//    OTRG_EAT
class CEatObjTriggerData : public CObjTriggerData
{

public : // 

	CEatObjTriggerData( CSubject* obj, CCharPlayer* c ) :
		CObjTriggerData( OTRG_EAT, obj ),
		m_char( c )
	{}

public : // 

	CCharPlayer* m_char;  // ,  

};

//    OTRG_PORTAL
//        
class CPortalObjTriggerData : public CObjTriggerData
{

public : // 

	CPortalObjTriggerData( CSubject* obj, CChar* c, CRoom* fromRoom, CRoom* toRoom, bool follow ) :
		CObjTriggerData( OTRG_PORTAL, obj, false ),
		m_char( c ),
		m_fromRoom( fromRoom ),
		m_toRoom( toRoom ),
		m_follow( follow )
	{}

public : // 

	CChar* m_char;     // ,    
	CRoom* m_fromRoom; //   
	CRoom* m_toRoom;   //    ( ) [out]
	bool m_follow;     //  

};

//    OTRG_TELEPORTED
class CTeleportedObjTriggerData : public CObjTriggerData
{

public : // 

	CTeleportedObjTriggerData( CSubject* obj, CChar* c, CRoom* fromRoom, CRoom* toRoom ) :
		CObjTriggerData( OTRG_TELEPORTED, obj, false ),
		m_char( c ),
		m_fromRoom( fromRoom ),
		m_toRoom( toRoom )
	{}

public : // 

	CChar* m_char;     // ,    
	CRoom* m_fromRoom; //   
	CRoom* m_toRoom;   //   

};

//    OTRG_DELETE
class CDeleteObjTriggerData : public CObjTriggerData
{

public : // 

	CDeleteObjTriggerData( CSubject* obj ) :
		CObjTriggerData( OTRG_DELETE, obj )
	{}

public : // 

};

//    OTRG_UNLOCK
class CUnlockObjTriggerData : public CObjTriggerData
{

public : // 

	CUnlockObjTriggerData( CSubject* obj, CChar* c ) :
		CObjTriggerData( OTRG_UNLOCK, obj ),
		m_char( c )
	{}

public : // 

	CChar* m_char;  // ,   

};

//   
class CRoomTriggerData : public CTriggerData
{

public : // 

	CRoomTriggerData( RTrgType type, CRoom* room, bool forDelete = true ) :
		CTriggerData( type, forDelete ),
		m_room( room )
	{}

public : // 

	CRoom* m_room;   // ,    

};

//    RTRG_COMMAND
class CRoomCommandTriggerData : public CRoomTriggerData
{

public : // 

	CRoomCommandTriggerData( CRoom* room, CChar* c, const char* cmd ) :
		CRoomTriggerData( RTRG_COMMAND, room ),
		m_char( c ),
		m_cmd( cmd )
	{}

public : // 

	CChar* m_char;  // 
	CString m_cmd;  // 

};

//    RTRG_COMMAND_EX
class CRoomCommandExTriggerData : public CRoomTriggerData
{

public : // 

	CRoomCommandExTriggerData( CRoom* room, CChar* c, const char* cmd ) :
		CRoomTriggerData( RTRG_COMMAND_EX, room ),
		m_char( c ),
		m_cmd( cmd )
	{}

public : // 

	CChar* m_char;  // 
	CString m_cmd;  // 

};

//    RTRG_RIP
class CRipRoomTriggerData : public CRoomTriggerData
{

public : // 

	CRipRoomTriggerData( CRoom* room, CChar* c, CChar* who, CCorpseSubject* corpse ) :
		CRoomTriggerData( RTRG_RIP, room ),
		m_char( c ),
		m_who( who ),
		m_corpse( corpse )
	{}

public : // 

	CChar* m_char;  // ,    
	CChar* m_who;   //   ,   NULL,   
	CCorpseSubject* m_corpse; // ,   NULL,     

};

//    RTRG_INCOMING_LEADER
class CRoomIncomingLeaderTriggerData : public CRoomTriggerData
{

public : // 

	CRoomIncomingLeaderTriggerData( CRoom* room, CChar* c, bool group, RoomExit direction ) :
		CRoomTriggerData( RTRG_INCOMING_LEADER, room ),
		m_char( c ),
		m_group( group ),
		m_direction( direction )
	{}

public : // 

	CChar* m_char;    // ,   
	bool m_group;     //   
	byte m_direction; // ,   

};

//    RTRG_INCOMING_ANY
class CRoomIncomingAnyTriggerData : public CRoomTriggerData
{

public : // 

	CRoomIncomingAnyTriggerData( CRoom* room, CChar* c, RoomExit direction ) :
		CRoomTriggerData( RTRG_INCOMING_ANY, room ),
		m_char( c ),
		m_direction( direction )
	{}

public : // 

	CChar* m_char;    // ,   
	byte m_direction; // ,   

};

//    RTRG_CAST
class CRoomCastTriggerData : public CRoomTriggerData
{

public : // 

	CRoomCastTriggerData( CRoom* room, CCharPlayer* c, CSpell* spell ) :
		CRoomTriggerData( RTRG_CAST, room ),
		m_char( c ),
		m_spell( spell )
	{}

public : // 

	CCharPlayer* m_char;  // ,  
	CSpell* m_spell;      //  

};

//    RTRG_MOVE
//        
class CRoomMoveTriggerData : public CRoomTriggerData
{

public : // 

	CRoomMoveTriggerData( CRoom* room, CCharPlayer* c, RoomExit exit ) :
		CRoomTriggerData( RTRG_MOVE, room, false ),
		m_char( c ),
		m_exit( exit ),
		m_resultRoom( NULL )
	{}

public : // 

	CCharPlayer* m_char; // ,   
	byte m_exit;         //  
	CRoom* m_resultRoom; // ,        [out]

};

//    RTRG_SPEAK
class CRoomSpeakTriggerData : public CRoomTriggerData
{

public : // 

	CRoomSpeakTriggerData( CRoom* room, CCharPlayer* c, const char* text ) :
		CRoomTriggerData( RTRG_SPEAK, room ),
		m_char( c ),
		m_text( text )
	{}

public : // 

	CCharPlayer* m_char;  // ,  
	CString m_text;       // 

};

//    RTRG_OPEN_DOOR
class CRoomOpenDoorTriggerData : public CRoomTriggerData
{

public : // 

	CRoomOpenDoorTriggerData( CRoom* room, CChar* c, RoomExit exit ) :
		CRoomTriggerData( RTRG_OPEN_DOOR, room ),
		m_char( c ),
		m_exit( exit )
	{}

public : // 

	CChar* m_char; // ,   
	byte m_exit;   //   

};

//    RTRG_OUTCOMING_LEADER
class CRoomOutcomingLeaderTriggerData : public CRoomTriggerData
{

public : // 

	CRoomOutcomingLeaderTriggerData( CRoom* room, CChar* c, RoomExit direction ) :
		CRoomTriggerData( RTRG_OUTCOMING_LEADER, room ),
		m_char( c ),
		m_direction( direction )
	{}

public : // 

	CChar* m_char;    // ,   
	byte m_direction; // ,   

};

//    RTRG_INCOMING_MOB
class CRoomIncomingMobTriggerData : public CRoomTriggerData
{

public : // 

	CRoomIncomingMobTriggerData( CRoom* room, CCharMob* m, bool group, RoomExit direction ) :
		CRoomTriggerData( RTRG_INCOMING_MOB, room ),
		m_mob( m ),
		m_group( group ),
		m_direction( direction )
	{}

public : // 

	CCharMob* m_mob;  // ,   
	bool m_group;     //   
	byte m_direction; // ,   

};

//    RTRG_PRECAST
class CRoomPrecastTriggerData : public CRoomTriggerData
{

public : // 

	CRoomPrecastTriggerData( CRoom* room, CChar* c, CSpell* spell, int castType ) :
		CRoomTriggerData( RTRG_PRECAST, room ),
		m_char( c ),
		m_spell( spell ),
        m_castType( castType )
	{}

public : // 

	CChar* m_char;   //   
	CSpell* m_spell; //  
    int m_castType;  //  

};

//     
class CAddonInfo
{

public: // 

	CAddonInfo() {}

	CAddonInfo( int zoneID, void (*factory)(CCharAddon** pAddon, bool bDelete), CZoneInstance* inst ) :
		m_zoneID( zoneID ),
		m_inst( inst ),
		m_factory( factory )
	{}

public: // 

	int m_zoneID;               // 
	CZoneInstance* m_inst;      // 
	void (*m_factory)(CCharAddon** pAddon, bool bDelete); // -

};

//     
class CZoneAddonInfo
{

public: // 

	CZoneAddonInfo() {}

	CZoneAddonInfo( int zoneId, CZoneAddon* (*factory)() ) :
		m_zoneId( zoneId ),
		m_factory( factory )
	{}

public: // 

	int m_zoneId;               // 
	CZoneAddon* (*m_factory)(); // -

};

//    
class CTriggerEntry
{

public: // 

	//_KERNEL CTriggerEntry( TriggerObjectType type, int flags, int (*handler)( CTriggerData* ), CZoneInstance* inst = NULL );
	_KERNEL CTriggerEntry( TriggerObjectType type, int flags, int (*handler)( CTriggerData* ), int instIdx = INST_NONE );

public: // 

	//  
	_KERNEL int id( CTriggerData* data );

	//  
	_KERNEL CZoneInstance* GetInst( CTriggerData* data );

	//  
	_KERNEL bool AcceptInst( CZoneInstance* inst );

public: // 

	byte m_type;                            // 
	int m_flags;                            // 
	int (*m_handler)( CTriggerData* data ); // 
	int m_count;                            // 
	int m_instIdx;                          //  
	//CZoneInstance* m_inst;                  //  

};

//    
class CTriggerHandler
{

public: //   

	CTriggerHandler();
	~CTriggerHandler();

public: // 

	//  
	_KERNEL void AddTrigger( TriggerObjectType type, int flags, int (*handler)( CTriggerData* ), int instIdx = INST_NONE );
	_KERNEL void AddTrigger( int flags, int (*handler)( CMobTriggerData*  ), int instIdx = INST_NONE ) { AddTrigger( TOT_MOB, flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }
	_KERNEL void AddTrigger( int flags, int (*handler)( CObjTriggerData*  ), int instIdx = INST_NONE ) { AddTrigger( TOT_SUBJECT, flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }
	_KERNEL void AddTrigger( int flags, int (*handler)( CRoomTriggerData* ), int instIdx = INST_NONE ) { AddTrigger( TOT_ROOM, flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }

	//  
	_KERNEL void RemoveTrigger( int flags, int (*handler)( CTriggerData* ), int instIdx = INST_NONE );
	_KERNEL void RemoveTrigger( int flags, int (*handler)( CMobTriggerData*  ), int instIdx = INST_NONE ) { RemoveTrigger( flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }
	_KERNEL void RemoveTrigger( int flags, int (*handler)( CObjTriggerData*  ), int instIdx = INST_NONE ) { RemoveTrigger( flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }
	_KERNEL void RemoveTrigger( int flags, int (*handler)( CRoomTriggerData* ), int instIdx = INST_NONE ) { RemoveTrigger( flags, ( int(*)( CTriggerData* ) )handler, instIdx ); }

	//  
	_KERNEL bool CanTrigger( int flag, CCharMob* mob, void* param = NULL, BOOL bSkipExec = FALSE );
	_KERNEL bool CanTrigger( int flag, CSubject* obj, void* param = NULL );
	_KERNEL bool CanTrigger( int flag, CRoom* room, void* param = NULL );

	//  
	_KERNEL int ProcessTrigger( CTriggerData* data );

public: // 

	byte m_type;                      //  
	int m_flags;                      //  

	PArray<CTriggerEntry*> m_entries; //  

};

//  
class CTriggerFileInfo
{

public: // 

	CTriggerFileInfo() :
		m_name( "" ), m_path( "" ), m_lib( NULL )
	{}

public: // 

	CString m_name;  //  
	CString m_path;  //   
	HINSTANCE m_lib; // 

};

//  
class CTriggerManager
{

public: // 

	// 
	_KERNEL void Init();

	//   
	_KERNEL void LoadMobFactories();

	//  
	_KERNEL void LoadTriggers();

	//  
	_KERNEL void PerformPulse();

	//   
	_KERNEL void LoadMobTrigger( int mobID, int flags, int (*handler)( CMobTriggerData* ), int instIdx = INST_NONE );
	//   
	_KERNEL void RemoveMobTrigger( int mobID, int flags, int (*handler)( CMobTriggerData* ), int instIdx = INST_NONE );

	//   
	_KERNEL void LoadObjTrigger( int objID, int flags, int (*handler)( CObjTriggerData* ), int instIdx = INST_NONE );
	//   
	_KERNEL void RemoveObjTrigger( int objID, int flags, int (*handler)( CObjTriggerData* ), int instIdx = INST_NONE );

	//   
	_KERNEL void LoadRoomTrigger( int roomID, int flags, int (*handler)( CRoomTriggerData* ), int instIdx = INST_NONE );
	//   
	_KERNEL void RemoveRoomTrigger( int roomID, int flags, int (*handler)( CRoomTriggerData* ), int instIdx = INST_NONE );

	//    
	_KERNEL int TriggerWait( uint pulses, CMobTriggerData* data );
	//    
	_KERNEL int TriggerWait( uint pulses, CObjTriggerData* data );
	//    
	_KERNEL int TriggerWait( uint pulses, CRoomTriggerData* data );

	//     
	_KERNEL bool Executed( CCharMob* mob, MTrgType type ) const;
	//     
	_KERNEL bool Executed( CSubject* obj, OTrgType type ) const;
	//     
	_KERNEL bool Executed( CRoom* room, RTrgType type ) const;

	//      MTRG_INCOMING_ANY
	_KERNEL bool ExecutedIncomingAny( CCharMob* mob, CChar* c ) const;
	//      RTRG_INCOMING_ANY
	_KERNEL bool ExecutedIncomingAny( CRoom* room, CChar* c ) const;

	//   
	_KERNEL void LoadPulseHandler( void (*func)() );

	//   
	_KERNEL void UnloadPulseHandler( void (*func)() );

	//    
	_KERNEL void RemoveQueueTriggers( CCharMob* mob );
	_KERNEL void RemoveQueueTriggers( CRoom* room );
	_KERNEL void RemoveQueueTriggers( CSubject* obj );

	//    ( AI)
	_KERNEL void LoadMobFactory( int mobID, CCharMob* (*factory)( CMobPrototype*, CZoneInstance*, CMobPopData* ), int instIdx = INST_NONE );

	//    
	_KERNEL void ClearInstReference( CZoneInstance* zoneInst );

public: //  

	//  
	_KERNEL void LoadCharAddon( int zoneID, void (*func)(CCharAddon** pAddon, bool bDelete), int instFlag = INST_NONE, CZoneInstance* inst = NULL );

	//  
	_KERNEL CCharAddon* CreateAddon( int zoneID, CZoneInstance* inst );

	//  
	_KERNEL void DeleteAddon( CCharAddon* addon );

public: //  

	//   
	_KERNEL void LoadZoneAddon( int zoneId, CZoneAddon* (*func)() );

	//   
	_KERNEL CZoneAddon* CreateZoneAddon( int zoneId );

public: // 

	static PArray<CTriggerFileInfo*> m_libs;  //  

	static CMobTriggerData* m_mobTriggers;  //    
	static CObjTriggerData* m_objTriggers;  //    
	static CRoomTriggerData* m_roomTriggers;  //    

	static Array<CAddonInfo> m_addons; //    
	static Array<CZoneAddonInfo> m_zoneAddons; //    
	static Array<void (*)()> m_pulseHandlers; //  
	static Array<void (*)( CCharPlayer*, char**, int* )> m_questHandlers; //   

	_KERNEL static CZoneInstance* m_inst; //   

};
