/*    - 

	Copyright  2001. All Rights Reserved.
	:  ..
*/

#pragma once
#include <limits.h>

//#include <iostream>
#include <new>
using namespace std;
class CFileStore;
class CByteStore;

//   
template <class T>
class Array
{

public: //   

	Array( int size = 0 );
	Array( const Array<T>& );
	inline ~Array();

	// 
	Array<T>& operator=( const Array<T>& );

public: // 

	//    
	void add( const T& t );

	//     
	void addAt( const T& t, int index );

    //     
    bool addUniq( const T& t );

	//    
	void destroyByIdx( int index );

	//     ( )
	void deleteByIdx( int index );

	//  
	int find( const T& t ) const;

    //  
    void deleteByValue( const T& t );

	//   (  )
	void clear() { m_count = 0; }

	//   ( )
	void flush();

	//   ( )
	void Delete();

	//   
	inline T& operator[]( int i ) { return m_array[i]; }
	inline const T& operator[]( int i ) const { return m_array[i]; }

	//  
	int getCount() const { return m_count; }

	//    
	void setCount( int count ) { if( count > m_size ) resize( count ); m_count = count; }

	// 
	bool isEmpty() const { return !getCount(); }

	//    
	void freeExtra() { resize( m_count ); }

	//    
	const T* directMemory() const { return m_array; }
	T* directMemory() { return m_array; }

    // 
    void Serialize( CFileStore& fs );
    void SerializeRaw( CFileStore& fs );

    //   
    void Serialize( CByteStore& bs );
    void SerializeRaw( CByteStore& bs );

public: // 

	// -
	typedef void (* IterFunc )( T&, void * );
	typedef void (* IterFuncConst )( const T&, void * );

	//   
	void forEach( IterFunc iter, void *args );
	void forEach( IterFuncConst iter, void *args ) const;

    //   -
    template<typename F>
    void ForEach( F&& func, bool isReverse = false );
    template<typename F>
    int ForEachWithBreak( F&& func, int status = SOWERR_OK );

public: //  

	//   
	void resize( int newSize );

	//    
	inline int delta() const;

	//  
	void addsize( int addSize );

public: //  

	T* m_array;  //  
	int m_size;  //   
	int m_count; //   

};

// 
template <class T>
Array<T>::Array( int size ) :
	m_size( size ),   //   
	m_count( 0 )      //   
{
	try {

	//   
	if( size ) m_array = new T[m_size];
	else m_array = NULL;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::Array" ); throw; }
}

// 
template <class T>
Array<T>::Array( const Array<T>& a )
{
	try {

	//   
	m_size = a.m_size;
	m_count =  0;

	//  
	if( m_size ) m_array = new T[m_size];
	else m_array = NULL;

	//    
	int i; for( i = 0; i < a.getCount(); i++ ) add( a[i] );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::Array" ); throw; }
}

// 
template <class T>
Array<T>& Array<T>::operator=( const Array<T>& a )
{
	try {

	flush(); // 
	resize( a.getCount() );

	//    
	int i; for( i = 0; i < a.getCount(); i++ ) add( a[i] );

	return *this;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::operator=" ); throw; }
}

// 
template <class T>
inline Array<T>::~Array()
{
	try {

	//  
	if( m_array ) delete[] m_array;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::~Array" ); throw; }
}

//   
template <class T>
void Array<T>::resize( int newSize )
{
	try {

	if( newSize == m_size ) return; //    

	ASSERT( newSize >= m_count );

	//     
	T* m_newArray = ( newSize ? new T[newSize] : NULL );

	//  
	int i; for( i = 0; i < m_count; i++ ) m_newArray[i] = m_array[i];
	m_size = newSize;

	//  
	if( m_array ) delete[] m_array;
	m_array = m_newArray;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::resize" ); throw; }
}

//  
template <class T>
void Array<T>::addsize( int addSize )
{
	try {

	if( m_size < m_count + addSize )
		resize( m_count + addSize );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::addsize" ); throw; }
}

//    
template <class T>
inline int Array<T>::delta() const
{
	try {

	return m_size ? m_size : 1;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::delta" ); throw; }
}

//    
template <class T>
void Array<T>::add( const T& t )
{
	try {

	//  
	if( m_size < m_count + 1 ) resize( m_size + delta() );

	//  
	m_array[m_count++] = t;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::add" ); throw; }
}

//     
template <class T>
void Array<T>::addAt( const T& t, int index )
{
	try {

	//  
	if( m_size < m_count + 1 )
		resize( m_size + delta() );

	//  
	int i; for( i = m_count - 1; i >= index; i-- )
		m_array[i+1] = m_array[i];

	//  
	m_array[index] = t;
	m_count++;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::addAt" ); throw; }
}

//     
template <class T>
bool Array<T>::addUniq( const T& t )
{
	try {

    int nFindIdx = find( t );
    if( nFindIdx != INT_MAX )
        return false;

    add( t );

    return true;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::addUniq" ); throw; }
}

//    
template <class T>
void Array<T>::destroyByIdx( int index )
{
	try {

	//   
	int i; for( i = index + 1; i < m_count; i++ )
		m_array[i-1] = m_array[i];

	//   
	//   
	m_array[--m_count] = T();

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::destroyByIdx" ); throw; }
}

//     ( )
template <class T>
void Array<T>::deleteByIdx( int index )
{
	try {

	//  
	delete m_array[index];

	//    
	destroyByIdx( index );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::deleteByIdx" ); throw; }
}

//  
template <class T>
int Array<T>::find( const T& t ) const
{
	try {

	//   
	int i; for( i = 0; i < m_count; i++ )
		if( m_array[i] == t )
			return i; // 

	//  
	return INT_MAX;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::find" ); throw; }
}


//  
template <class T>
void Array<T>::deleteByValue( const T& t )
{
    int idx = find( t );

    if( idx != INT_MAX )
        deleteByIdx( idx );
}


//  
template <class T>
void Array<T>::flush()
{
	try {

	//  
	if( m_array ) delete[] m_array;

	m_size = 0;   //   
	m_count = 0;  //   
	m_array = NULL; // 

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::flush" ); throw; }
}

//   ( )
template <class T>
void Array<T>::Delete()
{
	int i; for( i = 0; i < m_count; i++ )
		delete m_array[i];

	flush();
}


// 
template <class T>
void Array<T>::Serialize( CFileStore& fs )
{
    // 
    if( fs.IsStoring() ) {

        fs.wrt( getCount() );

        for( int i = 0; i < getCount(); i++ )
            m_array[i].Serialize( fs );

    // 
    } else {

        int cnt = 0;
        fs >> cnt;
        setCount( cnt );

        for( int i = 0; i < cnt; i++ ) {
            m_array[i].Serialize( fs );
        }
    }
}


// 
template <class T>
void Array<T>::Serialize( CByteStore& bs )
{
    // 
    if( bs.IsStoring() ) {

        bs << getCount();

        for( int i = 0; i < getCount(); i++ )
            m_array[i].Serialize( bs );

    // 
    } else {

        int cnt = 0;
        bs >> cnt;
        setCount( cnt );

        for( int i = 0; i < cnt; i++ ) {
            m_array[i].Serialize( bs );
        }
    }
}


// 
template <class T>
void Array<T>::SerializeRaw( CFileStore& fs )
{
    // 
    if( fs.IsStoring() ) {

        fs.wrt( getCount() );

        for( int i = 0; i < getCount(); i++ )
            fs << m_array[i];

    // 
    } else {

        int cnt = 0;
        fs >> cnt;
        setCount( cnt );

        for( int i = 0; i < cnt; i++ ) {
            fs >> m_array[i];
        }
    }
}


// 
template <class T>
void Array<T>::SerializeRaw( CByteStore& bs )
{
    // 
    if( bs.IsStoring() ) {

        bs << getCount();

        for( int i = 0; i < getCount(); i++ )
            bs << m_array[i];

    // 
    } else {

        int cnt = 0;
        bs >> cnt;
        setCount( cnt );

        for( int i = 0; i < cnt; i++ ) {
            bs >> m_array[i];
        }
    }
}


//   
template <class T>
void Array<T>::forEach( IterFunc iter, void *args )
{
	try {

	//   
	int i; for( i = 0; i < m_count; i++ )
		iter( m_array[i], args );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::forEach" ); throw; }
}

//   
template <class T>
void Array<T>::forEach( IterFuncConst iter, void *args ) const
{
	try {

	//   
	int i; for( i = 0; i < m_count; i++ )
		iter( m_array[i], args );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::forEach" ); throw; }
}

//   -
template <class T>
template <typename F>
void Array<T>::ForEach( F&& func, bool isReverse /*= false*/ )
{
	try {

	//   
    int count = m_count;
	for( int i = 0; i < count; i++ )
		func( m_array[ isReverse ? count-i-1 : i ] );

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::ForEach" ); throw; }
}


//   -
template <class T>
template <typename F>
int Array<T>::ForEachWithBreak( F&& func, int status /*= SOWERR_OK*/ )
{
	try {

	//   
	for( int i = 0; i < m_count; i++ ) {
        int statusCur = func( m_array[i] );
		if( statusCur != status )
            return statusCur;
    }

    return status;

	} catch(...) { g_log.SystemLog( "Catch: Array<T>::ForEach" ); throw; }
}


//   
template <class T>
class SArray : public Array<T>
{

public: //   

	SArray( int size = 0 );
	SArray( const SArray<T>& a ) : Array<T>( a ) {}

public: // 

	//    
	void add( const T& t );

	//    - ,
	//  
	void save( const T& t );

	//  
	int find( const T& t ) const;

	//   
	int findNear( const T& t ) const;

	private: //    

	//     
	void addAt( const T& t, int index );

	// -
	typedef void (* IterFunc )( T&, void * );
	typedef void (* IterFuncConst )( const T&, void * );

	//   
	void forEach( IterFunc iter, void *args );

};

// 
template <class T>
SArray<T>::SArray( int size ) :
	Array<T>( size )
{}

//    
template <class T>
void SArray<T>::add( const T& t )
{
	try {

	//   
	int index = findNear( t );

	// 
	Array<T>::addAt( t, index );

	} catch(...) { g_log.SystemLog( "Catch: SArray<T>::add" ); throw; }
}

//    - ,
//  
template <class T>
void SArray<T>::save( const T& t )
{
	try {

	int index = findNear( t );
	if( index < this->m_count && this->m_array[index] == t )
		this->m_array[index] = t; // 
	else
		Array<T>::addAt( t, index ); // 

	} catch(...) { g_log.SystemLog( "Catch: SArray<T>::save" ); throw; }
}

//  
template <class T>
int SArray<T>::find( const T& t ) const
{
	try {

	int index = findNear( t );
	if( index < this->m_count && this->m_array[index] == t ) return index;
	else return INT_MAX;

	} catch(...) { g_log.SystemLog( "Catch: SArray<T>::find" ); throw; }
}

//   
template <class T>
int SArray<T>::findNear( const T& t ) const
{
	try {

	if( this->m_count == 0 )
		return 0; //    

	int lower = 0;
	int upper = this->m_count;

	//  
	while( lower < upper )
	{
		//  
		int middle = ( lower + upper ) / 2;

		if( this->m_array[middle] >= t )
			upper = middle;
		else
			lower = middle + 1;
	}

	//  
	return lower;

	} catch(...) { g_log.SystemLog( "Catch: SArray<T>::findNear" ); throw; }
}

// 
template <class T>
class Set : public SArray<T>
{

public: // 

	Set<T>() {}
	Set<T>( const Set<T>& set ) : SArray<T>( set ) {}

public: // 

	// , ,   
	Set<T> operator+( const Set<T>& set );
	Set<T> operator*( const Set<T>& set );
	Set<T> operator-( const Set<T>& set );

	// , ,    
	Set<T>& operator+=( const Set<T>& set );
	Set<T>& operator*=( const Set<T>& set );
	Set<T>& operator-=( const Set<T>& set );

public: // 

	bool add( const T& t );      //    
	bool destroy( const T& t );  //    

	//  -  
	bool hasMember( const T& t ) const;

};

//  -  
template <class T>
bool Set<T>::hasMember( const T& t ) const
{
	try {

	return SArray<T>::find( t ) == INT_MAX ? false : true;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::hasMember" ); throw; }
}

//    
template <class T>
bool Set<T>::add( const T& t )
{
	try {

	// 
	int index = SArray<T>::findNear( t );
	if( index == this->m_count || !( this->m_array[index] == t ) )
	{
		//  , 
		Array<T>::addAt( t, index );
		return true;
	}
	else
		return false; //   

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::add" ); throw; }
}

//    
template <class T>
bool Set<T>::destroy( const T& t )
{
	try {

	// 
	int index = SArray<T>::findNear( t );
	if( index == this->m_count || !( this->m_array[index] == t ) )
	{
		//  
		return false;
	}
	else
	{
		// , 
		this->destroyByIdx( index );
		return true;
	}

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::destroy" ); throw; }
}

//   
template <class T>
Set<T> Set<T>::operator+( const Set<T>& set )
{
	try {

	Set newSet( *this );
	newSet += set;
	return newSet;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator+" ); throw; }
}

//   
template <class T>
Set<T> Set<T>::operator*( const Set<T>& set )
{
	try {

	Set newSet;
	int i; for( i = 0; i < this->getCount(); i++ )
		if( set.hasMember( this->m_array[i] ) )
			newSet.SArray<T>::Add( this->m_array[i] );

	return newSet;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator*" ); throw; }
}

//   
template <class T>
Set<T> Set<T>::operator-( const Set<T>& set )
{
	try {

	Set newSet;
	int i; for( i = 0; i < this->getCount(); i++ )
		if( !set.hasMember( this->m_array[i] ) )
			newSet.SArray<T>::Add( this->m_array[i] );

	return newSet;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator-" ); throw; }
}

//    
template <class T>
Set<T>& Set<T>::operator+=( const Set<T>& set )
{
	try {

	int i; for( i = 0; i < set.getCount(); i++ ) add( set[i] );
	return *this;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator+=" ); throw; }
}

//   
template <class T>
Set<T>& Set<T>::operator-=( const Set<T>& set )
{
	try {

	int i; for( i = 0; i < set.m_count; i++ ) destroy( set[i] );
	return *this;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator-=" ); throw; }
}

//    
template <class T>
Set<T>& Set<T>::operator*=( const Set<T>& set )
{
	try {

	int i; for( i = 0; i < this->m_count; )
		if( set.hasMember( this->m_array[i] ) ) i++;
		else this->destroyByIdx( i );

	return *this;

	} catch(...) { g_log.SystemLog( "Catch: Set<T>::operator*=" ); throw; }
}

//   (  )
template <class T>
class BasePArray : public Array<T>
{

public: //   

	BasePArray<T>( int size = 0 ) : Array<T>( size ) {}

private: //     

	BasePArray<T>( const BasePArray<T>& );
	void operator=( const BasePArray<T>& );

public: // 

	//    
	void add( const T& t );

	//     
	void addAt( const T& t, int index );

	//    
	void destroyByIdx( int index );

	//     ( )
	void deleteByIdx( int index );

	//    
	void freeExtra() { resize( this->m_count ); }

	//  
	void addsize( int addSize );

    //   
    int findByKey( int key );

    //    
    T FindByMember( void* pValue, uint uType, int* pIdx = NULL );
    T FindByMember( int   nValue, uint uType, int* pIdx = NULL );

    // 
    void Serialize( CFileStore& fs );

//protected: //  

	//   
	void resize( int newSize );

};

//    
template <class T>
void BasePArray<T>::add( const T& t )
{
	try {

	//  
	if( m_size < m_count + 1 )
		resize( m_size + delta() );

	//  
	m_array[m_count++] = t;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::add" ); throw; }
}

//     
template <class T>
void BasePArray<T>::addAt( const T& t, int index )
{
	try {

	//  
	if( m_size < m_count + 1 )
		resize( m_size + delta() );

	//  
	::memmove( &m_array[index+1], &m_array[index], (m_count - index) * sizeof(T) );

	//  
	m_array[index] = t;
	m_count++;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::addAt" ); throw; }
}

//    
template <class T>
void BasePArray<T>::destroyByIdx( int index )
{
	try {

	//   
	//   
	::memmove( &m_array[index], &m_array[index+1], (--m_count - index) * sizeof(T) );

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::destroyByIdx" ); throw; }
}

//     ( )
template <class T>
void BasePArray<T>::deleteByIdx( int index )
{
	try {

	//  
	delete m_array[index];

	//   
	destroyByIdx( index );

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::deleteByIdx" ); throw; }
}

//   
template <class T>
void BasePArray<T>::resize( int newSize )
{
	try {

	if( newSize == m_size )
		return; //    

	T* m_newArray = NULL;

	try {

	//     
	m_newArray = ( newSize ? new T[newSize] : NULL );

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::resize 1 size %d, newSize %d", this->m_size, newSize ); throw; }

	try {

	//  
	::memcpy( m_newArray, m_array, m_count * sizeof(T) );
	m_size = newSize;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::resize 2 size %d, newSize %d", this->m_size, newSize ); throw; }

	//  
	if( m_array ) delete[] m_array;
	m_array = m_newArray;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::resize size %d, newSize %d", this->m_size, newSize ); throw; }
}


//  
template <class T>
void BasePArray<T>::addsize( int addSize )
{
	resize( m_size + addSize );
}


//   
template <class T>
int BasePArray<T>::findByKey( int key )
{
    if( m_count <= 0 )
        return INT_MAX;

    int i; for( i = 0; i < m_count; i++ )
        if( m_array[i]->data() == key )
            return i;
    
    return INT_MAX;
}


//    
template <class T>
T BasePArray<T>::FindByMember( void* pValue, uint uType, int* pIdx /*= NULL*/ )
{
    int i; for( i = 0; i < m_count; i++ )
        if( m_array[i]->Compare( pValue, uType ) ) {
            if( pIdx != NULL )
                *pIdx = i;
            return m_array[i];
        }

    if( pIdx != NULL )
        *pIdx = -1;
    return NULL;
}


//    
template <class T>
T BasePArray<T>::FindByMember( int nValue, uint uType, int* pIdx /*= NULL*/ )
{
    int i; for( i = 0; i < m_count; i++ )
        if( m_array[i]->Compare( nValue, uType ) ) {
            if( pIdx != NULL )
                *pIdx = i;
            return m_array[i];
        }

    if( pIdx != NULL )
        *pIdx = -1;
    return NULL;
}


// 
template <class T>
void BasePArray<T>::Serialize( CFileStore& fs )
{
    Array<T>::Serialize( fs );
}


//  
template <class T>
class PArray : public BasePArray<T>
{

public: //   

	PArray( int size = 0 ) : BasePArray<T>( size ) {}

public: // 

    void Serialize( CFileStore& fs );

};


// 
template <class T>
void PArray<T>::Serialize( CFileStore& fs )
{
    BasePArray<T>::Serialize( fs );
}


//   
template <class T>
class PArray<T*> : public BasePArray<T*>
{

public: //   

	PArray( int size = 0 ) : BasePArray<T*>( size ) {}

public: // 

    // 
    void Serialize( CFileStore& fs );

};


//   
template <class T>
void PArray<T*>::Serialize( CFileStore& fs )
{
    // 
    if( fs.IsStoring() ) {

        fs.wrt( getCount() );

        for( int i = 0; i < getCount(); i++ )
            m_array[i]->Serialize( fs );

    // 
    } else {

        int cnt = 0;
        fs >> cnt;
        setCount( cnt );

        for( int i = 0; i < cnt; i++ ) {
            m_array[i] = new T();
            m_array[i]->Serialize( fs );
        }
    }
}



//   (  )
template <class T>
class PSet : public PArray<T>
{

public: // 

	PSet<T>() {}

	private: //     

	PSet<T>( const PSet<T>& set );
	void operator=( const PSet<T>& set );

public: //  

	//    
	bool add( const T& t );

	//    
	bool destroy( const T& t );

	//  
	int find( const T& t ) const;

	//   
	int findNear( const T& t ) const;

	//  -  
	bool hasMember( const T& t ) const;

};

//    
template <class T>
bool PSet<T>::add( const T& t )
{
	try {

	// 
	int index = findNear( t );
	if( index == this->m_count || !( this->m_array[index] == t ) )
	{
		//  , 
		addAt( t, index );
		return true;
	}
	else
		return false; //   

	} catch(...) { g_log.SystemLog( "Catch: PSet<T>::add" ); throw; }
}

//    
template <class T>
bool PSet<T>::destroy( const T& t )
{
	try {

	// 
	int index = findNear( t );
	if( index == this->m_count || !( this->m_array[index] == t ) )
	{
		//  
		return false;
	}
	else
	{
		// , 
		this->destroyByIdx( index );
		return true;
	}

	} catch(...) { g_log.SystemLog( "Catch: PSet<T>::destroy" ); throw; }
}

//  
template <class T>
int PSet<T>::find( const T& t ) const
{
	try {

	int index = findNear( t );
	if( index < this->m_count && this->m_array[index] == t ) return index;
	else return INT_MAX;

	} catch(...) { g_log.SystemLog( "Catch: PSet<T>::find" ); throw; }
}

//   
template <class T>
int PSet<T>::findNear( const T& t ) const
{
	try {

	if( this->m_count == 0 )
		return 0; //    

	int lower = 0;
	int upper = this->m_count;

	//  
	while( lower < upper )
	{
		//  
		int middle = ( lower + upper ) / 2;

		if( this->m_array[middle] >= t )
			upper = middle;
		else
			lower = middle + 1;
	}

	//  
	return lower;

	} catch(...) { g_log.SystemLog( "Catch: PSet<T>::findNear" ); throw; }
}

//  -  
template <class T>
bool PSet<T>::hasMember( const T& t ) const
{
	try {

	return find( t ) == INT_MAX ? false : true;

	} catch(...) { g_log.SystemLog( "Catch: PSet<T>::hasMember" ); throw; }
}

//   (  ,  )
template <class T>
class SPArray : public PArray<T>
{

public: //   

	SPArray<T>() {}

	private: //     

	SPArray<T>( const SPArray<T>& );
	void operator=( const SPArray<T>& );

public: // 

	//  
	int find( const T& t ) const;

};

//  
template <class T>
int SPArray<T>::find( const T& t ) const
{
	try {

	//   
	int i; for( i = 0; i < this->m_count; i++ )
		if( *this->m_array[i] == *t )
			return i; // 

	//  
	return INT_MAX;

	} catch(...) { g_log.SystemLog( "Catch: SPArray<T>::find" ); throw; }
}

//   (  ,  )
template <class T>
class SPSet : public PArray<T>
{

public: //   

	SPSet<T>() {}

	private: //     

	SPSet<T>( const SPSet<T>& set );
	void operator=( const SPSet<T>& set );

public: //  

	//    
	bool add( const T& t );

	//    
	bool destroy( const T& t );

	//  
	int find( const T& t ) const;

	//   
	int findNear( const T& t ) const;

	//  -  
	bool hasMember( const T& t ) const;

};

//    
template <class T>
bool SPSet<T>::add( const T& t )
{
	try {

	// 
	int index = findNear( t );
	if( index == this->m_count || !( *this->m_array[index] == *t ) )
	{
		//  , 
		addAt( t, index );
		return true;
	}
	else
		return false; //   

	} catch(...) { g_log.SystemLog( "Catch: SPSet<T>::add" ); throw; }
}

//    
template <class T>
bool SPSet<T>::destroy( const T& t )
{
	try {

	// 
	int index = findNear( t );
	if( index == this->m_count || !( *this->m_array[index] == *t ) )
	{
		//  
		return false;
	}
	else
	{
		// , 
		this->destroyByIdx( index );
		return true;
	}

	} catch(...) { g_log.SystemLog( "Catch: SPSet<T>::destroy" ); throw; }
}

//  
template <class T>
int SPSet<T>::find( const T& t ) const
{
	try {

	int index = findNear( t );
	if( index < this->m_count && *this->m_array[index] == *t ) return index;
	else return INT_MAX;

	} catch(...) { g_log.SystemLog( "Catch: SPSet<T>::find" ); throw; }
}

//   
template <class T>
int SPSet<T>::findNear( const T& t ) const
{
	try {

	if( this->m_count == 0 )
		return 0; //    

	int lower = 0;
	int upper = this->m_count;

	//  
	while( lower < upper )
	{
		//  
		int middle = ( lower + upper ) / 2;

		if( *this->m_array[middle] >= *t ) upper = middle;
		else lower = middle + 1;
	}

	//  
	return lower;

	} catch(...) { g_log.SystemLog( "Catch: SPSet<T>::findNear" ); throw; }
}

//  -  
template <class T>
bool SPSet<T>::hasMember( const T& t ) const
{
	try {

	return find( t ) == INT_MAX ? false : true;

	} catch(...) { g_log.SystemLog( "Catch: SPSet<T>::hasMember" ); throw; }
}

//  
class StringSet : public PArray<char*>
{

public: //   

	StringSet() {}

	private: //     

	StringSet( const StringSet& set );
	void operator=( const StringSet& set );

public: //  

	//    
	_KERNEL bool add( char* t );

	//  
	_KERNEL int find( const char* t ) const;

	//   
	_KERNEL int findNear( const char* t ) const;

	//  -  
	_KERNEL bool hasMember( const char* t ) const;

};

//  ,  
template <class T>
class DArray : public PArray<T>
{

public: //   

    DArray<T>( int size = 0, int delta = 0 ) :
        PArray<T>( size ), m_delta( delta )
    {}

private: //     

	DArray<T>( const DArray<T>& );
	void operator=( const DArray<T>& );

public: // 

	//    
	void add( const T& t );

	//     
	void addAt( const T& t, int index );

	//    
	inline int delta() const  { return m_delta > 0 ? m_delta : Array<T>::delta(); }

public: // 

    ushort m_delta;    // 

};

//    
template <class T>
void DArray<T>::add( const T& t )
{
	try {

	//  
	if( m_size < m_count + 1 )
    {
        SLog( 10, "DArray::resize: %d + %d\n", m_size, delta() );
		resize( m_size + delta() );
    }

	//  
	m_array[m_count++] = t;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::add" ); throw; }
}

//     
template <class T>
void DArray<T>::addAt( const T& t, int index )
{
	try {

	//  
	if( m_size < m_count + 1 )
		resize( m_size + delta() );

	//  
	::memmove( &m_array[index+1], &m_array[index], (m_count - index) * sizeof(T) );

	//  
	m_array[index] = t;
	m_count++;

	} catch(...) { g_log.SystemLog( "Catch: PArray<T>::addAt" ); throw; }
}
