/*    

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once

//   
class CFileStore
{

public: // , 

	_KERNEL CFileStore( const char* fileName, bool store );
	_KERNEL ~CFileStore();

public: // 

	//  
	_KERNEL const char* FileName() const { return m_file.FileName(); }

	//  (/)
	_KERNEL bool IsStoring() const { return m_store; }

	//  
	_KERNEL bool IsOpen() const { return m_file.IsOpen(); }

public: //    

	//   
	_KERNEL void WriteBuf( byte* data, int size );
	_KERNEL void WriteBufInt( int* data );
	_KERNEL void WriteBufShort( short* data );
	_KERNEL void WriteBufByte( byte* data );

	//    
	_KERNEL void FlushBuf();

public: // 

	// bool
	_KERNEL void wrt( bool b ) { WriteBufByte( (byte*)&b ); }
	_KERNEL void operator<<( bool& b ) { WriteBufByte( (byte*)&b ); }
	_KERNEL void operator>>( bool& b ) { m_file >> b; }
    _KERNEL void operator%( bool& b ) { if( IsStoring() ) operator<<( b ); else operator>>( b ); }

	// byte
	_KERNEL void wrt( byte b ) { WriteBufByte( (byte*)&b ); }
	_KERNEL void operator<<( byte& b ) { WriteBufByte( (byte*)&b ); }
	_KERNEL void operator>>( byte& b ) { m_file >> b; }
    _KERNEL void operator%( byte& b ) { if( IsStoring() ) operator<<( b ); else operator>>( b ); }

	// char
	_KERNEL void wrt( char c ) { WriteBufByte( (byte*)&c ); }
	_KERNEL void operator<<( char& c ) { WriteBufByte( (byte*)&c ); }
	_KERNEL void operator>>( char& c ) { m_file >> c; }
    _KERNEL void operator%( char& c ) { if( IsStoring() ) operator<<( c ); else operator>>( c ); }

	// ushort
	_KERNEL void wrt( ushort i ) { WriteBufShort( (short*)&i ); }
	_KERNEL void operator<<( ushort& i ) { WriteBufShort( (short*)&i ); }
	_KERNEL void operator>>( ushort& i ) { m_file >> i; }
    _KERNEL void operator%( ushort& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// short
	_KERNEL void wrt( short i ) { WriteBufShort( (short*)&i ); }
	_KERNEL void operator<<( short& i ) { WriteBufShort( (short*)&i ); }
	_KERNEL void operator>>( short& i ) { m_file >> i; }
    _KERNEL void operator%( short& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// uint
	_KERNEL void wrt( uint i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator<<( uint& i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator>>( uint& i ) { m_file >> i; }
    _KERNEL void operator%( uint& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// uint
	_KERNEL void wrt( unsigned long i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator<<( unsigned long& i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator>>( unsigned long& i ) { m_file >> i; }
    _KERNEL void operator%( unsigned long& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// int
	_KERNEL void wrt( int i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator<<( int& i ) { WriteBufInt( (int*)&i ); }
	_KERNEL void operator>>( int& i ) { m_file >> i; }
    _KERNEL void operator%( int& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// double
	_KERNEL void wrt( double i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator<<( double& i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator>>( double& i ) { m_file >> i; }
    _KERNEL void operator%( double& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// int64
	_KERNEL void wrt( int64 i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator<<( int64& i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator>>( int64& i ) { m_file >> i; }
    _KERNEL void operator%( int64& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// uint64
	_KERNEL void wrt( uint64 i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator<<( uint64& i ) { WriteBuf( (byte*)&i, sizeof(i) ); }
	_KERNEL void operator>>( uint64& i ) { m_file >> i; }
    _KERNEL void operator%( uint64& i ) { if( IsStoring() ) operator<<( i ); else operator>>( i ); }

	// 
	_KERNEL void operator<<( const char* str );
	_KERNEL void operator>>( char* str );
    _KERNEL void operator%( char* str ) { if( IsStoring() ) operator<<( str ); else operator>>( str ); }

	_KERNEL void operator<<( const CString& str) { operator<<( (const char*)str ); }
	_KERNEL void operator>>( CString& str );
    _KERNEL void operator%( CString& str ) { if( IsStoring() ) operator<<( str ); else operator>>( str ); }
	//_KERNEL int ReadStrLen();

	// time_t
	_KERNEL void operator<<( time_t& t ) { WriteBuf( (byte*)&t, sizeof(t) ); }
	_KERNEL void operator>>( time_t& t ) { m_file.Read( &t, sizeof(t) ); }
    _KERNEL void operator%( time_t& t ) { if( IsStoring() ) operator<<( t ); else operator>>( t ); }

	// IpAddr
	_KERNEL void wrt( IpAddr ip );
	_KERNEL void operator<<( IpAddr& ip );
	_KERNEL void operator>>( IpAddr& ip );
    _KERNEL void operator%( IpAddr& ip ) { if( IsStoring() ) operator<<( ip ); else operator>>( ip ); }

    // SYSTEMTIME
	_KERNEL void operator<<( SYSTEMTIME& t );
	_KERNEL void operator>>( SYSTEMTIME& t );
    _KERNEL void operator%( SYSTEMTIME& t ) { if( IsStoring() ) operator<<( t ); else operator>>( t ); }

    //    
    _KERNEL void wrtVar( uint64 val );
    _KERNEL uint64 readVar(); 

	//   
	_KERNEL void write( const void* buf, int size ) { WriteBuf( (byte*)buf, size ); }
	_KERNEL void read( void* buf, int size ) { m_file.Read( buf, size ); }

public: // 

	CFile m_file;  //  
	bool m_store;  //  (/)
	int m_version; // 

	byte* m_storeBuf; //   
	byte* m_endBuf;   //   ()
	byte* m_curBuf;   //    

};
