/*    
*
* Copyright  2005. All Rights Reserved.
* :   aka Impuls
*/

#pragma once
#include "Tools/Arrays.h"

template <class T>
class TimeoutQueue
{
public:
	TimeoutQueue();

	private:
	PArray<T> items;
	Array<int> timeouts;

public:
	/*    .
	*     ,      .
	*/
	void setItemTimeout(const T& item, int timeout);

	/*       .
	*     ,  0.
	*/
	int getItemTimeout(const T& item);

	/*      (   ).
	*      ,    .
	*/
	void clearItemTimeout(const T& item);

	/*   . */
	void clearAllTimeouts();

	/*        . */
	void incAllTimeouts(int delta = 1);

	/*        . */
	void decAllTimeouts(int delta = 1);

	private:
	/*     . */
	void removeItem(const T& item);

	/*        . */
	void removeItem(int index);
};

template <class T>
TimeoutQueue<T>::TimeoutQueue(): items(), timeouts()
{}

template <class T>
void TimeoutQueue<T>::setItemTimeout(const T& item, int timeout)
{
	int index = items.find(item);
	int count = items.getCount();
	if(index > count)
	{
		index = count;
		items.addAt(item, index);
		timeouts.addAt(timeout, index);
	}
	else
	{
		timeouts[index] = timeout;
	}
	if(timeout == 0)
	{
		removeItem(index);
	}
}

template <class T>
int TimeoutQueue<T>::getItemTimeout(const T& item)
{
	int index = items.find(item);
	int count = items.getCount();
	if(index > count)
	{
		return 0;
	}
	else
	{
		return timeouts[index];
	}
}

template <class T>
void TimeoutQueue<T>::clearItemTimeout(const T& item)
{
	removeItem(item);
}

template <class T>
void TimeoutQueue<T>::clearAllTimeouts()
{
	int count = items.getCount();
	while(count--)
	{
		removeItem(0);
	}
}

template <class T>
void TimeoutQueue<T>::incAllTimeouts(int delta)
{
	int count = items.getCount();
	for(int i = 0; i < count; i++)
	{
		timeouts[i] += delta;
	}
}

template <class T>
void TimeoutQueue<T>::decAllTimeouts(int delta)
{
	int count = items.getCount();
	for(int i = 0; i < count; i++)
	{
		timeouts[i] -= delta;
		if(timeouts[i] < 0)
		{
			timeouts[i] = 0;
		}
	}

	int index = timeouts.find(0);
	while(index < count)
	{
		removeItem(index);
		count--;
		index = timeouts.find(0);
	}
}

template <class T>
void TimeoutQueue<T>::removeItem(const T& item)
{
	int index = items.find(item);
	int count = items.getCount();
	if(index < count)
	{
		removeItem(index);
	}
}

template <class T>
void TimeoutQueue<T>::removeItem(int index)
{
	int count = items.getCount();
	if(index < count)
	{
		items.destroyByIdx(index);
		timeouts.destroyByIdx(index);
	}
}
