/*  

	Copyright (c) 2001 - 2004. All Rights Reserved.
	:  ..
*/
#pragma once
#define ASSERT(x)

#include "Tools/Arrays.h"
#include "Tools/List.h"
#include "Tools/String.h"
#include "Tools/File.h"
#include "Tools/FileStore.h"
#include "Tools/ClassArray.h"
#include "Tools/Pair.h"
#include "Tools/ByteStore.h"

#define bufferAddStr( buf, str, ... ) { sprintf( buf, str, ##__VA_ARGS__ ); buf += strlen( buf ); }

#define exitFunc( act ) { act; goto exit; }

#define FStepInit       uint fDbgStep = 0
#define FStep           fDbgStep++
#define FStepInc( x )   fDbgStep += (x)
#define FStepVal        fDbgStep

#define LogP( var, func )   (var) ? (var->func) : -1

class IpAddr
{

public:

	IpAddr& operator=( struct in_addr& addr );

public:

	union
	{
		struct { u_char s_b1,s_b2,s_b3,s_b4; } S_un_b;
		struct { u_short s_w1,s_w2; } S_un_w;
		u_long S_addr;
	} S_un;

};

//  
inline void SkipSpaces( char*& src ) { while( *src &&  isspace( *src ) ) src++; }
inline void SkipSpaces( const char*& src ) { while( *src &&  isspace( *src ) ) src++; }

//   
void CharUpperLetter( char *c );

//   
_KERNEL void CharCaseLower( char *text );

//  
bool isAlpha( char c );

//        
_KERNEL void NameConvert( char* name );

//  
_KERNEL bool CheckLen( const char* text, int max, int min = 0 );

//   
_KERNEL bool CheckAccountName( const char* text );

//   
_KERNEL bool CheckCharacterName( const char* text, bool first = false );

//  
_KERNEL void ColorFormat( const char* src, char* dst );

//   
_KERNEL void DescFormat( const char* src, char* dst );
_KERNEL void MapDescFormat( const char* src, char* dst );

//    
_KERNEL const char* NumColorStr( int val, int max );

//         
_KERNEL void CharUpperColor( char* text );

//  
inline char* GenderSwitch( Gender g, SwitchType type )
{
	static char* s_types[SWT_COUNT][GENDER_COUNT] = {
		{ "", "",    "",   "",   "",   "" }, // SWT_S1
		{ "", "",    "",   "",   "",   "" }, // SWT_S2
		{ "", "",  "", "", "", "" }, // SWT_S3
		{ "", "", "",  "", "",  "" }, // SWT_S4
		{ "", "",    "",  "",  "",  "" }, // SWT_S5
		{ "", "",  "",  "",  "",  "" }, // SWT_S6
		{ "", "",  "",  "",  "",  "" }, // SWT_S7
		{ "", "", "",  "", "",  "" }, // SWT_S8
		{ "", "",  "",  "",  "", "" }, // SWT_S9
		{ "", "",  "",  "",  "",  "" }, // SWT_S10
		{ "", "",  "",  "",  "",  "" }, // SWT_S11
		{ "", "", "",  "", "",  "" }, // SWT_S12
		{ "", "",  "",  "",  "",  "" }, // SWT_S13
		{ "", "", "",  "",  "",  "" }, // SWT_S14
	};

	return s_types[type][g];
}

//  
inline char* GenderSwitch( Gender g, char* male, char* female, char* neutral, char* multiple )
{
	switch( g )
	{
		case GENDER_MALE:     return male;
		case GENDER_FEMALE:   return female;
		case GENDER_NEUTRAL:  return neutral;
		case GENDER_MULTIPLE: return multiple;
		default:              return "";
	}
}

//  
_KERNEL char* NumberSwitch( int num, char* form1, char* form2, char* form5 );
_KERNEL char* NumberSwitch( double num, char* form1, char* form2, char* form5 );
_KERNEL char* NumberSwitch( uint64 num, char* form1, char* form2, char* form5 );

//    
//_KERNEL char* MoneySwitch( int num );
//_KERNEL char* MoneySwitch3( int num );
_KERNEL char* MoneySwitch( uint64 num );
_KERNEL char* MoneySwitch3( uint64 num );

//   
_KERNEL inline int numForm( int num );

//   
_KERNEL inline const char* timeName( uint num, TimePrecise precise, CaseType caseType = CT_IMEN );

//  
_KERNEL bool GetToken( const char*& src, char* token );

//   
_KERNEL bool IsInteger( const char* str );

//     
_KERNEL char* FullTimeAsStr( time_t t );

//  
_KERNEL uint CountTimePulses( time_t t );

//     
_KERNEL char* SmallTimeAsStr( time_t t );

//    
_KERNEL char* TimeAsStr( time_t t );

//     
_KERNEL char* TimeStr( char* buffer, uint pulses, int precise = TP_MIN, CaseType caseType = CT_IMEN, TimePreciseFlag flags = TPF_NONE );

//    
_KERNEL char* DayAsStr( time_t t );

//     
_KERNEL int timePulseDiff( tm* strTime );

//    
inline bool IsAffProtect( AffectID id )
{
	switch( id )
	{
		case AFF_PROTECTION_ASTRAL:
		case AFF_PROTECTION_MENTAL:
		case AFF_PROTECTION_LIFE:
		case AFF_PROTECTION_DEATH:
		case AFF_PROTECTION_FIRE:
		case AFF_PROTECTION_WATER:
		case AFF_PROTECTION_AIR:
		case AFF_PROTECTION_EARTH:
			return true;
	}
	return false;
}

//    
inline AffectID SphereToProtect( MagicSphere sphere )
{
	switch( sphere )
	{
		case MS_ASTRAL: return AFF_PROTECTION_ASTRAL;
		case MS_MENTAL: return AFF_PROTECTION_MENTAL;
		case MS_LIFE:   return AFF_PROTECTION_LIFE;
		case MS_DEATH:  return AFF_PROTECTION_DEATH;
		case MS_FIRE:   return AFF_PROTECTION_FIRE;
		case MS_WATER:  return AFF_PROTECTION_WATER;
		case MS_AIR:    return AFF_PROTECTION_AIR;
		case MS_EARTH:  return AFF_PROTECTION_EARTH;
	}
	return AFF_PROTECTION_ASTRAL;
}

//     
_KERNEL ResistID ProtectToResist( AffectID id );

//    
_KERNEL int ToQuestTarget( int oType );

// IP   
_KERNEL char* IPAsStr( IpAddr ip );

//    
_KERNEL char* PosAsStr( byte pos );

// Exp   
_KERNEL char* ExpAsStr( double exp );

//     
_KERNEL const char* GroupNameColor( int cur, int full );

//    
_KERNEL const char* PerColor( int cur, int full );

//     
_KERNEL const char* PerBlock( int cur, int full );

//   
_KERNEL void ParseSocial( const char* src, char* dst, int& form1, int& form2 );

//  "-"
inline const char* someone( int i = 0 )
{
	static char* someoneNames[] = { "-", "-", "-", "-", "-", "-" };
	return someoneNames[i];
}

//  "-"
inline const char* something( int i = 0 )
{
	static char* somethingNames[] = { "-", "-", "-", "-", "-", "-" };
	return somethingNames[i];
}

//        
inline char* StuffRestrictTypeName( StuffRestrictType type )
{
	char* typeNames[] = { "", "", "", "" };
	return typeNames[type];
}

//       
inline char* MaxExpendablesStr( int max )
{
	static char buf[0x10];
	if( max == 999 ) sprintf( buf, "" );
	else sprintf( buf, "%d", max );
	return buf;
}

//   
_KERNEL const char* DamageForceStr( int damage );

//   
_KERNEL const char* weaponFlagName( int flag );
_KERNEL int weaponNameFlags( const char* name );

//   
_KERNEL int DamageForceRange( int damage, bool min );

//   
_KERNEL const char* SkillColor( int skill );

//   
_KERNEL const char* InventColor( int invent );

//    
_KERNEL bool inArray( int array[], int id );

//   IP
_KERNEL IpAddr parseIPText( const char* ipText );

//  IP  
_KERNEL bool ipCompare( IpAddr mask, IpAddr ip );

//    
_KERNEL const char* MagicSphereColor( MagicSphere ms );

//   
_KERNEL const char* SpecResName( SpecialResist sr );

//    
_KERNEL char* FractionRank( FractionMember member );

//  
_KERNEL int ThrowDice( int d1, int d2 );

//   
static long holdrand = 1L;
_KERNEL void __cdecl _srand( unsigned int seed );
_KERNEL int __cdecl _rand( const char* callContext = "" );
_KERNEL int __cdecl _rand1();

//   
_KERNEL int Round( double num );
_KERNEL uint uRound( double num );

// 
_KERNEL int Min( int a, int b, int c );
//  
_KERNEL int GetMin( int a, int b );
//  
_KERNEL int GetMax( int a, int b );
//  
_KERNEL double GetMin( double a, double b );
//  
_KERNEL double GetMax( double a, double b );
//  
_KERNEL double GetMin( double a, int b );

//  
_KERNEL void ArrayInit( byte** array, const byte seq[] );

//   
#define srand _srand
//#define rand _rand1
#define rand _rand

//   qsort
extern "C" void __cdecl _qsort( void* base, unsigned num, unsigned width, int (__cdecl *comp)(const void *, const void *) );
#define qsort _qsort

//  
template <class T>
void ShuffleArray( T* array, int num )
{
	//   
	if( num <= 1 )
		return;

	//   
	int count = num + rand("Tools6")%num;

	//  
	int i; for( i = 0; i < count; i++ )
	{
		//   
		int idx1 = rand("Tools7")%num;
		int idx2 = rand("Tools8")%num;

		//  
		T tmp = array[idx1];
		array[idx1] = array[idx2];
		array[idx2] = tmp;
	}
}

//  
_KERNEL void KLog( const char *tFileName, const char *tString, ... );

//  
_KERNEL inline void SLog( int logId, const char *text, ... );

//  
_KERNEL inline void BLog( int id );

//  
_KERNEL inline void BLog( byte id );

//  
_KERNEL inline void BLog( const char *text, ... );

//   
#define returnb( b, r ) do { BLog( (byte)b ); return r; } while( 0 )
#define returnv( b ) do { BLog( (byte)b ); return; } while( 0 )
#define returnx( b ) do { BLog( (byte)b ); BLog( 22588 ); return; } while( 0 )


//     
class FLog
{

public: //   

    _KERNEL FLog( int logN, const char* func );
    _KERNEL ~FLog();

public: // 

    int         m_logN; //   
    const char* m_func; //  
    
};


//    
#define FDebug( logN ) FLog flog( (logN), __FUNCTION__ )
#define FSDebug FLog flog( 10, __FUNCTION__ )


// 
_KERNEL void CompressFile( char *tFrom, char *tTo );

// 
_KERNEL void DecompressFile( char *tFrom, char *tTo );

//  
_KERNEL void InstFileName( char* to, CTriggerFileInfo* info, int idx );

//  
_KERNEL RoomExit ViceVersa( int re );

//  
_KERNEL void InitLog();

//  
_KERNEL void debugLog( LogType lType, const char *args, ... );

//     
_KERNEL bool checkCmd( const char *text, const char *cmd, int len );

//     
_KERNEL bool CheckCommands( const char* cmd, const char** opts );

//  
_KERNEL const char* accountTypeStr( byte accType );

//   
_KERNEL char* shortNumber( uint num );
_KERNEL char* shortNumber( double num );

//   
_KERNEL char* charDebugInfo( CCharPlayer* cp );

//   
_KERNEL char* connectionDebugInfo( CConnection* conn );

//    
class CParamStr
{

public: // 

	CParamStr() :
		m_type( PST_ACTION ), m_str( "" )
	{}

	CParamStr( const char* str, ParamStrType type = PST_ACTION ) :
		m_type( type ), m_str( str )
	{}

public: // 

	_KERNEL void Serialize( CFileStore& fs ); //   

	_KERNEL void Perform( CChar* c1, CChar* c2 ); //  

public: // 

	byte m_type;   //  
	CString m_str; // 

};

//    
_KERNEL char* MsgStr( char* buffer, const char* src, CChar* c1, CChar* c2, bool actStr = false, CChar* seer = NULL );

//    
_KERNEL void MsgParamStr( const char* src, char** dest, CChar* c1, CChar* c2, CChar* seer );

//   
class CRandomValueNode
{

public: // 

	CRandomValueNode( double num = 0 ) :
		m_min( num ), m_max( num ), m_weight( 1 )
	{}

public: // 

	void Serialize( CFileStore& fs ); //   

public: // 

	double m_min; // 
	double m_max; // 

	int m_weight; // 

};

//  
class CRandomValue
{

public: // 

	CRandomValue( double num = 0 );

public: // 

	void Serialize( CFileStore& fs ); //   

	void Init( double num = 0 ); // 

	double GenerateValue(); //   

	double GetValue(); //   

public: // 

	PArray<CRandomValueNode*> m_nodes; // 
	int m_div;                         // 

};

class CIntPair
{

public: // 

	CIntPair( int val1 = 0, int val2 = 0 ) :
		m_val1( val1 ), m_val2( val2 )
	{}

public: // 

	//  
	void Set( int val1, int val2 );

public: // 

	int m_val1; //  1
	int m_val2; //  2

};

//  
class CRandomList
{

public: // 

	CRandomList() :
		m_min( 0 ), m_max( 0 )
	{}

public: // 

	void Serialize( CFileStore& fs ); //   

	PArray<int>* MakeRandomList();    //   

public: // 

	PArray<int> m_weights;
	int m_min;
	int m_max;

};

//   
_KERNEL bool CheckCharCondition( CChar *cl, PkFlag pkFlag, ActionType act1 = ACT_NONE, ActionType act2 = ACT_NONE, ActionType act3 = ACT_NONE, ActionType act4 = ACT_NONE, ActionType act5 = ACT_NONE );

//   
class CheckZoneInfo
{

public: //   

	_KERNEL CheckZoneInfo();

	_KERNEL CheckZoneInfo( int charLimit, int statLimit, int dropRoomId, bool zoneType, int id1 = 0, int id2 = 0, int id3 = 0, int id4 = 0, int id5 = 0, int id6 = 0, int id7 = 0, int id8 = 0, int id9 = 0, int id10 = 0 );

	_KERNEL ~CheckZoneInfo();

public: // 

	// 
	_KERNEL void Init();

public: // 

	int m_charLimit;        //    
	int m_statLimit;        //    

	int m_dropRoomId;       //   
	CRoom* m_dropRoom;      //  

	bool m_zoneType;        //  
	int m_zoneId;           //   
	CZone* m_zone;          //  

	PArray<int> m_roomIds;  //   
	PArray<CRoom*> m_rooms; //  

};

//     
_KERNEL void RemoveChar( CChar* cl, CRoom* dropRoom );

//   
_KERNEL void CheckRemoveZone( CheckZoneInfo* info, int instIdx = 0 );

//  -
inline PkFlag inc( PkFlag flag )
{
	if( flag == APF_NONE ) return APF_NOPK;
	else if( flag == APF_NOPK ) return APF_PK;
	return APF_COUNT;
}

//    -
inline PkFlag& operator &=( PkFlag& pkFlag, int val )
	{ return ( pkFlag = (PkFlag)(pkFlag & val) ); }
inline PkFlag& operator |=( PkFlag& pkFlag, int val )
	{ return ( pkFlag = (PkFlag)(pkFlag | val) ); }
inline PkFlag operator |( const PkFlag& pkFlag1, const PkFlag& pkFlag2 )
	{ return (PkFlag)( (int)pkFlag1 | (int)pkFlag2 ); }
inline PkFlag operator &( const PkFlag& pkFlag1, const PkFlag& pkFlag2 )
	{ return (PkFlag)( (int)pkFlag1 & (int)pkFlag2 ); }

//   
class CCharPulseUnit
{

public: // 

	CCharPulseUnit( CChar* ch = NULL, uint pulse = 0 ) :
		m_char( ch ), m_pulse( pulse )
	{}

public: // 

	CChar* m_char;
	uint m_pulse;

};

//  
class CMobUnit
{

public: // 

	_KERNEL CMobUnit( int roomId, int mobId, int count = 1 ) :
		m_roomId( roomId ), m_mobId( mobId ), m_count( count ), m_mob( NULL ), m_room( NULL )
	{}

public: // 
	
	// 
	_KERNEL bool Init( CZoneInstance* zoneInst = NULL );

public: // 

	int m_roomId;     //  
	int m_mobId;      //  
	int m_count;      //  

	bool m_init;      // 

	CCharMob* m_mob;  // 
	CRoom* m_room;    // 

};

//  
class CRoomUnit
{

public: // 

	_KERNEL CRoomUnit( int roomId ) :
		m_roomId( roomId ), m_room( NULL )
	{}

public: // 

	// 
	_KERNEL void Init( CZoneInstance* zoneInst = NULL );

public: // 

	int m_roomId;  // 
	CRoom* m_room; // 

};


//   
struct CMobRepopInfo
{
    int m_mobId; //  
    int m_count; //  
};


//  
class CCharRestrict
{

public: // 

	_KERNEL CCharRestrict( int level = -1, int prof = -1, int race = -1 ) :
		m_level( level ), m_prof( prof ), m_race( race )
	{}

	_KERNEL ~CCharRestrict() {}

public: // 

	//  
	_KERNEL void Reset();

	//  
	_KERNEL void SetRestrict( const char* src, CCharPlayer* cp );

	//   
	_KERNEL bool Accept( CSubject* obj );

public: // 

	int m_level; // 
	int m_prof;  //  
	int m_race;  //  

};

//       
_KERNEL void MightTransfer( CCharPlayer* charFrom, CCharPlayer* charTo, int maxSumm, int* setMightNum, int* setMightSumm );
//    
_KERNEL void InventTransfer( CCharPlayer* charFrom, CCharPlayer* charTo, int* setRebirthNum, int* setInventNum, int* setInventSumm );

//   
class CAbilityInfo
{

public: // 

	CAbilityInfo( byte type = BT_WEAPON ) :
		m_type( type )
	{}

public: // 

	//  
	_KERNEL void SetInfo( byte type, void* ability );

public: // 

	byte m_type;   //  
	union
	{
		CWeaponSkill* m_weapon; //  
		CSpell*       m_spell;  // 
		CSkill*       m_skill;  // 
	} m_ability;

};

//      
class CCalcLogDecreaseNode
{
public:

    // 
    CCalcLogDecreaseNode( int nMax, int nBonus );

    int m_max;    //  
    int m_bonus;  //  
    int m_limit;  //   
    int* m_table; // 
};

//    
class CCalcLogDecrease
{

public:

    //  
    static void InitTables();
    //  
    static void Init( int nMax, int nBonus );

    //  
    static CCalcLogDecreaseNode* FindTable( int nMax, int nBonus );

    //   
    static int Calc( int nValue, int nMax, int nBonus, int nBase = 0 );
    //  
    static int CalcInternal( int nValue, int nMax, int nBonus, int nBase = 0 );

    // 
    static PArray<CCalcLogDecreaseNode*> m_calcTable;
};


//  
class CTime
{
public:

    // 
    _KERNEL CTime( int nYear = 0, int nMonth = 0, int nDay = 0, int nHour = 0, int nMinute = 0, int nSecond = 0 );

    // 
    _KERNEL operator SYSTEMTIME&() { return m_time; }

    // 
    _KERNEL bool operator>( SYSTEMTIME& other );
    _KERNEL bool operator<( SYSTEMTIME& other );
    _KERNEL bool operator<=( SYSTEMTIME& other ) { return !operator>( other ); }
    _KERNEL bool operator>=( SYSTEMTIME& other ) { return !operator<( other ); }
    _KERNEL bool operator==( SYSTEMTIME& other );
    _KERNEL bool operator!=( SYSTEMTIME& other ) { return !operator==( other ); }

    // 
    _KERNEL CTime& operator = ( const SYSTEMTIME& other );

    // 
    _KERNEL void Serialize( CFileStore& fs );
    _KERNEL void Serialize( CByteStore& bs );

    // 
    _KERNEL void Reset();

    // 
    _KERNEL void Init( int nYear = 0, int nMonth = 0, int nDay = 0, int nHour = 0, int nMinute = 0, int nSecond = 0 );

    // 
    _KERNEL bool isSet();

    //  
    _KERNEL CTime& SetCurrentTime( int nTimePrecise = TP_NONE );

    //        04:00
    _KERNEL CTime& SetCurrentTaskDay();

    //   
    _KERNEL CTime& SetCurrentWeek();

    //   
    _KERNEL CTime& AddTime( int nNum, byte nPrecise, TimePrecise roundFlag = TP_NONE );

    //  
    _KERNEL int Difference( SYSTEMTIME& other );

    //    
    _KERNEL int DifferencePulses( SYSTEMTIME& other );

    //     
    _KERNEL int GetDayOfWeek();

    //    
    _KERNEL const char* GetTimeText( byte nPrecise = TP_MIN );

    //  
    static _KERNEL ULONGLONG SystemToULong( SYSTEMTIME& time );
    static _KERNEL void ULongToSystem( ULONGLONG ulongTime, SYSTEMTIME& sysTime );

    SYSTEMTIME m_time;

};


// 
class CInterval
{
public:

    _KERNEL CInterval( uint min = 0, uint max = 0 ) :
        m_min( min ),
        m_max( max )
    {}

	_KERNEL void Serialize( CFileStore& fs ); //   

public:

    uint m_min; // 
    uint m_max; // 
};


//  
class CWeekSchedule
{

public:

    // 
    CWeekSchedule();

    //  
    CWeekSchedule( const CWeekSchedule& other );

    // 
    _KERNEL void operator=( const CWeekSchedule& other );

    // 
    _KERNEL bool operator==( CWeekSchedule& other );

    //   
	_KERNEL void Serialize( CFileStore& fs );

    //   
    _KERNEL void PrintToBuffer( char* buffer );

    //   
    _KERNEL void IntervalsToHours( int nDay, byte* usedTime );

    //   
    _KERNEL void HoursToIntervals( int nDay, byte* usedTime );

    //   
    _KERNEL void AddInterval( int nDay, CInterval& interval );

    //   
    _KERNEL void RemoveInterval( int nDay, CInterval& interval );

    //   
    _KERNEL uint CheckMinHours( int nMinHoursDay, int nMinHoursWeek );

    //  
    _KERNEL uint RandomHour( int nDay );

    //      
    _KERNEL bool IsInSchedule( CTime& time );

public:

    PArray<CInterval*> m_days[WEEK_DAYS]; //  
};

//  
class CSelfDeleting
{

public: // 
    virtual void Delete() = 0;

};

//    
class BBLog
{

public: //   

    _KERNEL BBLog( uint id );
    _KERNEL ~BBLog();

public: // 

    uint m_id; // 

};

//  
_KERNEL char* FormatNumber( char* buffer, double num, char thousandSign = ' ' );

//    
_KERNEL int skipColorStr( const char* str );

//  
_KERNEL byte LetterType( char l );

//  
_KERNEL char RandomRussianLetter( char prev );

//  
_KERNEL void RandomString( char* buffer, int len );

//   
_KERNEL const char* AbilityTypeName( int type, int cas );

//  
_KERNEL const char* AbilityName( int type, int id );

//   
_KERNEL int GloryLevel( byte level );

//   
_KERNEL const char* GloryLevelName( int level, int caseType = CT_IMEN );

//    
_KERNEL bool GloryLevelReady( byte level );

//    
_KERNEL const char* GloryLevelReadyStr( byte level );

//  
_KERNEL bool IsMagicResist( int resId );

//   
_KERNEL int ResistToSphere( int resId );

//   
_KERNEL int SphereToResist( int sphere );

//    
_KERNEL int minExpLevel( int maxLvl );

//  
_KERNEL bool IsAttackAbil( int abilType, int abilId );

//   
_KERNEL bool IsCriticalSpecRes( int resId );

//   
_KERNEL bool CheckPerc( int chance, const char* mark = "", bool anyway = false );

//    
_KERNEL void ChooseNonRepeat( int* setArray, int setSize, int* chooseArray, int chooseSize );
_KERNEL void ChooseNonRepeat( int max, int* chooseArray, int chooseSize );

// :     
_KERNEL int SourceToSphere( int source );

// :     
_KERNEL int SourceToArmor( int source );

//   
_KERNEL bool SourceIsMagic( int source );

//  
_KERNEL int ResistMax( int type );

//  
_KERNEL int ZoneId( int id );

//    
_KERNEL double TrigValD( double val, int type, CZoneInstance* inst );

//    
_KERNEL int TrigVal( double val, int type, CZoneInstance* inst );

//    
_KERNEL void CalcHeraldTime( int days, SYSTEMTIME* pSystemTime );

//    
_KERNEL void ConvertPulsesToSystemTime( uint pulses, SYSTEMTIME& sysTime );

//    
_KERNEL void ConvertSystemTimeToPulses( SYSTEMTIME& sysTime, uint& pulses );

//     
_KERNEL bool IsItemPresent( CSubject* obj );

//   
_KERNEL const char* TextWordChar( int nCase, int nType );

//     
template <typename T>
uint max5( T* vals )
{
    uint c1 = vals[0] >= vals[1] ? 0 : 1;
    uint c2 = vals[2] >= vals[3] ? 2 : 3;
    uint c3 = 4;

    if( vals[c1] >= vals[c2] && vals[c1] >= vals[c3] )
        return c1;
    else if( vals[c2] >= vals[c1] && vals[c2] >= vals[c3] )
        return c2;
    else if( vals[c3] >= vals[c1] && vals[c3] >= vals[c2] )
        return c3;

    return 0;
}

template _KERNEL uint max5( uint* vals );
